// Note: For maximum-speed code, see "Optimizing Code" on the Emscripten wiki, https://github.com/kripken/emscripten/wiki/Optimizing-Code
// Note: Some Emscripten settings may limit the speed of the generated code.
// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = eval('(function() { try { return Module || {} } catch(e) { return {} } })()');

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  Module['arguments'] = process['argv'].slice(2);

  module['exports'] = Module;
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    return read(f, 'binary');
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  this['Module'] = Module;

  eval("if (typeof gc === 'function' && gc.toString().indexOf('[native code]') > 0) var gc = undefined"); // wipe out the SpiderMonkey shell 'gc' function, which can confuse closure (uses it as a minified name, and it is then initted to a non-falsey value unexpectedly)
}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WEB) {
    this['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] == 'undefined' && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Auto-generated preamble library stuff ===

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (quantum == 1) return target;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else if (isNumber(quantum) && isPowerOfTwo(quantum)) {
      return '(((' +target + ')+' + (quantum-1) + ')&' + -quantum + ')';
    }
    return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return type[type.length-1] == '*';
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (isArrayType(type)) return true;
  if (/<?{ ?[^}]* ?}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  STACK_ALIGN: 8,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (vararg) return 8;
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    var index = 0;
    type.flatIndexes = type.fields.map(function(field) {
      index++;
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = Runtime.getAlignSize(field, size);
      } else if (Runtime.isStructType(field)) {
        if (field[1] === '0') {
          // this is [0 x something]. When inside another structure like here, it must be at the end,
          // and it adds no size
          // XXX this happens in java-nbody for example... assert(index === type.fields.length, 'zero-length in the middle!');
          size = 0;
          if (Types.types[field]) {
            alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
          } else {
            alignSize = type.alignSize || QUANTUM_SIZE;
          }
        } else {
          size = Types.types[field].flatSize;
          alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
        }
      } else if (field[0] == 'b') {
        // bN, large number field, like a [N x i8]
        size = field.substr(1)|0;
        alignSize = 1;
      } else if (field[0] === '<') {
        // vector type
        size = alignSize = Types.types[field].flatSize; // fully aligned
      } else if (field[0] === 'i') {
        // illegal integer field, that could not be legalized because it is an internal structure field
        // it is ok to have such fields, if we just use them as markers of field size and nothing more complex
        size = alignSize = parseInt(field.substr(1))/8;
        assert(size % 1 === 0, 'cannot handle non-byte-size field ' + field);
      } else {
        assert(false, 'invalid type for calculateStructAlignment');
      }
      if (type.packed) alignSize = 1;
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    if (type.name_ && type.name_[0] === '[') {
      // arrays have 2 elements, so we get the proper difference. then we scale here. that way we avoid
      // allocating a potentially huge array for [999999 x i8] etc.
      type.flatSize = parseInt(type.name_.substr(1))*type.flatSize/2;
    }
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (type.fields.length != struct.length) {
        printErr('Number of named fields must match the type for ' + typeName + ': possibly duplicate struct names. Cannot return structInfo');
        return null;
      }
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    code = Pointer_stringify(code);
    if (code[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (code.indexOf('"', 1) === code.length-1) {
        code = code.substr(1, code.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + code + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    return Runtime.asmConstCache[code] = eval('(function(' + args.join(',') + '){ ' + code + ' })'); // new Function does not allow upvars in node
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[func]) {
      Runtime.funcWrappers[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return Runtime.funcWrappers[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;

      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }

      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }

      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          Math.floor((codePoint - 0x10000) / 0x400) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function processJSString(string) {
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+7)&-8); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + size)|0;STATICTOP = (((STATICTOP)+7)&-8); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + size)|0;DYNAMICTOP = (((DYNAMICTOP)+7)&-8); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 8))*(quantum ? quantum : 8); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*(+4294967296))) : ((+((low>>>0)))+((+((high|0)))*(+4294967296)))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// C calling interface. A convenient way to call C functions (in C files, or
// defined with extern "C").
//
// Note: LLVM optimizations can inline and remove functions, after which you will not be
//       able to call them. Closure can also do so. To avoid that, add your function to
//       the exports using something like
//
//         -s EXPORTED_FUNCTIONS='["_main", "_myfunc"]'
//
// @param ident      The name of the C function (note that C++ functions will be name-mangled - use extern "C")
// @param returnType The return type of the function, one of the JS types 'number', 'string' or 'array' (use 'number' for any C pointer, and
//                   'array' for JavaScript arrays and typed arrays; note that arrays are 8-bit).
// @param argTypes   An array of the types of arguments for the function (if there are no arguments, this can be ommitted). Types are as in returnType,
//                   except that 'array' is not possible (there is no way for us to know the length of the array)
// @param args       An array of the arguments to the function, as native JS values (as in returnType)
//                   Note that string arguments will be stored on the stack (the JS string will become a C string on the stack).
// @return           The return value, as a native JS value (as in returnType)
function ccall(ident, returnType, argTypes, args) {
  return ccallFunc(getCFunc(ident), returnType, argTypes, args);
}
Module["ccall"] = ccall;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  try {
    var func = Module['_' + ident]; // closure exported function
    if (!func) func = eval('_' + ident); // explicit lookup
  } catch(e) {
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

// Internal function that does a C call using a function, not an identifier
function ccallFunc(func, returnType, argTypes, args) {
  var stack = 0;
  function toC(value, type) {
    if (type == 'string') {
      if (value === null || value === undefined || value === 0) return 0; // null string
      value = intArrayFromString(value);
      type = 'array';
    }
    if (type == 'array') {
      if (!stack) stack = Runtime.stackSave();
      var ret = Runtime.stackAlloc(value.length);
      writeArrayToMemory(value, ret);
      return ret;
    }
    return value;
  }
  function fromC(value, type) {
    if (type == 'string') {
      return Pointer_stringify(value);
    }
    assert(type != 'array');
    return value;
  }
  var i = 0;
  var cArgs = args ? args.map(function(arg) {
    return toC(arg, argTypes[i++]);
  }) : [];
  var ret = fromC(func.apply(null, cArgs), returnType);
  if (stack) Runtime.stackRestore(stack);
  return ret;
}

// Returns a native JS wrapper for a C function. This is similar to ccall, but
// returns a function you can call repeatedly in a normal way. For example:
//
//   var my_function = cwrap('my_c_function', 'number', ['number', 'number']);
//   alert(my_function(5, 22));
//   alert(my_function(99, 12));
//
function cwrap(ident, returnType, argTypes) {
  var func = getCFunc(ident);
  return function() {
    return ccallFunc(func, returnType, argTypes, Array.prototype.slice.call(arguments));
  }
}
Module["cwrap"] = cwrap;

// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.
// Note that setValue and getValue only do *aligned* writes and reads!
// Note that ccall uses JS types as for defining types, while setValue and
// getValue need LLVM types ('i8', 'i32') - this is a lower-level operation
function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[(ptr)]=value; break;
      case 'i8': HEAP8[(ptr)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= (+1) ? (tempDouble > (+0) ? ((Math_min((+(Math_floor((tempDouble)/(+4294967296)))), (+4294967295)))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/(+4294967296))))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;

// Parallel to setValue.
function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[(ptr)];
      case 'i8': return HEAP8[(ptr)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)|0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    t = HEAPU8[(((ptr)+(i))|0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }

  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    t = HEAPU8[(((ptr)+(i))|0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16LE form. The copy will require at most (str.length*2+1)*2 bytes of space in the HEAP.
function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0;
}
Module['stringToUTF16'] = stringToUTF16;

// Given a pointer 'ptr' to a null-terminated UTF32LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32LE form. The copy will require at most (str.length+1)*4 bytes of space in the HEAP,
// but can use less, since str.length does not return the number of characters in the string, but the number of UTF-16 code units in the string.
function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit;
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0;
}
Module['stringToUTF32'] = stringToUTF32;

function demangle(func) {
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    var i = 3;
    // params, etc.
    var basicTypes = {
      'v': 'void',
      'b': 'bool',
      'c': 'char',
      's': 'short',
      'i': 'int',
      'l': 'long',
      'f': 'float',
      'd': 'double',
      'w': 'wchar_t',
      'a': 'signed char',
      'h': 'unsigned char',
      't': 'unsigned short',
      'j': 'unsigned int',
      'm': 'unsigned long',
      'x': 'long long',
      'y': 'unsigned long long',
      'z': '...'
    };
    function dump(x) {
      //return;
      if (x) Module.print(x);
      Module.print(func);
      var pre = '';
      for (var a = 0; a < i; a++) pre += ' ';
      Module.print (pre + '^');
    }
    var subs = [];
    function parseNested() {
      i++;
      if (func[i] === 'K') i++; // ignore const
      var parts = [];
      while (func[i] !== 'E') {
        if (func[i] === 'S') { // substitution
          i++;
          var next = func.indexOf('_', i);
          var num = func.substring(i, next) || 0;
          parts.push(subs[num] || '?');
          i = next+1;
          continue;
        }
        if (func[i] === 'C') { // constructor
          parts.push(parts[parts.length-1]);
          i += 2;
          continue;
        }
        var size = parseInt(func.substr(i));
        var pre = size.toString().length;
        if (!size || !pre) { i--; break; } // counter i++ below us
        var curr = func.substr(i + pre, size);
        parts.push(curr);
        subs.push(curr);
        i += pre + size;
      }
      i++; // skip E
      return parts;
    }
    var first = true;
    function parse(rawList, limit, allowVoid) { // main parser
      limit = limit || Infinity;
      var ret = '', list = [];
      function flushList() {
        return '(' + list.join(', ') + ')';
      }
      var name;
      if (func[i] === 'N') {
        // namespaced N-E
        name = parseNested().join('::');
        limit--;
        if (limit === 0) return rawList ? [name] : name;
      } else {
        // not namespaced
        if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
        var size = parseInt(func.substr(i));
        if (size) {
          var pre = size.toString().length;
          name = func.substr(i + pre, size);
          i += pre + size;
        }
      }
      first = false;
      if (func[i] === 'I') {
        i++;
        var iList = parse(true);
        var iRet = parse(true, 1, true);
        ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
      } else {
        ret = name;
      }
      paramLoop: while (i < func.length && limit-- > 0) {
        //dump('paramLoop');
        var c = func[i++];
        if (c in basicTypes) {
          list.push(basicTypes[c]);
        } else {
          switch (c) {
            case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
            case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
            case 'L': { // literal
              i++; // skip basic type
              var end = func.indexOf('E', i);
              var size = end - i;
              list.push(func.substr(i, size));
              i += size + 2; // size + 'EE'
              break;
            }
            case 'A': { // array
              var size = parseInt(func.substr(i));
              i += size.toString().length;
              if (func[i] !== '_') throw '?';
              i++; // skip _
              list.push(parse(true, 1, true)[0] + ' [' + size + ']');
              break;
            }
            case 'E': break paramLoop;
            default: ret += '?' + c; break paramLoop;
          }
        }
      }
      if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
      return rawList ? list : ret + flushList();
    }
    return parse();
  } catch(e) {
    return func;
  }
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function stackTrace() {
  var stack = new Error().stack;
  return stack ? demangleAll(stack) : '(no stack trace available)'; // Stack trace is not available at least on IE10 and Safari 6.
}

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays in asm.js. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', or (2) set Module.TOTAL_MEMORY before the program runs.');
}

var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 4096;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be more reasonable');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'Cannot fallback to non-typed array case: Code is too specialized');

var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;

function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools

// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

// Write a Javascript array to somewhere in the heap
function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))|0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))|0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    HEAP8[(((buffer)+(i))|0)]=str.charCodeAt(i);
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))|0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore, sig) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore, sig) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// === Body ===



STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 6160;


/* global initializers */ __ATINIT__.push({ func: function() { runPostSets() } });



var _stdout;
var _stdout=_stdout=allocate([0,0,0,0,0,0,0,0], "i8", ALLOC_STATIC);;















































































































































































































































































































































































































































































































































































































/* memory initializer */ allocate([82,105,103,108,111,108,32,49,46,48,51,100,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,160,7,0,0,152,7,0,0,144,7,0,0,136,7,0,0,120,7,0,0,104,7,0,0,96,7,0,0,56,7,0,0,48,7,0,0,40,7,0,0,32,7,0,0,24,7,0,0,16,7,0,0,8,7,0,0,248,6,0,0,232,6,0,0,216,6,0,0,168,6,0,0,152,6,0,0,144,6,0,0,136,6,0,0,128,6,0,0,120,6,0,0,104,6,0,0,96,6,0,0,80,6,0,0,72,6,0,0,32,6,0,0,16,6,0,0,8,6,0,0,0,6,0,0,248,5,0,0,240,5,0,0,224,5,0,0,216,5,0,0,208,5,0,0,200,5,0,0,168,5,0,0,160,5,0,0,152,5,0,0,144,5,0,0,136,5,0,0,128,5,0,0,120,5,0,0,56,22,0,0,112,5,0,0,104,5,0,0,72,5,0,0,64,5,0,0,56,5,0,0,48,5,0,0,40,5,0,0,32,5,0,0,24,5,0,0,16,5,0,0,8,5,0,0,0,5,0,0,216,4,0,0,208,4,0,0,200,4,0,0,192,4,0,0,184,4,0,0,176,4,0,0,168,4,0,0,160,4,0,0,152,4,0,0,144,4,0,0,144,21,0,0,136,21,0,0,128,21,0,0,120,21,0,0,112,21,0,0,96,21,0,0,80,21,0,0,72,21,0,0,64,21,0,0,56,21,0,0,16,21,0,0,8,21,0,0,0,21,0,0,240,20,0,0,232,20,0,0,224,20,0,0,208,20,0,0,192,20,0,0,176,20,0,0,160,20,0,0,104,20,0,0,96,20,0,0,88,20,0,0,72,20,0,0,56,20,0,0,40,20,0,0,24,20,0,0,8,20,0,0,248,19,0,0,232,19,0,0,184,19,0,0,168,19,0,0,152,19,0,0,136,19,0,0,128,19,0,0,112,19,0,0,96,19,0,0,80,19,0,0,64,19,0,0,48,19,0,0,240,18,0,0,232,18,0,0,216,18,0,0,200,18,0,0,176,18,0,0,160,18,0,0,152,18,0,0,136,18,0,0,120,18,0,0,104,18,0,0,72,18,0,0,48,18,0,0,56,22,0,0,32,18,0,0,16,18,0,0,248,17,0,0,232,17,0,0,56,22,0,0,216,17,0,0,200,17,0,0,184,17,0,0,168,17,0,0,128,17,0,0,112,17,0,0,96,17,0,0,72,17,0,0,56,17,0,0,40,17,0,0,24,17,0,0,8,17,0,0,248,16,0,0,232,16,0,0,184,16,0,0,168,16,0,0,152,16,0,0,128,16,0,0,112,16,0,0,88,16,0,0,64,16,0,0,48,16,0,0,32,16,0,0,24,16,0,0,232,15,0,0,224,15,0,0,208,15,0,0,192,15,0,0,168,15,0,0,144,15,0,0,56,22,0,0,128,15,0,0,112,15,0,0,96,15,0,0,80,15,0,0,40,15,0,0,24,15,0,0,8,15,0,0,248,14,0,0,232,14,0,0,216,14,0,0,200,14,0,0,184,14,0,0,168,14,0,0,152,14,0,0,56,14,0,0,40,14,0,0,24,14,0,0,8,14,0,0,248,13,0,0,232,13,0,0,216,13,0,0,200,13,0,0,184,13,0,0,168,13,0,0,128,13,0,0,112,13,0,0,96,13,0,0,80,13,0,0,64,13,0,0,48,13,0,0,32,13,0,0,16,13,0,0,0,13,0,0,240,12,0,0,192,12,0,0,184,12,0,0,160,12,0,0,144,12,0,0,128,12,0,0,112,12,0,0,96,12,0,0,80,12,0,0,64,12,0,0,48,12,0,0,8,12,0,0,248,11,0,0,232,11,0,0,224,11,0,0,208,11,0,0,192,11,0,0,176,11,0,0,160,11,0,0,144,11,0,0,128,11,0,0,72,11,0,0,56,11,0,0,40,11,0,0,32,11,0,0,8,11,0,0,248,10,0,0,232,10,0,0,216,10,0,0,200,10,0,0,176,10,0,0,104,10,0,0,88,10,0,0,64,10,0,0,48,10,0,0,32,10,0,0,16,10,0,0,0,10,0,0,240,9,0,0,224,9,0,0,208,9,0,0,152,9,0,0,136,9,0,0,120,9,0,0,104,9,0,0,88,9,0,0,72,9,0,0,56,9,0,0,40,9,0,0,24,9,0,0,16,9,0,0,208,8,0,0,192,8,0,0,176,8,0,0,160,8,0,0,65,66,67,68,69,70,71,72,74,75,76,77,78,80,81,82,83,84,85,86,87,88,89,90,50,51,52,53,54,55,56,57,65,69,66,70,57,52,68,53,67,54,65,65,55,49,0,0,65,69,66,70,57,52,67,69,69,51,69,55,48,55,0,0,50,56,66,69,55,70,65,70,68,50,65,48,53,50,0,0,55,65,51,69,56,48,56,53,57,57,65,53,50,53,0,0,51,52,48,56,0,0,0,0,50,57,56,50,0,0,0,0,50,51,52,53,54,55,56,57,65,83,68,70,71,72,74,75,76,80,85,89,84,82,69,87,81,77,78,66,86,67,88,90,0,0,0,0,0,0,0,0,102,97,115,105,110,104,0,0,102,115,105,110,104,0,0,0,102,97,116,97,110,104,0,0,102,116,97,110,104,0,0,0,102,97,99,111,115,0,0,0,102,99,111,115,0,0,0,0,102,97,115,105,110,0,0,0,102,115,105,110,0,0,0,0,102,97,116,97,110,0,0,0,102,116,97,110,0,0,0,0,73,108,108,101,103,97,108,32,112,97,114,97,109,101,116,101,114,32,117,115,97,103,101,10,0,0,0,0,0,0,0,0,102,112,111,119,102,0,0,0,102,108,111,103,0,0,0,0,102,101,120,112,0,0,0,0,102,112,105,0,0,0,0,0,102,114,111,111,116,0,0,0,102,112,111,119,101,114,0,0,101,120,112,105,110,116,0,0,108,111,103,98,50,0,0,0,98,117,105,108,100,0,0,0,115,102,116,98,105,116,0,0,73,108,108,101,103,97,108,32,110,117,109,98,101,114,32,98,97,115,101,10,0,0,0,0,102,114,97,110,100,0,0,0,102,116,114,117,110,99,0,0,102,105,110,99,114,0,0,0,102,112,109,117,108,0,0,0,102,114,101,99,105,112,0,0,102,99,111,110,118,0,0,0,102,99,111,109,112,0,0,0,102,115,117,98,0,0,0,0,102,97,100,100,0,0,0,0,73,110,112,117,116,32,102,111,114,109,97,116,32,101,114,114,111,114,10,0,0,0,0,0,102,100,105,118,0,0,0,0,102,109,117,108,0,0,0,0,109,114,111,117,110,100,0,0,109,114,95,115,104,105,102,116,0,0,0,0,0,0,0,0,100,99,111,110,118,0,0,0,102,112,97,99,107,0,0,0,120,103,99,100,0,0,0,0,109,105,114,115,121,115,0,0,115,117,98,116,114,97,99,116,0,0,0,0,0,0,0,0,97,100,100,0,0,0,0,0,73,110,116,101,114,110,97,108,32,114,101,115,117,108,116,32,105,115,32,110,101,103,97,116,105,118,101,10,0,0,0,0,112,117,116,100,105,103,0,0,109,117,108,116,105,95,105,110,118,101,114,115,101,0,0,0,109,97,100,0,0,0,0,0,109,105,114,118,97,114,0,0,48,0,0,0,0,0,0,0,105,115,112,114,105,109,101,0,110,120,112,114,105,109,101,0,98,105,103,114,97,110,100,0,98,105,103,100,105,103,0,0,112,111,119,109,111,100,0,0,68,83,65,0,0,0,0,0,112,111,119,101,114,0,0,0,79,118,101,114,102,108,111,119,32,45,32,78,117,109,98,101,114,32,116,111,111,32,98,105,103,10,0,0,0,0,0,0,68,83,52,0,0,0,0,0,110,114,111,111,116,0,0,0,68,83,50,0,0,0,0,0,99,111,116,110,117,109,0,0,68,83,49,0,0,0,0,0,99,105,110,110,117,109,0,0,68,80,56,0,0,0,0,0,99,98,97,115,101,0,0,0,101,103,99,100,0,0,0,0,102,100,115,105,122,101,0,0,115,117,98,100,105,118,0,0,112,114,101,109,117,108,116,0,100,101,99,114,0,0,0,0,105,110,99,114,0,0,0,0,68,105,118,105,115,105,111,110,32,98,121,32,122,101,114,111,32,97,116,116,101,109,112,116,101,100,10,0,0,0,0,0,100,105,118,105,100,101,0,0,109,117,108,116,105,112,108,121,0,0,0,0,0,0,0,0,110,111,114,109,97,108,105,115,101,0,0,0,0,0,0,0,106,97,99,107,0,0,0,0,111,116,110,117,109,0,0,0,105,110,110,117,109,0,0,0,121,111,117,114,32,112,114,111,103,114,97,109,0,0,0,0,85,110,100,101,102,105,110,101,100,32,101,114,114,111,114,10,0,0,0,0,0,0,0,0,73,110,112,117,116,47,111,117,116,112,117,116,32,101,114,114,111,114,32,119,104,101,110,32,114,101,97,100,105,110,103,32,102,114,111,109,32,82,78,71,32,100,101,118,105,99,101,32,110,111,100,101,10,0,0,0,67,111,109,112,111,115,105,116,101,32,109,111,100,117,108,117,115,10,0,0,0,0,0,0,78,117,109,98,101,114,32,98,97,115,101,32,116,111,111,32,98,105,103,32,102,111,114,32,114,101,112,114,101,115,101,110,116,97,116,105,111,110,10,0,78,111,32,105,114,114,101,100,117,99,105,98,108,101,32,98,97,115,105,115,32,100,101,102,105,110,101,100,10,0,0,0,66,97,115,101,32,109,117,115,116,32,98,101,32,98,105,110,97,114,121,32,40,77,82,95,65,76,87,65,89,83,95,66,73,78,65,82,89,32,100,101,102,105,110,101,100,32,105,110,32,109,105,114,100,101,102,46,104,32,63,41,10,0,0,0,101,99,110,50,95,109,117,108,116,52,0,0,0,0,0,0,122,122,110,52,95,102,114,111,109,95,98,105,103,0,0,0,122,122,110,52,95,108,109,117,108,0,0,0,0,0,0,0,122,122,110,52,95,105,109,117,108,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,99,111,110,116,114,111,108,32,70,108,111,97,116,105,110,103,45,112,111,105,110,116,32,114,111,117,110,100,105,110,103,10,0,0,0,0,0,0,122,122,110,52,95,116,120,0,122,122,110,52,95,112,111,119,113,0,0,0,0,0,0,0,122,122,110,52,95,100,105,118,50,0,0,0,0,0,0,0,122,122,110,52,95,105,110,118,0,0,0,0,0,0,0,0,122,122,110,52,95,109,117,108,0,0,0,0,0,0,0,0,122,122,110,52,95,115,113,114,0,0,0,0,0,0,0,0,122,122,110,52,95,115,109,117,108,0,0,0,0,0,0,0,122,122,110,52,95,115,115,117,98,0,0,0,0,0,0,0,122,122,110,52,95,115,117,98,0,0,0,0,0,0,0,0,122,122,110,52,95,115,97,100,100,0,0,0,0,0,0,0,83,112,101,99,105,102,105,101,100,32,98,97,115,105,115,32,105,115,32,78,79,84,32,105,114,114,101,100,117,99,105,98,108,101,10,0,0,0,0,0,122,122,110,52,95,97,100,100,0,0,0,0,0,0,0,0,122,122,110,52,95,99,111,110,106,0,0,0,0,0,0,0,122,122,110,52,95,110,101,103,97,116,101,0,0,0,0,0,122,122,110,52,95,102,114,111,109,95,105,110,116,0,0,0,110,114,101,115,95,99,111,109,112,108,101,120,0,0,0,0,122,122,110,51,95,116,105,109,101,115,105,50,0,0,0,0,101,99,110,50,95,109,117,108,116,110,0,0,0,0,0,0,101,99,110,50,95,109,117,108,95,98,114,105,99,107,95,103,108,115,0,0,0,0,0,0,101,99,110,50,95,98,114,105,99,107,95,105,110,105,116,0,101,99,110,50,95,109,117,108,50,95,103,108,115,0,0,0,83,112,101,99,105,102,105,101,100,32,100,111,117,98,108,101,32,108,101,110,103,116,104,32,116,121,112,101,32,105,115,110,39,116,32,100,111,117,98,108,101,32,108,101,110,103,116,104,10,0,0,0,0,0,0,0,101,99,110,50,95,112,114,101,99,111,109,112,95,103,108,115,0,0,0,0,0,0,0,0,101,99,110,50,95,109,117,108,50,0,0,0,0,0,0,0,101,99,110,50,95,109,117,108,52,95,103,108,115,95,118,0,101,99,110,50,95,112,114,101,99,111,109,112,0,0,0,0,101,99,110,50,95,109,117,108,116,105,95,110,111,114,109,0,122,122,110,50,95,109,117,108,116,105,95,105,110,118,101,114,115,101,0,0,0,0,0,0,105,110,118,109,111,100,112,0,101,99,110,50,95,112,115,105,0,0,0,0,0,0,0,0,101,99,110,50,95,97,100,100,95,115,117,98,0,0,0,0,122,122,110,50,95,115,113,114,0,0,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,70,101,97,116,117,114,101,32,45,32,99,104,101,99,107,32,109,105,114,100,101,102,46,104,10,0,0,0,122,122,110,50,95,100,105,118,53,0,0,0,0,0,0,0,110,114,101,115,95,100,105,118,53,0,0,0,0,0,0,0,101,99,110,50,95,109,117,108,0,0,0,0,0,0,0,0,101,99,110,50,95,109,117,108,50,95,106,115,102,0,0,0,101,99,110,50,95,97,100,100,0,0,0,0,0,0,0,0,122,122,110,50,95,115,113,114,116,0,0,0,0,0,0,0,122,122,110,50,95,113,114,0,101,99,110,50,95,114,104,115,0,0,0,0,0,0,0,0,101,99,110,50,95,115,101,116,120,0,0,0,0,0,0,0,122,122,110,50,95,100,105,118,51,0,0,0,0,0,0,0,69,120,112,111,110,101,110,116,32,116,111,111,32,98,105,103,10,0,0,0,0,0,0,0,110,114,101,115,95,100,105,118,51,0,0,0,0,0,0,0,110,114,101,115,95,100,105,118,50,0,0,0,0,0,0,0,122,122,110,50,95,116,120,100,0,0,0,0,0,0,0,0,122,122,110,50,95,116,120,120,0,0,0,0,0,0,0,0,101,99,110,50,95,115,101,116,0,0,0,0,0,0,0,0,101,99,110,50,95,110,111,114,109,0,0,0,0,0,0,0,101,99,110,50,95,99,111,109,112,97,114,101,0,0,0,0,101,112,111,105,110,116,50,95,109,117,108,116,105,95,110,111,114,109,0,0,0,0,0,0,109,114,95,106,115,102,0,0,101,112,111,105,110,116,95,109,117,108,116,105,95,110,111,114,109,0,0,0,0,0,0,0,78,111,32,109,111,100,117,108,117,115,32,100,101,102,105,110,101,100,10,0,0,0,0,0,122,122,110,51,95,116,105,109,101,115,105,0,0,0,0,0,122,122,110,51,95,100,105,118,50,0,0,0,0,0,0,0,122,122,110,51,95,105,110,118,0,0,0,0,0,0,0,0,122,122,110,51,95,109,117,108,0,0,0,0,0,0,0,0,122,122,110,51,95,105,109,117,108,0,0,0,0,0,0,0,122,122,110,51,95,115,109,117,108,0,0,0,0,0,0,0,122,122,110,51,95,115,115,117,98,0,0,0,0,0,0,0,122,122,110,51,95,115,117,98,0,0,0,0,0,0,0,0,122,122,110,51,95,115,97,100,100,0,0,0,0,0,0,0,122,122,110,51,95,97,100,100,0,0,0,0,0,0,0,0,73,108,108,101,103,97,108,32,109,111,100,117,108,117,115,32,10,0,0,0,0,0,0,0,122,122,110,51,95,105,110,105,116,0,0,0,0,0,0,0,122,122,110,51,95,112,111,119,113,0,0,0,0,0,0,0,122,122,110,51,95,110,101,103,97,116,101,0,0,0,0,0,122,122,110,51,95,102,114,111,109,95,98,105,103,0,0,0,122,122,110,51,95,102,114,111,109,95,98,105,103,115,0,0,122,122,110,51,95,102,114,111,109,95,105,110,116,115,0,0,122,122,110,51,95,102,114,111,109,95,105,110,116,0,0,0,122,122,110,50,95,100,105,118,50,0,0,0,0,0,0,0,122,122,110,50,95,116,105,109,101,115,95,105,114,112,0,0,122,122,110,50,95,115,115,117,98,0,0,0,0,0,0,0,77,73,82,65,67,76,32,110,111,116,32,105,110,105,116,105,97,108,105,115,101,100,32,45,32,110,111,32,99,97,108,108,32,116,111,32,109,105,114,115,121,115,40,41,10,0,0,0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,99,97,108,108,101,100,32,102,114,111,109,32,0,0,0,0,0,0,122,122,110,50,95,115,97,100,100,0,0,0,0,0,0,0,122,122,110,50,95,102,114,111,109,95,105,110,116,115,0,0,122,122,110,50,95,102,114,111,109,95,98,105,103,0,0,0,122,122,110,50,95,102,114,111,109,95,98,105,103,115,0,0,122,122,110,50,95,112,111,119,108,0,0,0,0,0,0,0,122,122,110,50,95,116,105,109,101,115,105,0,0,0,0,0,122,122,110,50,95,105,110,118,0,0,0,0,0,0,0,0,122,122,110,50,95,109,117,108,0,0,0,0,0,0,0,0,122,122,110,50,95,115,109,117,108,0,0,0,0,0,0,0,122,122,110,50,95,115,117,98,0,0,0,0,0,0,0,0,73,47,79,32,98,117,102,102,101,114,32,111,118,101,114,102,108,111,119,10,0,0,0,0,122,122,110,50,95,97,100,100,0,0,0,0,0,0,0,0,122,122,110,50,95,99,111,110,106,0,0,0,0,0,0,0,122,122,110,50,95,110,101,103,97,116,101,0,0,0,0,0,122,122,110,50,95,102,114,111,109,95,105,110,116,0,0,0,110,114,101,115,95,100,111,117,98,108,101,95,109,111,100,115,117,98,0,0,0,0,0,0,110,114,101,115,95,100,111,117,98,108,101,95,109,111,100,97,100,100,0,0,0,0,0,0,122,122,110,50,95,105,109,117,108,0,0,0,0,0,0,0,110,114,101,115,95,108,97,122,121,0,0,0,0,0,0,0,98,105,103,98,105,116,115,0,101,120,112,98,50,0,0,0,70,108,97,115,104,32,116,111,32,100,111,117,98,108,101,32,99,111,110,118,101,114,115,105,111,110,32,102,97,105,108,117,114,101,10,0,0,0,0,0,104,97,109,109,105,110,103,0,101,112,111,105,110,116,95,120,0,0,0,0,0,0,0,0,100,111,117,98,108,101,95,105,110,118,101,114,115,101,0,0,110,114,101,115,95,100,111,117,98,108,101,95,105,110,118,101,114,115,101,0,0,0,0,0,101,112,111,105,110,116,95,100,111,117,98,108,101,95,97,100,100,0,0,0,0,0,0,0,101,112,111,105,110,116,95,103,101,116,120,121,122,0,0,0,115,101,116,95,105,111,95,98,117,102,102,101,114,95,115,105,122,101,0,0,0,0,0,0,98,105,103,95,116,111,95,98,121,116,101,115,0,0,0,0,98,121,116,101,115,95,116,111,95,98,105,103,0,0,0,0,115,116,114,111,110,103,95,98,105,103,114,97,110,100,0,0,76,111,103,32,111,102,32,97,32,110,111,110,45,112,111,115,105,116,105,118,101,32,110,117,109,98,101,114,10,0,0,0,112,114,101,112,97,114,101,95,98,97,115,105,115,0,0,0,109,117,108,50,95,98,114,105,99,107,0,0,0,0,0,0,101,98,114,105,99,107,50,95,105,110,105,116,0,0,0,0,101,99,117,114,118,101,50,95,109,117,108,116,50,0,0,0,101,99,117,114,118,101,50,95,109,117,108,116,110,0,0,0,101,99,117,114,118,101,50,95,109,117,108,116,0,0,0,0,101,99,117,114,118,101,50,95,109,117,108,116,105,95,97,100,100,0,0,0,0,0,0,0,101,99,117,114,118,101,50,95,115,117,98,0,0,0,0,0,101,112,111,105,110,116,50,95,110,101,103,97,116,101,0,0,101,99,117,114,118,101,50,95,97,100,100,0,0,0,0,0,78,117,109,98,101,114,115,32,116,111,111,32,98,105,103,10,0,0,0,0,0,0,0,0,101,112,111,105,110,116,50,95,99,111,109,112,0,0,0,0,101,112,111,105,110,116,50,95,103,101,116,0,0,0,0,0,101,112,111,105,110,116,50,95,110,111,114,109,0,0,0,0,101,112,111,105,110,116,50,95,115,101,116,0,0,0,0,0,101,99,117,114,118,101,50,95,105,110,105,116,0,0,0,0,101,99,117,114,118,101,95,109,117,108,116,105,95,97,100,100,0,0,0,0,0,0,0,0,101,112,111,105,110,116,95,110,101,103,97,116,101,0,0,0,110,114,101,115,95,100,111,116,112,114,111,100,0,0,0,0,110,114,101,115,95,109,117,108,116,105,95,105,110,118,101,114,115,101,0,0,0,0,0,0,101,112,111,105,110,116,95,110,111,114,109,0,0,0,0,0,70,108,97,115,104,32,111,118,101,114,102,108,111,119,10,0,109,117,108,95,98,114,105,99,107,0,0,0,0,0,0,0,101,98,114,105,99,107,95,105,110,105,116,0,0,0,0,0,101,99,117,114,118,101,95,109,117,108,116,110,0,0,0,0,112,111,119,109,111,100,110,0,110,114,101,115,95,112,111,119,109,111,100,110,0,0,0,0,115,101,116,95,117,115,101,114,95,102,117,110,99,116,105,111,110,0,0,0,0,0,0,0,112,111,119,95,98,114,105,99,107,0,0,0,0,0,0,0,98,114,105,99,107,95,105,110,105,116,0,0,0,0,0,0,108,117,99,97,115,0,0,0,110,114,101,115,95,108,117,99,97,115,0,0,0,0,0,0,73,110,116,101,103,101,114,32,111,112,101,114,97,116,105,111,110,32,97,116,116,101,109,112,116,101,100,32,111,110,32,70,108,97,115,104,32,110,117,109,98,101,114,10,0,0,0,0,110,120,115,97,102,101,112,114,105,109,101,0,0,0,0,0,116,114,105,97,108,95,100,105,118,105,115,105,111,110,0,0,101,99,117,114,118,101,95,115,117,98,0,0,0,0,0,0,101,99,117,114,118,101,95,109,117,108,116,50,0,0,0,0,110,114,101,115,95,112,114,101,109,117,108,116,0,0,0,0,115,113,114,111,111,116,0,0,110,114,101,115,95,115,113,114,111,111,116,0,0,0,0,0,110,114,101,115,95,112,111,119,109,111,100,50,0,0,0,0,101,112,111,105,110,116,95,103,101,116,0,0,0,0,0,0,101,112,111,105,110,116,95,115,101,116,0,0,0,0,0,0,65,116,116,101,109,112,116,32,116,111,32,116,97,107,101,32,105,108,108,101,103,97,108,32,114,111,111,116,10,0,0,0,101,112,111,105,110,116,95,105,110,105,116,0,0,0,0,0,101,99,117,114,118,101,95,109,117,108,116,0,0,0,0,0,101,99,117,114,118,101,95,97,100,100,0,0,0,0,0,0,101,99,117,114,118,101,95,105,110,105,116,0,0,0,0,0,110,114,101,115,95,110,101,103,97,116,101,0,0,0,0,0,110,114,101,115,95,109,111,100,115,117,98,0,0,0,0,0,110,114,101,115,95,109,111,100,97,100,100,0,0,0,0,0,102,109,111,100,117,108,111,0,114,101,109,97,105,110,0,0,100,105,118,105,115,105,98,108,101,0,0,0,0,0,0,0,82,97,105,115,105,110,103,32,105,110,116,101,103,101,114,32,116,111,32,110,101,103,97,116,105,118,101,32,112,111,119,101,114,10,0,0,0,0,0,0,110,114,101,115,95,112,111,119,108,116,114,0,0,0,0,0,110,114,101,115,95,109,111,100,100,105,118,0,0,0,0,0,110,114,101,115,95,112,111,119,109,111,100,0,0,0,0,0,110,114,101,115,95,109,111,100,109,117,108,116,0,0,0,0,114,101,100,99,0,0,0,0,110,114,101,115,0,0,0,0,112,114,101,112,97,114,101,95,109,111,110,116,121,0,0,0,112,111,119,109,111,100,50,0,99,105,110,115,116,114,0,0,99,111,116,115,116,114,0,0,69,118,101,110,32,114,111,111,116,32,111,102,32,97,32,110,101,103,97,116,105,118,101,32,110,117,109,98,101,114,10,0,105,110,115,116,114,0,0,0,111,116,115,116,114,0,0,0,99,114,116,0,0,0,0,0,99,114,116,95,105,110,105,116,0,0,0,0,0,0,0,0,102,102,116,95,109,117,108,116,0,0,0,0,0,0,0,0,112,111,119,108,116,114,0,0,103,112,114,105,109,101,0,0,102,108,111,112,0,0,0,0,102,97,99,111,115,104,0,0,102,99,111,115,104,0,0,0,79,117,116,32,111,102,32,115,112,97,99,101,10,0,0,0,63,63,63,0,0,0,0,0,10,77,73,82,65,67,76,32,101,114,114,111,114,32,102,114,111,109,32,114,111,117,116,105,110,101,32,0,0,0,0,0,56,48,52,52,52,68,70,69,67,69,57,48,51,69,0,0,56,69,69,66,68,52,68,48,52,67,51,55,55,49,0,0,54,70,49,49,48,54,68,68,65,57,57,52,68,65,0,0,53,67,51,57,51,67,51,48,70,65,67,67,70,52,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);



var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            continue;
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          if (stream.tty.output.length) {
            stream.tty.ops.put_char(stream.tty, 10);
          }
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }}};
  
  var MEMFS={ops_table:null,CONTENT_OWNING:1,CONTENT_FLEXIBLE:2,CONTENT_FIXED:3,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 0777, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            },
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.contents = [];
          node.contentMode = MEMFS.CONTENT_FLEXIBLE;
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },ensureFlexible:function (node) {
        if (node.contentMode !== MEMFS.CONTENT_FLEXIBLE) {
          var contents = node.contents;
          node.contents = Array.prototype.slice.call(contents);
          node.contentMode = MEMFS.CONTENT_FLEXIBLE;
        }
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.contents.length;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.ensureFlexible(node);
            var contents = node.contents;
            if (attr.size < contents.length) contents.length = attr.size;
            else while (attr.size > contents.length) contents.push(0);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 0777 | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          var node = stream.node;
          node.timestamp = Date.now();
          var contents = node.contents;
          if (length && contents.length === 0 && position === 0 && buffer.subarray) {
            // just replace it with the new data
            if (canOwn && offset === 0) {
              node.contents = buffer; // this could be a subarray of Emscripten HEAP, or allocated from some other source.
              node.contentMode = (buffer.buffer === HEAP8.buffer) ? MEMFS.CONTENT_OWNING : MEMFS.CONTENT_FIXED;
            } else {
              node.contents = new Uint8Array(buffer.subarray(offset, offset+length));
              node.contentMode = MEMFS.CONTENT_FIXED;
            }
            return length;
          }
          MEMFS.ensureFlexible(node);
          var contents = node.contents;
          while (contents.length < position) contents.push(0);
          for (var i = 0; i < length; i++) {
            contents[position + i] = buffer[offset + i];
          }
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.contents.length;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.ungotten = [];
          stream.position = position;
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.ensureFlexible(stream.node);
          var contents = stream.node.contents;
          var limit = offset + length;
          while (limit > contents.length) contents.push(0);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < contents.length) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = {};
        for (var key in src.files) {
          if (!src.files.hasOwnProperty(key)) continue;
          var e = src.files[key];
          var e2 = dst.files[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create[key] = e;
            total++;
          }
        }
  
        var remove = {};
        for (var key in dst.files) {
          if (!dst.files.hasOwnProperty(key)) continue;
          var e = dst.files[key];
          var e2 = src.files[key];
          if (!e2) {
            remove[key] = e;
            total++;
          }
        }
  
        if (!total) {
          // early out
          return callback(null);
        }
  
        var completed = 0;
        function done(err) {
          if (err) return callback(err);
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        // create a single transaction to handle and IDB reads / writes we'll need to do
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        transaction.onerror = function transaction_onerror() { callback(this.error); };
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        for (var path in create) {
          if (!create.hasOwnProperty(path)) continue;
          var entry = create[path];
  
          if (dst.type === 'local') {
            // save file to local
            try {
              if (FS.isDir(entry.mode)) {
                FS.mkdir(path, entry.mode);
              } else if (FS.isFile(entry.mode)) {
                var stream = FS.open(path, 'w+', 0666);
                FS.write(stream, entry.contents, 0, entry.contents.length, 0, true /* canOwn */);
                FS.close(stream);
              }
              done(null);
            } catch (e) {
              return done(e);
            }
          } else {
            // save file to IDB
            var req = store.put(entry, path);
            req.onsuccess = function req_onsuccess() { done(null); };
            req.onerror = function req_onerror() { done(this.error); };
          }
        }
  
        for (var path in remove) {
          if (!remove.hasOwnProperty(path)) continue;
          var entry = remove[path];
  
          if (dst.type === 'local') {
            // delete file from local
            try {
              if (FS.isDir(entry.mode)) {
                // TODO recursive delete?
                FS.rmdir(path);
              } else if (FS.isFile(entry.mode)) {
                FS.unlink(path);
              }
              done(null);
            } catch (e) {
              return done(e);
            }
          } else {
            // delete file from IDB
            var req = store.delete(path);
            req.onsuccess = function req_onsuccess() { done(null); };
            req.onerror = function req_onerror() { done(this.error); };
          }
        }
      },getLocalSet:function (mount, callback) {
        var files = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint)
          .filter(isRealDir)
          .map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat, node;
  
          try {
            var lookup = FS.lookupPath(path);
            node = lookup.node;
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path)
              .filter(isRealDir)
              .map(toAbsolute(path)));
  
            files[path] = { mode: stat.mode, timestamp: stat.mtime };
          } else if (FS.isFile(stat.mode)) {
            files[path] = { contents: node.contents, mode: stat.mode, timestamp: stat.mtime };
          } else {
            return callback(new Error('node type not supported'));
          }
        }
  
        return callback(null, { type: 'local', files: files });
      },getDB:function (name, callback) {
        // look it up in the cache
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        req.onupgradeneeded = function req_onupgradeneeded() {
          db = req.result;
          db.createObjectStore(IDBFS.DB_STORE_NAME);
        };
        req.onsuccess = function req_onsuccess() {
          db = req.result;
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function req_onerror() {
          callback(this.error);
        };
      },getRemoteSet:function (mount, callback) {
        var files = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function transaction_onerror() { callback(this.error); };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          store.openCursor().onsuccess = function store_openCursor_onsuccess(event) {
            var cursor = event.target.result;
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, files: files });
            }
  
            files[cursor.key] = cursor.value;
            cursor.continue();
          };
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          stream.position = position;
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      // we don't currently perform any user-space buffering of data
    }var FS={root:null,mounts:[],devices:[null],streams:[null],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || { recurse_count: 0 };
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            current = current.mount.root;
          }
  
          // follow symlinks
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
            this.parent = null;
            this.mount = null;
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            FS.hashAddNode(this);
          };
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          FS.FSNode.prototype = {};
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); },
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); },
            },
          });
        }
        return new FS.FSNode(parent, name, mode, rdev);
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        return FS.nodePermissions(dir, 'x');
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 1;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        if (stream.__proto__) {
          // reuse the object
          stream.__proto__ = FS.FSStream.prototype;
        } else {
          var newStream = new FS.FSStream();
          for (var p in stream) {
            newStream[p] = stream[p];
          }
          stream = newStream;
        }
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var completed = 0;
        var total = FS.mounts.length;
        function done(err) {
          if (err) {
            return callback(err);
          }
          if (++completed >= total) {
            callback(null);
          }
        };
  
        // sync all mounts
        for (var i = 0; i < FS.mounts.length; i++) {
          var mount = FS.mounts[i];
          if (!mount.type.syncfs) {
            done(null);
            continue;
          }
          mount.type.syncfs(mount, populate, done);
        }
      },mount:function (type, opts, mountpoint) {
        var lookup;
        if (mountpoint) {
          lookup = FS.lookupPath(mountpoint, { follow: false });
          mountpoint = lookup.path;  // use the absolute path
        }
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          root: null
        };
        // create a root node for the fs
        var root = type.mount(mount);
        root.mount = mount;
        mount.root = root;
        // assign the mount info to the mountpoint's node
        if (lookup) {
          lookup.node.mount = mount;
          lookup.node.mounted = true;
          // compatibility update FS.root if we mount to /
          if (mountpoint === '/') {
            FS.root = mount.root;
          }
        }
        // add to our cached list of mounts
        FS.mounts.push(mount);
        return root;
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 0666;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 0777;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 0666;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
      },readlink:function (path) {
        var lookup = FS.lookupPath(path, { follow: false });
        var link = lookup.node;
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 0666 : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions
        var err = FS.mayOpen(node, flags);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        return stream.stream_ops.llseek(stream, offset, whence);
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        } else {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0);
        } else {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=stdin.fd;
        assert(stdin.fd === 1, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=stdout.fd;
        assert(stdout.fd === 2, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=stderr.fd;
        assert(stderr.fd === 3, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno) {
          this.errno = errno;
          for (var key in ERRNO_CODES) {
            if (ERRNO_CODES[key] === errno) {
              this.code = key;
              break;
            }
          }
          this.message = ERRNO_MESSAGES[errno];
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.root = FS.createNode(null, '/', 16384 | 0777, 0);
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
          function LazyUint8Array() {
            this.lengthKnown = false;
            this.chunks = []; // Loaded chunks. Index is the chunk number
          }
          LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
            if (idx > this.length-1 || idx < 0) {
              return undefined;
            }
            var chunkOffset = idx % this.chunkSize;
            var chunkNum = Math.floor(idx / this.chunkSize);
            return this.getter(chunkNum)[chunkOffset];
          }
          LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
            this.getter = getter;
          }
          LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
              // Find length
              var xhr = new XMLHttpRequest();
              xhr.open('HEAD', url, false);
              xhr.send(null);
              if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
              var datalength = Number(xhr.getResponseHeader("Content-length"));
              var header;
              var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
              var chunkSize = 1024*1024; // Chunk size in bytes
  
              if (!hasByteServing) chunkSize = datalength;
  
              // Function to get a range from the remote URL.
              var doXHR = (function(from, to) {
                if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
                if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
                // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
                var xhr = new XMLHttpRequest();
                xhr.open('GET', url, false);
                if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
                // Some hints to the browser that we want binary data.
                if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
                if (xhr.overrideMimeType) {
                  xhr.overrideMimeType('text/plain; charset=x-user-defined');
                }
  
                xhr.send(null);
                if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
                if (xhr.response !== undefined) {
                  return new Uint8Array(xhr.response || []);
                } else {
                  return intArrayFromString(xhr.responseText || '', true);
                }
              });
              var lazyArray = this;
              lazyArray.setDataGetter(function(chunkNum) {
                var start = chunkNum * chunkSize;
                var end = (chunkNum+1) * chunkSize - 1; // including this byte
                end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
                if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
                  lazyArray.chunks[chunkNum] = doXHR(start, end);
                }
                if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
                return lazyArray.chunks[chunkNum];
              });
  
              this._length = datalength;
              this._chunkSize = chunkSize;
              this.lengthKnown = true;
          }
  
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};
  
  
  
  
  var _mkport=undefined;var SOCKFS={mount:function (mount) {
        return FS.createNode(null, '/', 16384 | 0777, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              var url = 'ws://' + addr + ':' + port;
              // the node ws library API is slightly different than the browser's
              var opts = ENVIRONMENT_IS_NODE ? {headers: {'websocket-protocol': ['binary']}} : ['binary'];
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('error', function() {
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODS.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
            }
          });
          sock.server.on('closed', function() {
            sock.server = null;
          });
          sock.server.on('error', function() {
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fputc(c, stream) {
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[((_fputc.ret)|0)]=chr;
      var ret = _write(stream, _fputc.ret, 1);
      if (ret == -1) {
        var streamObj = FS.getStream(stream);
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        return chr;
      }
    }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  
   
  Module["_memset"] = _memset;var _llvm_memset_p0i8_i32=_memset;

  var _llvm_memset_p0i8_i64=_memset;

  function _llvm_uadd_with_overflow_i32(x, y) {
      x = x>>>0;
      y = y>>>0;
      return ((asm["setTempRet0"](x+y > 4294967295),(x+y)>>>0)|0);
    }

  
  
  
  function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStream(stream);
      if (!streamObj) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)|0)]=streamObj.ungotten.pop();
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(stream, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return Math.floor(bytesRead / size);
    }function _fgetc(stream) {
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      var streamObj = FS.getStream(stream);
      if (!streamObj) return -1;
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _fread(_fgetc.ret, 1, 1, stream);
      if (ret == 0) {
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAPU8[((_fgetc.ret)|0)];
      }
    }

  
   
  Module["_memcpy"] = _memcpy;var _llvm_memcpy_p0i8_p0i8_i32=_memcpy;

  function _toupper(chr) {
      if (chr >= 97 && chr <= 122) {
        return chr - 97 + 65;
      } else {
        return chr;
      }
    }

   
  Module["_strlen"] = _strlen;

  function _strncmp(px, py, n) {
      var i = 0;
      while (i < n) {
        var x = HEAPU8[(((px)+(i))|0)];
        var y = HEAPU8[(((py)+(i))|0)];
        if (x == y && x == 0) return 0;
        if (x == 0) return -1;
        if (y == 0) return 1;
        if (x == y) {
          i ++;
          continue;
        } else {
          return x > y ? 1 : -1;
        }
      }
      return 0;
    }

   
  Module["_strcpy"] = _strcpy;

   
  Module["_strcat"] = _strcat;

  
  
  function _isspace(chr) {
      return (chr == 32) || (chr >= 9 && chr <= 13);
    }
  function __parseInt64(str, endptr, base, min, max, unsign) {
      var isNegative = false;
      // Skip space.
      while (_isspace(HEAP8[(str)])) str++;
  
      // Check for a plus/minus sign.
      if (HEAP8[(str)] == 45) {
        str++;
        isNegative = true;
      } else if (HEAP8[(str)] == 43) {
        str++;
      }
  
      // Find base.
      var ok = false;
      var finalBase = base;
      if (!finalBase) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            finalBase = 16;
            str += 2;
          } else {
            finalBase = 8;
            ok = true; // we saw an initial zero, perhaps the entire thing is just "0"
          }
        }
      } else if (finalBase==16) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            str += 2;
          }
        }
      }
      if (!finalBase) finalBase = 10;
      var start = str;
  
      // Get digits.
      var chr;
      while ((chr = HEAP8[(str)]) != 0) {
        var digit = parseInt(String.fromCharCode(chr), finalBase);
        if (isNaN(digit)) {
          break;
        } else {
          str++;
          ok = true;
        }
      }
  
      if (!ok) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return ((asm["setTempRet0"](0),0)|0);
      }
  
      // Set end pointer.
      if (endptr) {
        HEAP32[((endptr)>>2)]=str;
      }
  
      try {
        var numberString = isNegative ? '-'+Pointer_stringify(start, str - start) : Pointer_stringify(start, str - start);
        i64Math.fromString(numberString, finalBase, min, max, unsign);
      } catch(e) {
        ___setErrNo(ERRNO_CODES.ERANGE); // not quite correct
      }
  
      return ((asm["setTempRet0"](((HEAP32[(((tempDoublePtr)+(4))>>2)])|0)),((HEAP32[((tempDoublePtr)>>2)])|0))|0);
    }function _strtoll(str, endptr, base) {
      return __parseInt64(str, endptr, base, '-9223372036854775808', '9223372036854775807');  // LLONG_MIN, LLONG_MAX.
    }

  function _abort() {
      Module['abort']();
    }

  function ___errno_location() {
      return ___errno_state;
    }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }

  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: return 1;
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  function _time(ptr) {
      var ret = Math.floor(Date.now()/1000);
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }






  var Browser={mainLoop:{scheduler:null,shouldPause:false,paused:false,queue:[],pause:function () {
          Browser.mainLoop.shouldPause = true;
        },resume:function () {
          if (Browser.mainLoop.paused) {
            Browser.mainLoop.paused = false;
            Browser.mainLoop.scheduler();
          }
          Browser.mainLoop.shouldPause = false;
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted || ENVIRONMENT_IS_WORKER) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                    canvas['mozRequestPointerLock'] ||
                                    canvas['webkitRequestPointerLock'];
        canvas.exitPointerLock = document['exitPointerLock'] ||
                                 document['mozExitPointerLock'] ||
                                 document['webkitExitPointerLock'] ||
                                 function(){}; // no-op if function does not exist
        canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas;
        }
  
        document.addEventListener('pointerlockchange', pointerLockChange, false);
        document.addEventListener('mozpointerlockchange', pointerLockChange, false);
        document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
  
        if (Module['elementPointerLock']) {
          canvas.addEventListener("click", function(ev) {
            if (!Browser.pointerLock && canvas.requestPointerLock) {
              canvas.requestPointerLock();
              ev.preventDefault();
            }
          }, false);
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        var ctx;
        try {
          if (useWebGL) {
            var contextAttributes = {
              antialias: false,
              alpha: false
            };
  
            if (webGLContextAttributes) {
              for (var attribute in webGLContextAttributes) {
                contextAttributes[attribute] = webGLContextAttributes[attribute];
              }
            }
  
  
            var errorInfo = '?';
            function onContextCreationError(event) {
              errorInfo = event.statusMessage || errorInfo;
            }
            canvas.addEventListener('webglcontextcreationerror', onContextCreationError, false);
            try {
              ['experimental-webgl', 'webgl'].some(function(webglId) {
                return ctx = canvas.getContext(webglId, contextAttributes);
              });
            } finally {
              canvas.removeEventListener('webglcontextcreationerror', onContextCreationError, false);
            }
          } else {
            ctx = canvas.getContext('2d');
          }
          if (!ctx) throw ':(';
        } catch (e) {
          Module.print('Could not create canvas: ' + [errorInfo, e]);
          return null;
        }
        if (useWebGL) {
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
  
          // Warn on context loss
          canvas.addEventListener('webglcontextlost', function(event) {
            alert('WebGL context lost. You will need to reload the page.');
          }, false);
        }
        if (setInModule) {
          GLctx = Module.ctx = ctx;
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement']) === canvas) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'];
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else if (Browser.resizeCanvas){
            Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
        }
  
        canvas.requestFullScreen = canvas['requestFullScreen'] ||
                                   canvas['mozRequestFullScreen'] ||
                                   (canvas['webkitRequestFullScreen'] ? function() { canvas['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvas.requestFullScreen();
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          setTimeout(func, 1000/60);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           window['setTimeout'];
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var x, y;
          
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          if (event.type == 'touchstart' ||
              event.type == 'touchend' ||
              event.type == 'touchmove') {
            var t = event.touches.item(0);
            if (t) {
              x = t.pageX - (scrollX + rect.left);
              y = t.pageY - (scrollY + rect.top);
            } else {
              return;
            }
          } else {
            x = event.pageX - (scrollX + rect.left);
            y = event.pageY - (scrollY + rect.top);
          }
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        canvas.width = width;
        canvas.height = height;
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        var canvas = Module['canvas'];
        this.windowedWidth = canvas.width;
        this.windowedHeight = canvas.height;
        canvas.width = screen.width;
        canvas.height = screen.height;
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        var canvas = Module['canvas'];
        canvas.width = this.windowedWidth;
        canvas.height = this.windowedHeight;
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      }};
_fputc.ret = allocate([0], "i8", ALLOC_STATIC);
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
_fgetc.ret = allocate([0], "i8", ALLOC_STATIC);
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + 5242880;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);

var Math_min = Math.min;
function invoke_i(index) {
  try {
    return Module["dynCall_i"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function asmPrintInt(x, y) {
  Module.print('int ' + x + ',' + y);// + ' ' + new Error().stack);
}
function asmPrintFloat(x, y) {
  Module.print('float ' + x + ',' + y);// + ' ' + new Error().stack);
}
// EMSCRIPTEN_START_ASM
var asm=(function(global,env,buffer){"use asm";var a=new global.Int8Array(buffer);var b=new global.Int16Array(buffer);var c=new global.Int32Array(buffer);var d=new global.Uint8Array(buffer);var e=new global.Uint16Array(buffer);var f=new global.Uint32Array(buffer);var g=new global.Float32Array(buffer);var h=new global.Float64Array(buffer);var i=env.STACKTOP|0;var j=env.STACK_MAX|0;var k=env.tempDoublePtr|0;var l=env.ABORT|0;var m=env.cttz_i8|0;var n=env.ctlz_i8|0;var o=env._stdout|0;var p=+env.NaN;var q=+env.Infinity;var r=0;var s=0;var t=0;var u=0;var v=0,w=0,x=0,y=0,z=0.0,A=0,B=0,C=0,D=0.0;var E=0;var F=0;var G=0;var H=0;var I=0;var J=0;var K=0;var L=0;var M=0;var N=0;var O=global.Math.floor;var P=global.Math.abs;var Q=global.Math.sqrt;var R=global.Math.pow;var S=global.Math.cos;var T=global.Math.sin;var U=global.Math.tan;var V=global.Math.acos;var W=global.Math.asin;var X=global.Math.atan;var Y=global.Math.atan2;var Z=global.Math.exp;var _=global.Math.log;var $=global.Math.ceil;var aa=global.Math.imul;var ba=env.abort;var ca=env.assert;var da=env.asmPrintInt;var ea=env.asmPrintFloat;var fa=env.min;var ga=env.invoke_i;var ha=env.invoke_vi;var ia=env.invoke_ii;var ja=env.invoke_v;var ka=env.invoke_iii;var la=env._strncmp;var ma=env._sysconf;var na=env._fgetc;var oa=env._fread;var pa=env._abort;var qa=env._toupper;var ra=env._pread;var sa=env._strtoll;var ta=env._fflush;var ua=env._fputc;var va=env.___setErrNo;var wa=env._send;var xa=env._write;var ya=env._exit;var za=env._isspace;var Aa=env._read;var Ba=env._llvm_uadd_with_overflow_i32;var Ca=env._recv;var Da=env.__parseInt64;var Ea=env._pwrite;var Fa=env._sbrk;var Ga=env.___errno_location;var Ha=env._time;var Ia=env.__exit;var Ja=0.0;
// EMSCRIPTEN_START_FUNCS
function Pa(a){a=a|0;var b=0;b=i;i=i+a|0;i=i+7&-8;return b|0}function Qa(){return i|0}function Ra(a){a=a|0;i=a}function Sa(a,b){a=a|0;b=b|0;if((r|0)==0){r=a;s=b}}function Ta(b){b=b|0;a[k]=a[b];a[k+1|0]=a[b+1|0];a[k+2|0]=a[b+2|0];a[k+3|0]=a[b+3|0]}function Ua(b){b=b|0;a[k]=a[b];a[k+1|0]=a[b+1|0];a[k+2|0]=a[b+2|0];a[k+3|0]=a[b+3|0];a[k+4|0]=a[b+4|0];a[k+5|0]=a[b+5|0];a[k+6|0]=a[b+6|0];a[k+7|0]=a[b+7|0]}function Va(a){a=a|0;E=a}function Wa(a){a=a|0;F=a}function Xa(a){a=a|0;G=a}function Ya(a){a=a|0;H=a}function Za(a){a=a|0;I=a}function _a(a){a=a|0;J=a}function $a(a){a=a|0;K=a}function ab(a){a=a|0;L=a}function bb(a){a=a|0;M=a}function cb(a){a=a|0;N=a}function db(){}function eb(){return c[1414]|0}function fb(a){a=a|0;var b=0,d=0,e=0;b=c[1414]|0;if((c[b+548>>2]|0)!=0){c[b+552>>2]=a;return}gb(5552);b=c[1414]|0;d=c[b+28>>2]|0;if((d|0)<24){gb(c[40+(c[b+32+(d<<2)>>2]<<2)>>2]|0)}else{gb(5544)}d=c[o>>2]|0;ua(10,d|0)|0;b=c[(c[1414]|0)+28>>2]|0;if((b|0)>0){e=b;while(1){b=e-1|0;gb(3704);if((e|0)<25){gb(c[40+(c[(c[1414]|0)+32+(b<<2)>>2]<<2)>>2]|0)}else{gb(5544)}ua(10,d|0)|0;if((b|0)>0){e=b}else{break}}}switch(a|0){case 1:{gb(2072);break};case 2:{gb(1856);break};case 3:{gb(1712);break};case 4:{gb(1576);break};case 5:{gb(1456);break};case 6:{gb(1360);break};case 7:{gb(1248);break};case 8:{gb(5528);break};case 9:{gb(5400);break};case 10:{gb(5240);break};case 11:{gb(5064);break};case 12:{gb(4864);break};case 13:{gb(4696);break};case 14:{gb(4496);break};case 15:{gb(4296);break};case 16:{gb(4080);break};case 17:{gb(3896);break};case 18:{gb(3656);break};case 19:{gb(3472);break};case 20:{gb(3288);break};case 21:{gb(3096);break};case 22:{gb(2904);break};case 23:{gb(2680);break};case 24:{gb(2472);break};case 25:{gb(2272);break};case 26:{gb(2144);break};case 27:{gb(2112);break};case 28:{gb(2048);break};case 29:{gb(1992);break};default:{gb(1968)}}ya(0)}function gb(b){b=b|0;var d=0,e=0,f=0,g=0;d=a[b]|0;if(d<<24>>24==0){return}e=c[o>>2]|0;f=0;g=d;do{f=f+1|0;ua(g<<24>>24|0,e|0)|0;g=a[b+f|0]|0;}while(g<<24>>24!=0);return}function hb(){var a=0,b=0;a=c[o>>2]|0;if((c[(c[1414]|0)+28>>2]|0)>0){b=0;do{ua(45,a|0)|0;b=b+1|0;}while((b|0)<(c[(c[1414]|0)+28>>2]|0))}ua(62,a|0)|0;b=c[1414]|0;gb(c[40+(c[b+32+(c[b+28>>2]<<2)>>2]<<2)>>2]|0);ua(10,a|0)|0;return}function ib(a,b){a=a|0;b=b|0;var c=0;do{if((b|0)==0){c=a}else{if((b|0)>0){c=a<<b;break}else{c=a>>>((-b|0)>>>0);break}}}while(0);return c|0}function jb(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;b=32;while(1){d=(b|0)/2|0;e=1<<d;if((e|0)==(a|0)){f=5;break}if(e>>>0<a>>>0){f=6;break}if((d&1|0)==0&(b|0)>3){b=d}else{f=7;break}}if((f|0)==5){c[(c[1414]|0)+4>>2]=a;c[(c[1414]|0)+8>>2]=32/(d|0)|0;c[c[1414]>>2]=0;return 0}else if((f|0)==6){c[(c[1414]|0)+4>>2]=a}else if((f|0)==7){c[(c[1414]|0)+4>>2]=a}c[(c[1414]|0)+8>>2]=1;c[c[1414]>>2]=a;f=c[1414]|0;d=c[f>>2]|0;if((d|0)==0){return 0}b=2147483648/(a>>>0)|0;if(b>>>0<a>>>0){return 0}else{g=b;h=f;i=d}while(1){c[h>>2]=aa(i,a)|0;d=(c[1414]|0)+8|0;c[d>>2]=(c[d>>2]|0)+1;d=(g>>>0)/(a>>>0)|0;if(d>>>0<a>>>0){break}f=c[1414]|0;g=d;h=f;i=c[f>>2]|0}return 0}function kb(a){a=a|0;var b=0;b=c[a>>2]|0;return(b>>>16&32767)+(b&65535)|0}function lb(a){a=a|0;var b=0;if((a|0)==0){return}b=kb(a)|0;if((b|0)>0){od(c[a+4>>2]|0,0,b<<2|0)|0}c[a>>2]=0;return}function mb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;lb(b);if((a|0)==0){return}d=c[c[1414]>>2]|0;e=b+4|0;if((d|0)==0){c[c[e>>2]>>2]=a;f=1}else{g=a;a=0;h=d;while(1){d=a+1|0;c[(c[e>>2]|0)+(a<<2)>>2]=(g>>>0)%(h>>>0)|0;i=c[c[1414]>>2]|0;j=(g>>>0)/(i>>>0)|0;if((j|0)==0){f=d;break}else{g=j;a=d;h=i}}}c[b>>2]=f;return}function nb(a,b){a=a|0;b=b|0;var d=0;if((a|0)==0){lb(b);return}else{mb((a|0)<0?-a|0:a,b);d=b|0;c[d>>2]=c[d>>2]|a&-2147483648;return}}function ob(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=c[1414]|0;if((c[b+552>>2]|0)!=0){d=0;return d|0}e=b+28|0;c[e>>2]=(c[e>>2]|0)+1;e=c[1414]|0;b=c[e+28>>2]|0;do{if((b|0)<24){c[e+32+(b<<2)>>2]=23;f=c[1414]|0;if((c[f+576>>2]|0)==0){g=f;break}hb();g=c[1414]|0}else{g=e}}while(0);if((c[g+140>>2]|0)==0){fb(18);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=0;return d|0}e=ic((c[g+24>>2]<<2)+15&-4,1)|0;g=e;if((e|0)==0){b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;d=g;return d|0}c[e+4>>2]=e+(12-(e+8&3));if((a|0)!=0){nb(a,g)}a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)-1;d=g;return d|0}function pb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=a&3;f=(aa((d<<2)+19&-4,b)|0)+((e|0)==0?0:4-e|0)|0;c[a+(f+4)>>2]=a+(f+12-(a+(f+8)&3));return a+f|0}function qb(a,b){a=a|0;b=b|0;var d=0,e=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}e=pb(a,b,(c[d+24>>2]|0)-1|0)|0;return e|0}function rb(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=a|0;d=c[b>>2]|0;e=d&-2147483648;f=a+4|0;a=d&2147483647;while(1){if((a|0)<=0){g=0;h=4;break}d=a-1|0;if((c[(c[f>>2]|0)+(d<<2)>>2]|0)==0){a=d}else{g=e;h=4;break}}if((h|0)==4){c[b>>2]=g|a;return}}function sb(a,b){a=a|0;b=b|0;var d=0;c[1414]=hc()|0;d=eb()|0;c[1414]=d;return tb(d,a,b)|0}function tb(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((b|0)==0){f=0;return f|0}g=b+28|0;c[b+32>>2]=0;c[g>>2]=1;c[b+36>>2]=29;c[b+548>>2]=0;c[b+364>>2]=0;c[b+372>>2]=0;c[b+404>>2]=0;c[b+20>>2]=0;c[b+452>>2]=0;c[b+456>>2]=0;c[b+460>>2]=0;c[b+352>>2]=0;c[b+336>>2]=0;if((e|0)==1|e>>>0>2147483648>>>0){fb(6);c[g>>2]=(c[g>>2]|0)-1;f=b;return f|0}jb(e)|0;e=c[b>>2]|0;h=b+12|0;c[h>>2]=0;i=b+16|0;c[i>>2]=1;do{if((e|0)==0){c[h>>2]=32;c[i>>2]=0;j=32}else{if(e>>>0>1>>>0){k=e;l=0;m=1}else{j=0;break}while(1){n=l+1|0;o=m<<1;if(k>>>0>3>>>0){k=k>>>1;l=n;m=o}else{break}}c[h>>2]=n;c[i>>2]=o;j=n}}while(0);if((d|0)>0){p=(d-1|0)/(c[b+8>>2]|0)|0}else{p=((d*-8|0)-1|0)/(j|0)|0}d=(p|0)==-1?2:p+2|0;p=b+24|0;c[p>>2]=d;c[b+588>>2]=d;n=b+592|0;c[n>>2]=d;a:do{if((d|0)>2){o=52/(j|0)|0;i=d;while(1){if((i|0)<=(o|0)){q=i;break a}h=(i+1|0)/2|0;c[n>>2]=h;if((i|0)>4){i=h}else{q=h;break}}}else{q=d}}while(0);if((q|0)<2){c[n>>2]=2}c[b+128>>2]=1;c[b+564>>2]=10;c[b+552>>2]=0;c[b+556>>2]=6;c[b+324>>2]=1;c[b+568>>2]=1;c[b+572>>2]=0;c[b+576>>2]=0;c[b+560>>2]=0;c[b+544>>2]=1024;c[b+580>>2]=0;n=ic(1025,1)|0;c[b+584>>2]=n;a[n]=0;od(b+704|0,0,20)|0;c[b+152>>2]=1431655765;c[b+156>>2]=305419896;n=2;q=632165481;d=1431655765;while(1){j=q+d|0;c[b+152+(n<<2)>>2]=j;i=n+1|0;if((i|0)>=37){break}o=c[b+152+(n-1<<2)>>2]|0;n=i;q=j+326745585|0;d=o}c[b+300>>2]=37;c[b+304>>2]=0;d=c[p>>2]<<1|1;c[p>>2]=d;if((d|0)==(d&65535|0)){d=ub(28)|0;q=b+700|0;c[q>>2]=d;c[b+348>>2]=0;c[b+136>>2]=0;c[b+132>>2]=0;c[b+140>>2]=1;c[p>>2]=((c[p>>2]|0)-1|0)/2|0;c[b+464>>2]=qb(d,0)|0;c[b+468>>2]=qb(c[q>>2]|0,2)|0;c[b+472>>2]=qb(c[q>>2]|0,3)|0;c[b+476>>2]=qb(c[q>>2]|0,4)|0;c[b+480>>2]=qb(c[q>>2]|0,5)|0;c[b+484>>2]=qb(c[q>>2]|0,6)|0;c[b+488>>2]=qb(c[q>>2]|0,8)|0;c[b+492>>2]=qb(c[q>>2]|0,10)|0;c[b+496>>2]=qb(c[q>>2]|0,12)|0;c[b+500>>2]=qb(c[q>>2]|0,13)|0;c[b+504>>2]=qb(c[q>>2]|0,14)|0;c[b+508>>2]=qb(c[q>>2]|0,15)|0;c[b+512>>2]=qb(c[q>>2]|0,16)|0;c[b+516>>2]=qb(c[q>>2]|0,17)|0;c[b+520>>2]=qb(c[q>>2]|0,18)|0;c[b+524>>2]=qb(c[q>>2]|0,19)|0;c[b+528>>2]=qb(c[q>>2]|0,20)|0;c[b+312>>2]=qb(c[q>>2]|0,21)|0;c[b+316>>2]=qb(c[q>>2]|0,22)|0;c[b+536>>2]=qb(c[q>>2]|0,24)|0;c[b+540>>2]=qb(c[q>>2]|0,25)|0;c[b+532>>2]=qb(c[q>>2]|0,26)|0;c[b+696>>2]=qb(c[q>>2]|0,27)|0;c[g>>2]=(c[g>>2]|0)-1;f=b;return f|0}else{fb(14);c[p>>2]=((c[p>>2]|0)-1|0)/2|0;c[g>>2]=(c[g>>2]|0)-1;f=b;return f|0}return 0}function ub(a){a=a|0;return ic((aa(a<<2,(((c[(c[1414]|0)+24>>2]<<2)+11|0)>>>2)+1|0)|0)+4|0,1)|0}function vb(a,b){a=a|0;b=b|0;if((a|0)==0){return}od(a|0,0,(aa(b<<2,(((c[(c[1414]|0)+24>>2]<<2)+11|0)>>>2)+1|0)|0)+4|0)|0;jc(a);return}function wb(a){a=a|0;return c[a>>2]>>31|1|0}function xb(a,b){a=a|0;b=b|0;var d=0;d=b|0;b=c[d>>2]|0;if((b|0)==0){return}if((a|0)<0){c[d>>2]=b|-2147483648;return}else{c[d>>2]=b&2147483647;return}}function yb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=b-1|0;b=c[1414]|0;e=c[b+8>>2]|0;f=c[(c[a+4>>2]|0)+(((d|0)/(e|0)|0)<<2)>>2]|0;if((e|0)==1){g=f;return g|0}a=(d|0)%(e|0)|0;e=c[b+4>>2]|0;if((a|0)<1){h=f}else{b=1;d=f;while(1){f=(d>>>0)/(e>>>0)|0;if((b|0)<(a|0)){b=b+1|0;d=f}else{h=f;break}}}g=(h>>>0)%(e>>>0)|0;return g|0}function zb(a){a=a|0;var b=0,d=0,e=0;b=c[a>>2]|0;if((b|0)==0){d=0;return d|0}e=aa(c[(c[1414]|0)+8>>2]|0,b&2147483647)|0;while(1){if((yb(a,e)|0)==0){e=e-1|0}else{d=e;break}}return d|0}function Ab(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=26;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);f=b|0;e=c[f>>2]|0;g=e&-2147483648;i=e&2147483647;e=yb(b,d)|0;j=d-1|0;d=c[h+8>>2]|0;k=(j|0)/(d|0)|0;l=(j|0)%(d|0)|0;if((l|0)<1){m=e;n=a}else{d=c[h+4>>2]|0;j=e;e=a;a=1;while(1){o=aa(d,j)|0;p=aa(d,e)|0;if((a|0)<(l|0)){j=o;e=p;a=a+1|0}else{m=o;n=p;break}}}a=c[h+24>>2]|0;do{if((k|0)>=(a|0)){if((c[h+128>>2]|0)==0&(k|0)<(a<<1|0)){break}fb(3);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}}while(0);a=(c[b+4>>2]|0)+(k<<2)|0;c[a>>2]=n-m+(c[a>>2]|0);if((k|0)>=(i|0)){c[f>>2]=k+1|g}rb(b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}function Bb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if((a|0)==(b|0)|(b|0)==0){return}if((a|0)==0){lb(b);return}d=kb(b)|0;e=kb(a)|0;f=c[a+4>>2]|0;g=c[b+4>>2]|0;if((d|0)>(e|0)){od(g+(e<<2)|0,0,d-e<<2|0)|0}if((e|0)>0){d=0;do{c[g+(d<<2)>>2]=c[f+(d<<2)>>2];d=d+1|0;}while((d|0)<(e|0))}c[b>>2]=c[a>>2];return}function Cb(a,b){a=a|0;b=b|0;Bb(a,b);a=b|0;b=c[a>>2]|0;if((b|0)==0){return}c[a>>2]=b^-2147483648;return}function Db(a){a=a|0;return(c[a>>2]&2147418112|0)!=0|0}function Eb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0;e=c[d+4>>2]|0;f=e;if((c[(c[1414]|0)+552>>2]|0)!=0){return}Bb(a,d);a=d|0;if((c[a>>2]|0)==0|(b|0)==0){return}g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;h=c[g+28>>2]|0;do{if((h|0)<24){c[g+32+(h<<2)>>2]=33;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Db(d)|0)!=0){fb(12)}h=c[a>>2]|0;g=h&-2147483648;i=h&2147483647;j=i+b|0;if((j|0)<1){lb(d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}d=c[1414]|0;do{if((j|0)>(c[d+24>>2]|0)){if((c[d+128>>2]|0)==0){k=d;break}fb(3);k=c[1414]|0}else{k=d}}while(0);if((c[k+552>>2]|0)!=0){d=k+28|0;c[d>>2]=(c[d>>2]|0)-1;return}do{if((b|0)>0){if((i|0)!=0){d=j;do{d=d-1|0;c[e+(d<<2)>>2]=c[e+(d-b<<2)>>2];}while((d|0)>(b|0))}od(f|0,0,b<<2|0)|0}else{d=-b|0;if((j|0)>0){k=0;do{c[e+(k<<2)>>2]=c[e+(k-b<<2)>>2];k=k+1|0;}while((k|0)<(j|0))}if((b|0)>=0){break}od(e+((h&2147483647)+b<<2)|0,0,((d|0)>1?d<<2:4)|0)|0}}while(0);c[a>>2]=j|g;g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}function Fb(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){return 0}b=c[a>>2]|0;d=b&2147483647;if((d|0)==1){e=c[c[a+4>>2]>>2]|0;if(e>>>0<1073741824>>>0){f=e}else{g=4}}else if((d|0)==0){return 0}else{g=4}if((g|0)==4){f=1073741824}return((b|0)<0?-f|0:f)|0}function Gb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;if((a|0)==(b|0)){d=0;return d|0}e=c[a>>2]|0;f=e&-2147483648;g=c[b>>2]|0;h=f>>>30^2;i=h-1|0;if((f|0)!=(g&-2147483648|0)){d=i;return d|0}f=e&2147483647;e=g&2147483647;if(f>>>0>e>>>0){d=i;return d|0}if(f>>>0<e>>>0){d=1-h|0;return d|0}e=a+4|0;a=b+4|0;b=f;while(1){if((b|0)<=0){d=0;j=11;break}f=b-1|0;g=c[(c[e>>2]|0)+(f<<2)>>2]|0;k=c[(c[a>>2]|0)+(f<<2)>>2]|0;if(g>>>0>k>>>0){d=i;j=11;break}if(g>>>0<k>>>0){j=10;break}else{b=f}}if((j|0)==10){d=1-h|0;return d|0}else if((j|0)==11){return d|0}return 0}function Hb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=31;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);e=b|0;f=c[e>>2]&2147483647;if((f|0)==1){g=(c[c[b+4>>2]>>2]|0)!=1|0}else if((f|0)==0){fb(13);g=0}else{g=f}if((d|0)==(b|0)){fb(7)}if((Db(a)|0)==0){if((Db(b)|0)!=0){h=12}}else{h=12}if((h|0)==12){fb(12)}h=a|0;f=c[h>>2]|0;i=f&2147483647;if((i|0)==1){j=(c[c[a+4>>2]>>2]|0)!=1|0}else{j=i}i=j+g|0;k=c[1414]|0;l=c[k+24>>2]|0;do{if((i|0)>(l|0)){if(!((c[k+128>>2]|0)!=0|(i|0)>(l<<1|0))){m=k;break}fb(13);m=c[1414]|0}else{m=k}}while(0);if((c[m+552>>2]|0)!=0){k=m+28|0;c[k>>2]=(c[k>>2]|0)-1;return}Bb(a,d);if((c[h>>2]|0)==0){h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;return}h=(c[e>>2]^f)&-2147483648;if((g|0)==0){f=d|0;e=c[f>>2]|0;if((e|0)!=0){c[f>>2]=e|h}e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}else{e=b+4|0;b=d+4|0;f=0;do{c[(c[b>>2]|0)+(f+j<<2)>>2]=c[(c[e>>2]|0)+(f<<2)>>2];f=f+1|0;}while((f|0)<(g|0));c[d>>2]=h|j+(g<<16);g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}}function Ib(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;if((c[(c[1414]|0)+552>>2]|0)!=0){return}if((Db(a)|0)==0){Bb(a,b);return}d=c[a>>2]|0;e=d&65535;if((e|0)==0){if((d|0)<0){nb(-1,b);return}else{nb(1,b);return}}f=d>>>16&32767;do{if((a|0)==(b|0)){if((f|0)==0){break}g=a+4|0;h=0;do{c[(c[g>>2]|0)+(h+e<<2)>>2]=0;h=h+1|0;}while((h|0)<(f|0))}else{h=a+4|0;g=b+4|0;i=0;do{c[(c[g>>2]|0)+(i<<2)>>2]=c[(c[h>>2]|0)+(i<<2)>>2];i=i+1|0;}while((i|0)<(e|0));if((e|0)>=(kb(b)|0)){break}i=b+4|0;h=e;do{c[(c[i>>2]|0)+(h<<2)>>2]=0;h=h+1|0;}while((h|0)<(kb(b)|0))}}while(0);c[b>>2]=d&-2147418113;return}function Jb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;if((c[(c[1414]|0)+552>>2]|0)!=0){return}if((Db(a)|0)==0){nb(1,b);return}d=c[a>>2]|0;e=d&65535;f=d>>>16&32767;if((f|0)!=0){d=a+4|0;g=b+4|0;h=0;do{c[(c[g>>2]|0)+(h<<2)>>2]=c[(c[d>>2]|0)+(h+e<<2)>>2];h=h+1|0;}while((h|0)<(f|0))}do{if((a|0)==(b|0)){if((e|0)==0){break}h=a+4|0;d=0;do{c[(c[h>>2]|0)+(d+f<<2)>>2]=0;d=d+1|0;}while((d|0)<(e|0))}else{if((f|0)>=(kb(b)|0)){break}d=b+4|0;h=f;do{c[(c[d>>2]|0)+(h<<2)>>2]=0;h=h+1|0;}while((h|0)<(kb(b)|0))}}while(0);c[b>>2]=f;return}function Kb(a,b){a=a|0;b=b|0;var d=0;d=c[(c[1414]|0)+12>>2]|0;return(1<<((b|0)%(d|0)|0)&c[(c[a+4>>2]|0)+(((b|0)/(d|0)|0)<<2)>>2]|0)!=0|0}function Lb(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=Kb(b,d)|0;i=Kb(a,d)|0;j=h-i|0;c[e>>2]=1;c[f>>2]=0;if((h|0)==(i|0)){k=0;return k|0}if((d|0)==0){k=j;return k|0}i=(g<<1)-1|0;g=(j|0)>0?1:-1;h=d;d=j;while(1){l=h-1|0;if((l|0)<=0){m=g;n=d;break}c[e>>2]=(c[e>>2]|0)+1;j=Kb(b,l)|0;o=j-(Kb(a,l)|0)|0;j=((o|0)>0|g<<1)+(o>>31)|0;if((((j|0)>-1?j:-j|0)|0)>(i|0)){m=j;n=o;break}else{g=j;h=l;d=o}}if((m&1|0)!=0&(l|0)!=0){if((n|0)>0){p=(m-1|0)/2|0}else{p=m}if((n|0)<0){q=(p+1|0)/2|0}else{q=p}c[e>>2]=(c[e>>2]|0)-1;r=q}else{r=m}if((r&1|0)==0){s=r}else{k=r;return k|0}while(1){r=(s|0)/2|0;c[f>>2]=(c[f>>2]|0)+1;c[e>>2]=(c[e>>2]|0)-1;if((r&1|0)==0){s=r}else{k=r;break}}return k|0}function Mb(){var a=0,b=0,d=0,e=0,f=0;a=c[1414]|0;if((c[a+552>>2]|0)!=0){b=0;return b|0}d=a+28|0;c[d>>2]=(c[d>>2]|0)+1;d=c[1414]|0;a=c[d+28>>2]|0;do{if((a|0)<24){c[d+32+(a<<2)>>2]=96;e=c[1414]|0;if((c[e+576>>2]|0)==0){f=e;break}hb();f=c[1414]|0}else{f=d}}while(0);d=ic(((((c[f+24>>2]<<2)+11|0)>>>2)*12|0)+35&-4,1)|0;f=d+16|0;c[d+4>>2]=qb(f,0)|0;c[d+8>>2]=qb(f,1)|0;c[d+12>>2]=qb(f,2)|0;c[d>>2]=2;f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;b=d;return b|0}function Nb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=a&3;f=(((d<<2)+15|0)>>>2)+1|0;if((c[(c[1414]|0)+336>>2]|0)==1){g=(f<<3|4)+15|0}else{g=(f*12|0)+19|0}f=(aa(b<<2,(g>>>2)+1|0)|0)+((e|0)==0?0:4-e|0)|0;e=a+f|0;g=e;b=a+(f+16)|0;c[a+(f+4)>>2]=pb(b,0,d)|0;c[a+(f+8)>>2]=pb(b,1,d)|0;if((c[(c[1414]|0)+336>>2]|0)==1){h=e;c[h>>2]=2;return g|0}c[a+(f+12)>>2]=pb(b,2,d)|0;h=e;c[h>>2]=2;return g|0}function Ob(a,b){a=a|0;b=b|0;var d=0,e=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}e=Nb(a,b,(c[d+24>>2]|0)-1|0)|0;return e|0}function Pb(a){a=a|0;var b=0,d=0,e=0;b=c[1414]|0;d=(((c[b+24>>2]<<2)+11|0)>>>2)+1|0;if((c[b+336>>2]|0)==1){e=ic((aa(a<<2,(((d<<3|4)+15|0)>>>2)+1|0)|0)+4|0,1)|0;return e|0}else{e=ic((aa(a<<2,(((d*12|0)+19|0)>>>2)+1|0)|0)+4|0,1)|0;return e|0}return 0}function Qb(a,b){a=a|0;b=b|0;var d=0,e=0;if((a|0)==0){return}d=c[1414]|0;e=(((c[d+24>>2]<<2)+11|0)>>>2)+1|0;if((c[d+336>>2]|0)==1){od(a|0,0,(aa(b<<2,(((e<<3|4)+15|0)>>>2)+1|0)|0)+4|0)|0}else{od(a|0,0,(aa(b<<2,(((e*12|0)+19|0)>>>2)+1|0)|0)+4|0)|0}jc(a);return}function Rb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=c[a>>2]|0;f=c[b>>2]|0;do{if((f|0)>(e|0)){if((a|0)==(d|0)){g=f;h=f;break}Bb(b,d);g=f;h=e}else{if((b|0)==(d|0)){g=e;h=e;break}Bb(a,d);g=e;h=f}}while(0);f=d|0;c[f>>2]=g;e=c[a+4>>2]|0;a=c[b+4>>2]|0;b=c[d+4>>2]|0;d=c[1414]|0;if((g|0)<(c[d+24>>2]|0)){i=8}else{if((c[d+128>>2]|0)==0){i=8}else{j=d}}if((i|0)==8){c[f>>2]=g+1;j=c[1414]|0}i=c[j>>2]|0;j=(h|0)>0;a:do{if((i|0)==0){if(j){k=0;l=0}else{break}while(1){d=c[e+(l<<2)>>2]|0;m=d+k+(c[a+(l<<2)>>2]|0)|0;if(m>>>0>d>>>0){n=0}else{n=m>>>0<d>>>0?1:k}c[b+(l<<2)>>2]=m;m=l+1|0;if((m|0)<(h|0)){k=n;l=m}else{break}}b:do{if((h|0)<(g|0)&(n|0)!=0){m=n;d=h;while(1){o=c[e+(d<<2)>>2]|0;p=o+m+(c[a+(d<<2)>>2]|0)|0;if(p>>>0>o>>>0){break}q=p>>>0<o>>>0?1:m;c[b+(d<<2)>>2]=p;o=d+1|0;if((o|0)<(g|0)&(q|0)!=0){m=q;d=o}else{r=q;s=o;break b}}c[b+(d<<2)>>2]=p;break a}else{r=n;s=h}}while(0);if((r|0)==0){break}m=c[1414]|0;do{if((c[m+128>>2]|0)!=0){if((s|0)<(c[m+24>>2]|0)){break}fb(3);return}}while(0);c[b+(s<<2)>>2]=r}else{if(j){t=0;u=0;v=i}else{break}while(1){m=(c[e+(u<<2)>>2]|0)+t+(c[a+(u<<2)>>2]|0)|0;w=m>>>0<v>>>0;x=w&1^1;c[b+(u<<2)>>2]=m-(w?0:v);m=u+1|0;if((m|0)>=(h|0)){break}t=x;u=m;v=c[c[1414]>>2]|0}if((h|0)<(g|0)&(w^1)){m=x;o=h;while(1){q=(c[e+(o<<2)>>2]|0)+m+(c[a+(o<<2)>>2]|0)|0;y=c[c[1414]>>2]|0;z=q>>>0<y>>>0;A=z&1^1;c[b+(o<<2)>>2]=q-(z?0:y);y=o+1|0;if((y|0)<(g|0)&(z^1)){m=A;o=y}else{B=A;C=y;break}}}else{B=x;C=h}if((B|0)==0){break}o=c[1414]|0;do{if((c[o+128>>2]|0)!=0){if((C|0)<(c[o+24>>2]|0)){break}fb(3);return}}while(0);c[b+(C<<2)>>2]=B}}while(0);B=(c[f>>2]|0)-1|0;if((c[b+(B<<2)>>2]|0)!=0){return}c[f>>2]=B;return}function Sb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=c[a>>2]|0;f=c[b>>2]|0;if((f|0)>(e|0)){fb(4);return}if((b|0)==(d|0)){g=e}else{Bb(a,d);g=f}c[d>>2]=e;f=c[a+4>>2]|0;a=c[b+4>>2]|0;b=c[d+4>>2]|0;h=(g|0)>0;a:do{if((c[c[1414]>>2]|0)==0){if(h){i=0;j=0}else{break}while(1){if((j|0)>(e|0)){break}k=c[f+(j<<2)>>2]|0;l=k-(c[a+(j<<2)>>2]|0)-i|0;if(l>>>0<k>>>0){m=0}else{m=l>>>0>k>>>0?1:i}c[b+(j<<2)>>2]=l;l=j+1|0;if((l|0)<(g|0)|(m|0)!=0){i=m;j=l}else{break a}}fb(4);return}else{if(h){n=0;o=0}else{break}while(1){if((o|0)>(e|0)){break}l=(c[a+(o<<2)>>2]|0)+n|0;k=c[f+(o<<2)>>2]|0;if(k>>>0<l>>>0){p=1;q=(c[c[1414]>>2]|0)+k|0}else{p=0;q=k}c[b+(o<<2)>>2]=q-l;l=o+1|0;if((l|0)<(g|0)|(p|0)!=0){n=p;o=l}else{break a}}fb(4);return}}while(0);rb(d);return}function Tb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=27;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);Xb(a,1,b,d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Ub(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=28;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);Xb(a,-1,b,d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Vb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=7;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);nb(b,c[h+464>>2]|0);Xb(a,1,c[(c[1414]|0)+464>>2]|0,d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Wb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=8;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);nb(b,c[h+464>>2]|0);Xb(a,-1,c[(c[1414]|0)+464>>2]|0,d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Xb(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0;do{if((Db(a)|0)==0){if((Db(d)|0)!=0){break}f=wb(a)|0;g=wb(d)|0;h=a|0;c[h>>2]=c[h>>2]&2147483647;i=d|0;c[i>>2]=c[i>>2]&2147483647;j=Gb(a,d)|0;k=f+1+(((aa(g,b)|0)+1|0)/2|0)|0;do{if((k|0)==0){if((j|0)>-1){Rb(a,d,e);l=17;break}else{Rb(d,a,e);l=17;break}}else if((k|0)==2){if((j|0)>-1){Sb(a,d,e);break}else{Sb(d,a,e);l=17;break}}else if((k|0)==1){if((j|0)<1){Sb(d,a,e);break}else{Sb(a,d,e);l=17;break}}else if((k|0)==3){if((j|0)>-1){Rb(a,d,e);break}else{Rb(d,a,e);break}}}while(0);if((l|0)==17){j=e|0;c[j>>2]=c[j>>2]^-2147483648}if((a|0)!=(e|0)&(f|0)<0){c[h>>2]=c[h>>2]^-2147483648}if((d|0)==(e|0)){return}if(!((d|0)!=(a|0)&(g|0)<0)){return}c[i>>2]=c[i>>2]^-2147483648;return}}while(0);fb(12);return}function Yb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;do{if((a|0)==(d|0)){if((b|0)!=0){break}lb(a);return}else{lb(d);if((b|0)!=0){break}return}}while(0);e=c[a>>2]|0;f=e&-2147483648;g=e&2147483647;e=c[1414]|0;a:do{if((c[e>>2]|0)==0){h=a+4|0;i=d+4|0;do{if((g|0)!=0){j=0;k=0;do{j=_c(c[(c[h>>2]|0)+(k<<2)>>2]|0,b,j,(c[i>>2]|0)+(k<<2)|0)|0;k=k+1|0;}while((k|0)<(g|0));if((j|0)==0){break}k=c[1414]|0;do{if((g|0)>=(c[k+24>>2]|0)){if((c[k+128>>2]|0)==0){break}fb(3);return}}while(0);c[(c[i>>2]|0)+(g<<2)>>2]=j;k=g+1|0;c[d>>2]=k;l=k;break a}}while(0);c[d>>2]=g;m=g;n=22}else{if((g|0)==0){m=c[d>>2]|0;n=22;break}i=a+4|0;h=d+4|0;k=d|0;o=0;p=0;q=e;while(1){if((p|0)>(c[q+24>>2]|0)){if((c[q+128>>2]|0)!=0){break}}r=Yc(c[(c[i>>2]|0)+(p<<2)>>2]|0,b,o,c[q>>2]|0,(c[h>>2]|0)+(p<<2)|0)|0;s=p+1|0;c[k>>2]=s;if(!((s|0)<(g|0)|(r|0)!=0)){m=s;n=22;break a}o=r;p=s;q=c[1414]|0}fb(3);return}}while(0);do{if((n|0)==22){if((m|0)!=0){l=m;break}return}}while(0);c[d>>2]=l|f;return}function Zb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=9;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Db(a)|0)!=0){fb(12);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}if((b|0)==0){lb(d);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}else if((b|0)==1){Bb(a,d);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}else{do{if((b|0)<0){Yb(a,-b|0,d);e=d|0;f=c[e>>2]|0;if((f|0)==0){break}c[e>>2]=f^-2147483648}else{Yb(a,b,d)}}while(0);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}}function _b(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=i;i=i+8|0;f=e|0;c[f>>2]=0;g=a|0;h=c[g>>2]&2147483647;if((a|0)!=(d|0)){lb(d)}j=c[c[1414]>>2]|0;a:do{if((j|0)==0){k=a+4|0;l=d+4|0;if((h|0)==0){break}else{m=h}do{m=m-1|0;n=Zc(c[f>>2]|0,c[(c[k>>2]|0)+(m<<2)>>2]|0,b,f)|0;c[(c[l>>2]|0)+(m<<2)>>2]=n;}while((m|0)>0)}else{if((h|0)==0){break}l=a+4|0;k=d+4|0;n=h;o=j;while(1){p=n-1|0;q=Yc(c[f>>2]|0,o,c[(c[l>>2]|0)+(p<<2)>>2]|0,b,f)|0;c[(c[k>>2]|0)+(p<<2)>>2]=q;if((p|0)<=0){break a}n=p;o=c[c[1414]>>2]|0}}}while(0);c[d>>2]=c[g>>2];rb(d);i=e;return c[f>>2]|0}function $b(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){f=0;return f|0}g=e+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;e=c[g+28>>2]|0;do{if((e|0)<24){c[g+32+(e<<2)>>2]=10;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Db(a)|0)!=0){fb(12)}if((b|0)==0){fb(2)}e=c[1414]|0;if((c[e+552>>2]|0)!=0){g=e+28|0;c[g>>2]=(c[g>>2]|0)-1;f=0;return f|0}g=c[a>>2]|0;if((g|0)==0){lb(d);h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;f=0;return f|0}if((b|0)==1){Bb(a,d);h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;f=0;return f|0}h=g&-2147483648;do{if((b|0)==2){if((c[e>>2]|0)!=0){i=27;break}Bb(a,d);g=c[d>>2]|0;j=(g&2147483647)-1|0;k=d+4|0;l=c[k>>2]|0;m=c[l>>2]|0;n=m&1;c[l>>2]=m>>>1;if((j|0)==0){o=0}else{m=g&2147483647;g=0;while(1){l=g+1|0;p=c[k>>2]|0;q=p+(g<<2)|0;c[q>>2]=c[q>>2]|c[p+(l<<2)>>2]<<31;p=(c[k>>2]|0)+(l<<2)|0;c[p>>2]=(c[p>>2]|0)>>>1;if((l|0)==(j|0)){break}else{g=l}}o=m-1|0}if((c[(c[k>>2]|0)+(o<<2)>>2]|0)!=0){r=c[1414]|0;s=r+28|0;t=c[s>>2]|0;u=t-1|0;c[s>>2]=u;v=(h|0)==0;w=-n|0;x=v?n:w;return x|0}rb(d);r=c[1414]|0;s=r+28|0;t=c[s>>2]|0;u=t-1|0;c[s>>2]=u;v=(h|0)==0;w=-n|0;x=v?n:w;return x|0}else{if((b|0)>=0){i=27;break}g=_b(a,-b|0,d)|0;j=d|0;l=c[j>>2]|0;if((l|0)==0){y=g;break}c[j>>2]=l^-2147483648;y=g}}while(0);if((i|0)==27){y=_b(a,b,d)|0}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;f=(h|0)==0?y:-y|0;return f|0}function ac(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}f=d+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;d=c[f+28>>2]|0;do{if((d|0)<24){c[f+32+(d<<2)>>2]=88;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);d=c[a>>2]&-2147483648;do{if((b|0)==8){f=c[1414]|0;if((c[f>>2]&7|0)!=0){g=f;break}h=f+28|0;c[h>>2]=(c[h>>2]|0)-1;h=c[c[a+4>>2]>>2]&7;e=(d|0)==0?h:-h|0;return e|0}else if((b|0)==2){h=c[1414]|0;if((c[h>>2]&1|0)!=0){g=h;break}f=h+28|0;c[f>>2]=(c[f>>2]|0)-1;if((c[c[a+4>>2]>>2]&1|0)==0){e=0;return e|0}e=(d>>>30^2)-1|0;return e|0}else{g=c[1414]|0}}while(0);Bb(a,c[g+464>>2]|0);g=c[(c[1414]|0)+464>>2]|0;a=$b(g,b,g)|0;g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;e=a;return e|0}function bc(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;g=c[1414]|0;if((c[g+552>>2]|0)!=0){return}h=g+28|0;c[h>>2]=(c[h>>2]|0)+1;h=c[1414]|0;g=c[h+28>>2]|0;do{if((g|0)<24){c[h+32+(g<<2)>>2]=140;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);lb(f);if((b|0)<1){g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}else{i=b;j=e}while(1){k=a[j]|0;if(k<<24>>24!=0){break}e=i-1|0;if((e|0)==0){l=9;break}else{i=e;j=j+1|0}}if((l|0)==9){l=(c[1414]|0)+28|0;c[l>>2]=(c[l>>2]|0)-1;return}l=c[1414]|0;a:do{if((c[l>>2]|0)==0){e=(i|0)/4|0;b=(i|0)%4|0;g=(b|0)!=0;b:do{if(g){h=e+1|0;if((b|0)>0){m=1;n=j;o=0;p=k}else{q=0;r=j;s=h;break}while(1){t=n+1|0;u=p&255|o;if((m|0)>=(b|0)){q=u;r=t;s=h;break b}m=m+1|0;n=t;o=u<<8;p=a[t]|0}}else{q=0;r=j;s=e}}while(0);c[f>>2]=s;e=c[1414]|0;do{if((s|0)>(c[e+24>>2]|0)){if((c[e+128>>2]|0)==0){break}fb(3);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}}while(0);if(g){e=s-1|0;c[(c[f+4>>2]|0)+(e<<2)>>2]=q;v=e}else{v=s}if((v|0)>0){e=f+4|0;b=r;h=q;t=v;while(1){u=t-1|0;w=d[b+3|0]|0|(d[b+2|0]|0|(d[b+1|0]|0|(d[b]|0|h<<8)<<8)<<8)<<8;c[(c[e>>2]|0)+(u<<2)>>2]=w;if((u|0)>0){b=b+4|0;h=w;t=u}else{break}}}rb(f)}else{if((i|0)>0){x=0;y=l}else{break}while(1){if((c[y+552>>2]|0)!=0){break a}Zb(f,256,f);Vb(f,d[j+x|0]|0,f);t=x+1|0;if((t|0)>=(i|0)){break a}x=t;y=c[1414]|0}}}while(0);y=(c[1414]|0)+28|0;c[y>>2]=(c[y>>2]|0)-1;return}function cc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=i;i=i+8|0;e=d|0;f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;g=c[f+28>>2]|0;do{if((g|0)<24){c[f+32+(g<<2)>>2]=4;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)!=(b|0)){Bb(a,b)}a=c[c[1414]>>2]|0;g=(c[(c[b+4>>2]|0)+((c[b>>2]&2147483647)-1<<2)>>2]|0)+1|0;do{if((a|0)==0){c[e>>2]=g;if((g|0)==0){h=1;break}f=Zc(1,0,g,e)|0;if((f|0)==1){h=1;break}Yb(b,f,b);h=f}else{f=(a>>>0)/(g>>>0)|0;if((f|0)==1){h=1;break}Yb(b,f,b);h=f}}while(0);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;i=d;return h|0}function dc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;e=i;i=i+8|0;f=e|0;g=c[1414]|0;if((c[g+552>>2]|0)!=0){i=e;return}h=b|0;do{if((c[h>>2]|0)!=0){j=a|0;if((c[j>>2]|0)==0){break}k=c[g+484>>2]|0;if((k|0)!=(b|0)&(k|0)!=(a|0)&(k|0)==(d|0)){l=d}else{l=c[g+464>>2]|0}k=g+28|0;c[k>>2]=(c[k>>2]|0)+1;k=c[1414]|0;m=c[k+28>>2]|0;do{if((m|0)<24){c[k+32+(m<<2)>>2]=5;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);do{if((Db(a)|0)==0){if((Db(b)|0)!=0){break}m=c[j>>2]|0;k=c[h>>2]|0;n=(k^m)&-2147483648;o=m&2147483647;p=k&2147483647;lb(l);k=c[1414]|0;do{if((c[k+128>>2]|0)!=0){if((p+o|0)<=(c[k+24>>2]|0)){break}fb(3);q=(c[1414]|0)+28|0;c[q>>2]=(c[q>>2]|0)-1;i=e;return}}while(0);a:do{if((c[k>>2]|0)==0){q=a+4|0;r=b+4|0;s=l+4|0;if(!((a|0)==(b|0)&o>>>0>5>>>0)){if((o|0)==0){break}if((p|0)==0){t=0;while(1){c[f>>2]=0;c[(c[s>>2]|0)+(t<<2)>>2]=0;t=t+1|0;if((t|0)>=(o|0)){break a}}}else{u=0}while(1){c[f>>2]=0;t=0;do{$c(c[(c[q>>2]|0)+(u<<2)>>2]|0,c[(c[r>>2]|0)+(t<<2)>>2]|0,f,(c[s>>2]|0)+(t+u<<2)|0);t=t+1|0;}while((t|0)<(p|0));c[(c[s>>2]|0)+(u+p<<2)>>2]=c[f>>2];u=u+1|0;if((u|0)>=(o|0)){break a}}}r=o-1|0;if((r|0)>0){t=0;while(1){c[f>>2]=0;v=t+1|0;if((v|0)<(o|0)){w=v;do{x=c[q>>2]|0;$c(c[x+(t<<2)>>2]|0,c[x+(w<<2)>>2]|0,f,(c[s>>2]|0)+(w+t<<2)|0);w=w+1|0;}while((w|0)<(o|0));y=c[f>>2]|0}else{y=0}c[(c[s>>2]|0)+(t+o<<2)>>2]=y;if((v|0)<(r|0)){t=v}else{break}}}c[l>>2]=(m<<1)-1;Rb(l,l,l);c[f>>2]=0;if((o|0)==0){break}else{z=0}do{t=z<<1;r=c[(c[q>>2]|0)+(z<<2)>>2]|0;$c(r,r,f,(c[s>>2]|0)+(t<<2)|0);r=t|1;t=(c[s>>2]|0)+(r<<2)|0;c[t>>2]=(c[t>>2]|0)+(c[f>>2]|0);c[f>>2]=(c[(c[s>>2]|0)+(r<<2)>>2]|0)>>>0<(c[f>>2]|0)>>>0;z=z+1|0;}while((z|0)<(o|0))}else{if(!((a|0)==(b|0)&o>>>0>5>>>0)){if((o|0)==0){break}s=l+4|0;q=a+4|0;r=b+4|0;if((p|0)==0){t=0;while(1){c[f>>2]=0;c[(c[s>>2]|0)+(t<<2)>>2]=0;t=t+1|0;if((t|0)>=(o|0)){break a}}}else{A=0}while(1){c[f>>2]=0;t=0;w=0;do{x=(c[s>>2]|0)+(t+A<<2)|0;w=Yc(c[(c[q>>2]|0)+(A<<2)>>2]|0,c[(c[r>>2]|0)+(t<<2)>>2]|0,w+(c[x>>2]|0)|0,c[c[1414]>>2]|0,x)|0;c[f>>2]=w;t=t+1|0;}while((t|0)<(p|0));c[(c[s>>2]|0)+(A+p<<2)>>2]=w;A=A+1|0;if((A|0)>=(o|0)){break a}}}s=o-1|0;if((s|0)>0){r=l+4|0;q=b+4|0;t=0;while(1){c[f>>2]=0;v=t+1|0;if((v|0)<(o|0)){x=v;B=0;while(1){C=c[q>>2]|0;D=(c[r>>2]|0)+(x+t<<2)|0;E=Yc(c[C+(t<<2)>>2]|0,c[C+(x<<2)>>2]|0,B+(c[D>>2]|0)|0,c[c[1414]>>2]|0,D)|0;c[f>>2]=E;D=x+1|0;if((D|0)<(o|0)){x=D;B=E}else{F=E;break}}}else{F=0}c[(c[r>>2]|0)+(t+o<<2)>>2]=F;if((v|0)<(s|0)){t=v}else{break}}}c[l>>2]=(m<<1)-1;Rb(l,l,l);c[f>>2]=0;if((o|0)==0){break}t=b+4|0;s=l+4|0;r=0;q=0;while(1){B=r<<1;x=c[(c[t>>2]|0)+(r<<2)>>2]|0;w=(c[s>>2]|0)+(B<<2)|0;E=Yc(x,x,q+(c[w>>2]|0)|0,c[c[1414]>>2]|0,w)|0;c[f>>2]=E;w=B|1;B=(c[s>>2]|0)+(w<<2)|0;c[B>>2]=(c[B>>2]|0)+E;c[f>>2]=0;E=c[1414]|0;if((c[(c[s>>2]|0)+(w<<2)>>2]|0)>>>0>=(c[E>>2]|0)>>>0){c[f>>2]=1;B=(c[s>>2]|0)+(w<<2)|0;c[B>>2]=(c[B>>2]|0)-(c[E>>2]|0)}E=r+1|0;if((E|0)>=(o|0)){break a}r=E;q=c[f>>2]|0}}}while(0);c[l>>2]=n|p+o;rb(l);Bb(l,d);m=(c[1414]|0)+28|0;c[m>>2]=(c[m>>2]|0)-1;i=e;return}}while(0);fb(12);j=(c[1414]|0)+28|0;c[j>>2]=(c[j>>2]|0)-1;i=e;return}}while(0);lb(d);i=e;return}function ec(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ba=0,ca=0,da=0,ea=0;e=i;i=i+24|0;f=e|0;g=e+8|0;h=e+16|0;j=c[1414]|0;if((c[j+552>>2]|0)!=0){i=e;return}k=c[j+464>>2]|0;l=j+28|0;c[l>>2]=(c[l>>2]|0)+1;l=c[1414]|0;j=c[l+28>>2]|0;do{if((j|0)<24){c[l+32+(j<<2)>>2]=6;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)==(b|0)){fb(7)}if((Db(a)|0)==0){if((Db(b)|0)!=0){m=9}}else{m=9}if((m|0)==9){fb(12)}j=b|0;if((c[j>>2]|0)==0){fb(2)}l=c[1414]|0;if((c[l+552>>2]|0)!=0){n=l+28|0;c[n>>2]=(c[n>>2]|0)-1;i=e;return}n=a|0;l=c[n>>2]|0;o=l&-2147483648;p=c[j>>2]&-2147483648;q=p^o;c[n>>2]=l&2147483647;l=c[j>>2]&2147483647;c[j>>2]=l;r=c[n>>2]|0;Bb(a,k);s=k|0;t=c[s>>2]|0;u=c[1414]|0;do{if((c[u+128>>2]|0)!=0){if((t-l|0)<(c[u+24>>2]|0)){break}fb(3);v=(c[1414]|0)+28|0;c[v>>2]=(c[v>>2]|0)-1;i=e;return}}while(0);do{if((r|0)==(l|0)){if((l|0)==1){u=c[k+4>>2]|0;v=c[u>>2]|0;w=c[c[b+4>>2]>>2]|0;c[u>>2]=(v>>>0)%(w>>>0)|0;rb(k);x=(v>>>0)/(w>>>0)|0;break}w=l-1|0;if((c[(c[k+4>>2]|0)+(w<<2)>>2]|0)>>>2>>>0>=(c[(c[b+4>>2]|0)+(w<<2)>>2]|0)>>>0){x=0;break}if((Gb(k,b)|0)>-1){y=0}else{x=0;break}while(1){Sb(k,b,k);w=y+1|0;if((Gb(k,b)|0)>-1){y=w}else{x=w;break}}}else{x=0}}while(0);if((Gb(k,b)|0)<0){do{if((a|0)!=(d|0)){Bb(k,a);y=c[n>>2]|0;if((y|0)==0){break}c[n>>2]=y|o}}while(0);do{if((b|0)!=(d|0)){lb(d);c[c[d+4>>2]>>2]=x;if((x|0)==0){break}c[d>>2]=q|1}}while(0);c[j>>2]=c[j>>2]|p;x=(c[1414]|0)+28|0;c[x>>2]=(c[x>>2]|0)-1;i=e;return}if((l|0)==1){c[g>>2]=_b(k,c[c[b+4>>2]>>2]|0,k)|0;if((b|0)!=(d|0)){Bb(k,d);x=d|0;c[x>>2]=c[x>>2]|q}do{if((a|0)!=(d|0)){lb(a);c[c[a+4>>2]>>2]=c[g>>2];if((c[g>>2]|0)==0){break}c[n>>2]=o|1}}while(0);c[j>>2]=c[j>>2]|p;x=(c[1414]|0)+28|0;c[x>>2]=(c[x>>2]|0)-1;i=e;return}x=(b|0)!=(d|0);if(x){lb(d)}y=cc(b,b)|0;r=(c[1414]|0)+128|0;w=c[r>>2]|0;c[r>>2]=0;r=(y|0)!=1;a:do{if((c[c[1414]>>2]|0)==0){if(r){Yb(k,y,k)}v=l-1|0;u=b+4|0;z=c[u>>2]|0;A=c[z+(v<<2)>>2]|0;B=c[z+(l-2<<2)>>2]|0;z=k+4|0;if((t|0)<=(v|0)){C=t;break}D=d+4|0;F=(l|0)==0;G=t;H=t;while(1){I=H;while(1){J=I-1|0;K=c[z>>2]|0;L=c[K+(I<<2)>>2]|0;M=c[K+(J<<2)>>2]|0;do{if((L|0)==(A|0)){K=Ba(A|0,M|0)|0;c[f>>2]=K;if(!E){N=-1;m=50;break}O=-1;P=J-l+1|0;m=56}else{N=Zc(L,M,A,f)|0;m=50}}while(0);if((m|0)==50){m=0;M=I-2|0;L=N;while(1){K=_c(B,L,0,g)|0;Q=c[f>>2]|0;if(K>>>0<Q>>>0){R=L;break}if((K|0)==(Q|0)){if((c[g>>2]|0)>>>0<=(c[(c[z>>2]|0)+(M<<2)>>2]|0)>>>0){R=L;break}}K=L-1|0;S=Ba(Q|0,A|0)|0;c[f>>2]=S;if(E){R=K;break}else{L=K}}L=J-l+1|0;if((R|0)==0){T=0;U=L}else{O=R;P=L;m=56}}do{if((m|0)==56){m=0;if(F){V=0}else{L=0;M=0;while(1){K=_c(O,c[(c[u>>2]|0)+(M<<2)>>2]|0,L,h)|0;S=(c[z>>2]|0)+(M+P<<2)|0;Q=c[S>>2]|0;W=c[h>>2]|0;X=(Q>>>0<W>>>0)+K|0;c[S>>2]=Q-W;W=M+1|0;if((W|0)<(l|0)){L=X;M=W}else{V=X;break}}}M=(c[z>>2]|0)+(I<<2)|0;L=c[M>>2]|0;if(L>>>0>=V>>>0){c[M>>2]=L-V;T=O;U=P;break}c[M>>2]=0;if(!F){M=0;L=0;while(1){X=(c[z>>2]|0)+(M+P<<2)|0;W=c[(c[u>>2]|0)+(M<<2)>>2]|0;Q=Ba((c[X>>2]|0)+L|0,W|0)|0;S=Q;c[X>>2]=S;X=M+1|0;if((X|0)<(l|0)){M=X;L=E?1:S>>>0>W>>>0?0:L}else{break}}}T=O-1|0;U=P}}while(0);if((I|0)==(G|0)&(T|0)==0){break}if(x){c[(c[D>>2]|0)+(U<<2)>>2]=T}if((J|0)>(v|0)){I=J}else{C=G;break a}}I=G-1|0;if((J|0)>(v|0)){G=I;H=J}else{C=I;break}}}else{if(r){Yb(k,y,k)}H=l-1|0;G=b+4|0;v=c[G>>2]|0;D=c[v+(H<<2)>>2]|0;u=c[v+(l-2<<2)>>2]|0;if((t|0)<=(H|0)){C=t;break}v=k+4|0;z=d+4|0;F=(l|0)==0;A=t;B=t;while(1){I=B;while(1){Y=I-1|0;L=c[v>>2]|0;M=c[L+(I<<2)>>2]|0;W=c[1414]|0;S=c[W>>2]|0;if((M|0)==(D|0)){X=(c[L+(Y<<2)>>2]|0)+D|0;c[f>>2]=X;Z=S-1|0;_=X;$=W}else{W=Yc(M,S,c[L+(Y<<2)>>2]|0,D,f)|0;Z=W;_=c[f>>2]|0;$=c[1414]|0}W=c[$>>2]|0;b:do{if(_>>>0<W>>>0){L=I-2|0;S=Z;M=W;while(1){X=Yc(u,S,0,M,g)|0;Q=c[f>>2]|0;if(X>>>0<Q>>>0){aa=S;break b}if((X|0)==(Q|0)){if((c[g>>2]|0)>>>0<=(c[(c[v>>2]|0)+(L<<2)>>2]|0)>>>0){aa=S;break b}}X=S-1|0;K=Q+D|0;c[f>>2]=K;Q=c[c[1414]>>2]|0;if(K>>>0<Q>>>0){S=X;M=Q}else{aa=X;break}}}else{aa=Z}}while(0);W=Y-l+1|0;do{if((aa|0)==0){ba=0}else{if(F){ca=0}else{M=0;S=0;while(1){L=Yc(aa,c[(c[G>>2]|0)+(S<<2)>>2]|0,M,c[c[1414]>>2]|0,h)|0;X=(c[v>>2]|0)+(S+W<<2)|0;Q=c[X>>2]|0;K=c[h>>2]|0;if(Q>>>0<K>>>0){da=L+1|0;ea=Q-K+(c[c[1414]>>2]|0)|0}else{da=L;ea=Q-K|0}c[X>>2]=ea;X=S+1|0;if((X|0)<(l|0)){M=da;S=X}else{ca=da;break}}}S=(c[v>>2]|0)+(I<<2)|0;M=c[S>>2]|0;if(M>>>0>=ca>>>0){c[S>>2]=M-ca;ba=aa;break}c[S>>2]=0;if(!F){S=0;M=0;while(1){X=(c[v>>2]|0)+(S+W<<2)|0;K=(c[X>>2]|0)+M+(c[(c[G>>2]|0)+(S<<2)>>2]|0)|0;Q=c[c[1414]>>2]|0;L=K>>>0<Q>>>0;c[X>>2]=K-(L?0:Q);Q=S+1|0;if((Q|0)<(l|0)){S=Q;M=L&1^1}else{break}}}ba=aa-1|0}}while(0);if((I|0)==(A|0)&(ba|0)==0){break}if(x){c[(c[z>>2]|0)+(W<<2)>>2]=ba}if((Y|0)>(H|0)){I=Y}else{C=A;break a}}I=A-1|0;if((Y|0)>(H|0)){A=I;B=Y}else{C=I;break}}}}while(0);if(x){c[d>>2]=1-l+C|q}c[s>>2]=l;rb(b);rb(d);do{if((a|0)!=(d|0)){rb(k);if(r){_b(k,y,a)|0}else{Bb(k,a)}l=c[n>>2]|0;if((l|0)==0){break}c[n>>2]=l|o}}while(0);if(r){_b(b,y,b)|0}c[j>>2]=c[j>>2]|p;c[(c[1414]|0)+128>>2]=w;w=(c[1414]|0)+28|0;c[w>>2]=(c[w>>2]|0)-1;i=e;return}function fc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}f=d+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;d=c[f+28>>2]|0;do{if((d|0)<24){c[f+32+(d<<2)>>2]=87;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);Bb(a,c[h+464>>2]|0);ec(c[(c[1414]|0)+464>>2]|0,b,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;e=(Fb(c[(c[1414]|0)+464>>2]|0)|0)==0|0;return e|0}function gc(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;h=c[1414]|0;if((c[h+552>>2]|0)!=0){return}i=h+28|0;c[i>>2]=(c[i>>2]|0)+1;i=c[1414]|0;h=c[i+28>>2]|0;do{if((h|0)<24){c[i+32+(h<<2)>>2]=24;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((e|0)==(g|0)){fb(7);h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;return}h=(c[1414]|0)+128|0;i=c[h>>2]|0;c[h>>2]=0;dc(a,b,c[(c[1414]|0)+464>>2]|0);if(!((a|0)==(d|0)|(b|0)==(d|0))){b=c[(c[1414]|0)+464>>2]|0;Tb(b,d,b)}ec(c[(c[1414]|0)+464>>2]|0,e,f);if((f|0)!=(g|0)){Bb(c[(c[1414]|0)+464>>2]|0,g)}c[(c[1414]|0)+128>>2]=i;i=(c[1414]|0)+28|0;c[i>>2]=(c[i>>2]|0)-1;return}function hc(){return nd(1,728)|0}function ic(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[1414]|0;do{if((d|0)==0){e=nd(a,b)|0}else{if((c[d+552>>2]|0)!=0){e=0;break}f=nd(a,b)|0;if((f|0)!=0){e=f;break}fb(8);e=0}}while(0);return e|0}function jc(a){a=a|0;if((a|0)==0){return}md(a);return}function kc(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ba=0;f=c[1414]|0;if((c[f+552>>2]|0)!=0){g=0;return g|0}h=f+28|0;c[h>>2]=(c[h>>2]|0)+1;h=c[1414]|0;f=c[h+28>>2]|0;do{if((f|0)<24){c[h+32+(f<<2)>>2]=76;i=c[1414]|0;if((c[i+576>>2]|0)==0){j=i;break}hb();j=c[1414]|0}else{j=h}}while(0);if(((c[j+4>>2]|0)-1|0)>>>0>255>>>0){fb(1);h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;g=0;return g|0}if((c[j+140>>2]|0)==0){fb(18);j=(c[1414]|0)+28|0;c[j>>2]=(c[j>>2]|0)-1;g=0;return g|0}lb(b);j=c[1414]|0;h=(c[j+136>>2]|0)==0;if(h){k=e}else{k=c[j+584>>2]|0}e=c[j+560>>2]|0;a:do{if((e|0)==0){do{if(h){l=j;m=0}else{f=0;i=j;while(1){n=na(c[i+144>>2]|0)|0;if((n|0)==-1){o=f;p=17;break}q=f+1|0;a[k+f|0]=n;r=c[1414]|0;if((q|0)>=(c[r+544>>2]|0)){break}if((n|0)==10|(n|0)==0){o=q;p=17;break}else{f=q;i=r}}if((p|0)==17){a[k+o|0]=0;i=c[1414]|0;l=i;m=c[i+560>>2]|0;break}fb(17);i=(c[1414]|0)+28|0;c[i>>2]=(c[i>>2]|0)-1;g=0;return g|0}}while(0);i=a[k+m|0]|0;if(i<<24>>24==0){s=l;break}else{t=l;u=m;v=i}while(1){if((c[t+4>>2]|0)>>>0<61>>>0&v<<24>>24==10){s=t;break a}c[t+560>>2]=u+1;i=c[1414]|0;f=c[i+560>>2]|0;if((k|0)==(c[i+584>>2]|0)){if((f|0)>=(c[i+544>>2]|0)){break}}r=a[k+f|0]|0;if(r<<24>>24==0){s=i;break a}else{t=i;u=f;v=r}}fb(17);r=(c[1414]|0)+28|0;c[r>>2]=(c[r>>2]|0)-1;g=0;return g|0}else{do{if((k|0)==(c[j+584>>2]|0)){if((e|0)<(c[j+544>>2]|0)){break}fb(17);r=(c[1414]|0)+28|0;c[r>>2]=(c[r>>2]|0)-1;g=0;return g|0}}while(0);if((e|0)>0&(h^1)){w=0;x=j}else{s=j;break}while(1){r=na(c[x+144>>2]|0)|0;if((r|0)==-1){break}a[k+w|0]=r;r=w+1|0;f=c[1414]|0;if((r|0)<(c[f+560>>2]|0)){w=r;x=f}else{s=f;break a}}c[(c[1414]|0)+560>>2]=w;s=c[1414]|0}}while(0);w=s+560|0;x=c[w>>2]|0;do{if((x|0)>0){if((c[s+4>>2]|0)>>>0>=61>>>0){y=0;z=1;A=x;break}j=x-1|0;if((a[k+j|0]|0)==47){c[w>>2]=j;B=0}else{B=0}while(1){j=a[k+B|0]|0;if((j<<24>>24|0)==45){p=37;break}else if((j<<24>>24|0)!=32){C=B;D=1;E=j;break}B=B+1|0}if((p|0)==37){j=B+1|0;C=j;D=-1;E=a[k+j|0]|0}y=(E<<24>>24==43)+C|0;z=D;A=c[(c[1414]|0)+560>>2]|0}else{y=0;z=1;A=x}}while(0);x=0;D=0;C=1;E=0;B=0;w=A;b:while(1){F=B+1|0;A=x;s=D;j=C;h=E;e=w;c:while(1){v=(h|0)>2;G=(h|0)!=0;H=h<<1;u=A;I=s;J=j;t=e;d:while(1){m=u;l=t;e:while(1){K=c[1414]|0;L=K+4|0;if((m|0)==0){M=l}else{N=l;p=44;break}while(1){O=M-1|0;if((M|0)<=(y|0)){P=0;break b}o=a[k+O|0]|0;f=o&255;r=c[L>>2]|0;i=(r|0)==64;if(!(r>>>0<61>>>0|i)){Q=f;R=O;S=0;T=r;break e}if(r>>>0<=60>>>0){if((o<<24>>24|0)==47){p=52;break c}else if((o<<24>>24|0)==46){break}}o=f+80|0;if(!i){U=O;V=f;W=o;X=0;Y=r;p=58;break e}if(o>>>0<113>>>0){M=O}else{Z=O;_=f;$=o;aa=0;p=56;break e}}lb(c[K+464>>2]|0);Ab(1,c[(c[1414]|0)+464>>2]|0,F);m=1;l=O}f:do{if((p|0)==44){while(1){p=0;l=N-1|0;if((N|0)<=(y|0)){P=m;break b}o=d[k+l|0]|0;f=c[L>>2]|0;r=(f|0)==64;if(!(f>>>0<61>>>0|r)){Q=o;R=l;S=m;T=f;break f}i=o+80|0;if(!r){U=l;V=o;W=i;X=m;Y=f;p=58;break f}if(i>>>0<113>>>0){N=l;p=44}else{Z=l;_=o;$=i;aa=m;p=56;break}}}}while(0);do{if((p|0)==56){p=0;m=_-65|0;i=m>>>0<26>>>0?m:$;m=(i-177|0)>>>0<26>>>0?i-151|0:i;i=(m-128|0)>>>0<10>>>0?m-76|0:m;m=(i|0)==123?62:i;i=(m|0)==127?63:m;if((i|0)==141){break d}else{Q=i;R=Z;S=aa;T=64}}else if((p|0)==58){p=0;i=V-48|0;m=i>>>0<10>>>0?i:W;i=(m-145|0)>>>0<26>>>0?m-135|0:m;m=(i-177|0)>>>0<26>>>0;if(Y>>>0<17>>>0){Q=m?i-167|0:i;R=U;S=X;T=Y;break}else{Q=m?i-141|0:i;R=U;S=X;T=Y;break}}}while(0);if(Q>>>0>=T>>>0|v){p=62;break b}if(!(G&(J|0)!=0)){break c}u=S;I=Q>>H;J=0;t=R}A=aa;s=I;j=J;h=h+1|0;e=Z}if((p|0)==52){p=0;Bb(b,c[K+464>>2]|0);lb(b);x=1;D=I;C=J;E=h;B=0;w=O;continue}if(G){Ab((Q<<6-H|I)&63,b,F);x=S;D=Q>>H;C=J;E=h;B=F;w=R;continue}else{Ab(Q,b,F);x=S;D=I;C=J;E=0;B=F;w=R;continue}}if((p|0)==62){fb(5);p=(c[1414]|0)+28|0;c[p>>2]=(c[p>>2]|0)-1;g=0;return g|0}if(G&(I|0)>0){Ab(I,b,F);ba=c[1414]|0}else{ba=K}K=ba+560|0;ba=c[K>>2]|0;c[K>>2]=0;xb(z,b);rb(b);rb(c[(c[1414]|0)+464>>2]|0);if((P|0)!=0){Hb(b,c[(c[1414]|0)+464>>2]|0,b)}b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;g=ba;return g|0}function lc(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){f=0;return f|0}g=e+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;e=c[g+28>>2]|0;do{if((e|0)<24){c[g+32+(e<<2)>>2]=75;h=c[1414]|0;if((c[h+576>>2]|0)==0){i=h;break}hb();i=c[1414]|0}else{i=g}}while(0);if(((c[i+4>>2]|0)-1|0)>>>0>255>>>0){fb(1);g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;f=0;return f|0}if((c[i+140>>2]|0)==0){fb(18);i=(c[1414]|0)+28|0;c[i>>2]=(c[i>>2]|0)-1;f=0;return f|0}i=wb(b)|0;xb(1,b);g=c[b>>2]|0;do{if((g|0)==0){e=c[1414]|0;if((c[e+4>>2]|0)>>>0>=61>>>0){break}if((c[e+132>>2]|0)==0){a[d]=48;a[d+1|0]=0}else{ua(48,c[e+148>>2]|0)|0;ua(10,c[(c[1414]|0)+148>>2]|0)|0}e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;f=1;return f|0}}while(0);do{if((i|0)==-1){e=c[1414]|0;if((c[e+4>>2]|0)>>>0>=61>>>0){j=0;break}if((c[e+132>>2]|0)==0){a[d]=45;j=1;break}else{ua(45,c[e+148>>2]|0)|0;j=1;break}}else{j=0}}while(0);Ib(b,c[(c[1414]|0)+488>>2]|0);e=c[1414]|0;do{if((c[e+572>>2]|0)==0){k=0}else{Jb(b,c[e+484>>2]|0);if((Fb(c[(c[1414]|0)+484>>2]|0)|0)<=1){k=0;break}h=g&65535;l=g>>>16;m=((h|0)==0)+h|0;h=(c[1414]|0)+128|0;n=c[h>>2]|0;c[h>>2]=0;if((l|0)<(m|0)){h=c[(c[1414]|0)+484>>2]|0;Eb(h,m-l|0,h)}if((l|0)>(m|0)){h=c[(c[1414]|0)+488>>2]|0;o=l-m|0;Eb(h,o,h);p=o}else{p=l-m|0}m=c[1414]|0;l=c[m+24>>2]|0;o=l+~((Gb(c[m+488>>2]|0,c[m+484>>2]|0)|0)>>31)|0;m=c[1414]|0;Bb(c[m+488>>2]|0,c[m+464>>2]|0);m=c[1414]|0;l=c[m+464>>2]|0;h=c[l>>2]|0;q=c[m+24>>2]<<1;m=(h+o|0)>(q|0)?q-h|0:o;Eb(l,m,l);l=c[1414]|0;ec(c[l+464>>2]|0,c[l+484>>2]|0,c[l+488>>2]|0);c[(c[1414]|0)+128>>2]=n;k=aa(c[(c[1414]|0)+8>>2]|0,p+m|0)|0}}while(0);p=0;g=j;a:while(1){j=zb(c[(c[1414]|0)+488>>2]|0)|0;e=c[1414]|0;if((c[e+4>>2]|0)==64){m=j;while(1){if((m&3|0)==0){r=m;break}else{m=m+1|0}}}else{r=j}m=(k|0)>(r|0)?k:r;b:do{if((m|0)>0){s=g;n=m;l=e;while(1){o=(c[l+132>>2]|0)==0;do{if(o){if((c[l+584>>2]|0)!=(d|0)){break}if((s|0)>=((c[l+544>>2]|0)-5|0)){t=35;break a}}}while(0);do{if((n|0)==(k|0)){if((c[l+4>>2]|0)>>>0>=61>>>0){u=s;break}if(o){a[d+s|0]=46}else{ua(46,c[l+148>>2]|0)|0}u=s+1|0}else{u=s}}while(0);o=c[1414]|0;if((n|0)>(j|0)){h=c[o+4>>2]|0;if((h|0)==64){t=44}else{v=48;w=o;x=h}}else{t=44}do{if((t|0)==44){t=0;h=yb(c[o+488>>2]|0,n)|0;c[(c[1414]|0)+128>>2]=0;Ab(0,c[(c[1414]|0)+488>>2]|0,n);q=c[1414]|0;y=c[q+4>>2]|0;if(y>>>0<61>>>0){z=h+48|0;A=(z|0)>57?h+55|0:z;v=(A|0)>90?A+6|0:A;w=q;x=y;break}if((y|0)!=64){v=h;w=q;x=y;break}y=(h|0)<26?h+65|0:h;h=(y-26|0)>>>0<26>>>0?y+71|0:y;y=(h-52|0)>>>0<10>>>0?h-4|0:h;h=(y|0)==62?43:y;v=(h|0)==63?47:h;w=q;x=64}}while(0);do{if((n|0)<(k|0)){if(!(x>>>0<61>>>0&(v|0)==48)){B=w;break}if((Fb(c[w+488>>2]|0)|0)==0){C=u;break b}B=c[1414]|0}else{B=w}}while(0);if((c[B+132>>2]|0)==0){a[d+u|0]=v}else{ua(v&255|0,c[B+148>>2]|0)|0}o=u+1|0;q=n-1|0;if((q|0)<=0){C=o;break b}s=o;n=q;l=c[1414]|0}}else{C=g}}while(0);if((p|0)!=0){break}j=c[1414]|0;if((c[j+572>>2]|0)!=0){break}Jb(b,c[j+488>>2]|0);if((Fb(c[(c[1414]|0)+488>>2]|0)|0)==1){break}j=c[1414]|0;if((c[j+4>>2]|0)>>>0>=61>>>0){p=1;g=C;continue}if((c[j+132>>2]|0)==0){a[d+C|0]=47}else{ua(47,c[j+148>>2]|0)|0}p=1;g=C+1|0}if((t|0)==35){fb(17);t=(c[1414]|0)+28|0;c[t>>2]=(c[t>>2]|0)-1;f=s;return f|0}s=c[1414]|0;do{if((c[s+132>>2]|0)==0){a[d+C|0]=0}else{t=c[s+4>>2]|0;if(!(t>>>0<61>>>0|(t|0)==64)){break}ua(10,c[s+148>>2]|0)|0}}while(0);xb(i,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;f=C;return f|0}function mc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}f=d+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;d=c[f+28>>2]|0;do{if((d|0)<24){c[f+32+(d<<2)>>2]=77;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);f=c[h+4>>2]|0;d=c[h>>2]|0;jb(c[h+564>>2]|0)|0;c[(c[1414]|0)+128>>2]=0;nc(a,d,c[(c[1414]|0)+484>>2]|0);c[(c[1414]|0)+128>>2]=1;d=lc(c[(c[1414]|0)+484>>2]|0,b)|0;lb(c[(c[1414]|0)+484>>2]|0);jb(f)|0;f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;e=d;return e|0}function nc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}if((c[e>>2]|0)==(b|0)){Bb(a,d);return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=13;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);e=wb(a)|0;Ib(a,c[(c[1414]|0)+468>>2]|0);Jb(a,c[(c[1414]|0)+472>>2]|0);xb(1,c[(c[1414]|0)+468>>2]|0);a=(b|0)==0;f=0;while(1){lb(c[(c[1414]|0)+488>>2]|0);nb(1,c[(c[1414]|0)+464>>2]|0);g=c[1414]|0;h=c[g+468>>2]|0;if((c[h>>2]|0)>0){if(a){i=0;j=g;k=h;while(1){Yb(c[j+464>>2]|0,c[(c[k+4>>2]|0)+(i<<2)>>2]|0,c[j+484>>2]|0);l=c[1414]|0;m=c[l+488>>2]|0;Tb(m,c[l+484>>2]|0,m);m=ib(1,16)|0;l=c[(c[1414]|0)+464>>2]|0;Yb(l,m,l);l=c[(c[1414]|0)+464>>2]|0;Yb(l,m,l);l=i+1|0;m=c[1414]|0;n=c[m+468>>2]|0;if((l|0)<(c[n>>2]|0)){i=l;j=m;k=n}else{o=m;p=n;break}}}else{k=0;j=g;i=h;while(1){Yb(c[j+464>>2]|0,c[(c[i+4>>2]|0)+(k<<2)>>2]|0,c[j+484>>2]|0);n=c[1414]|0;m=c[n+488>>2]|0;Tb(m,c[n+484>>2]|0,m);m=c[(c[1414]|0)+464>>2]|0;Yb(m,b,m);m=k+1|0;n=c[1414]|0;l=c[n+468>>2]|0;if((m|0)<(c[l>>2]|0)){k=m;j=n;i=l}else{o=n;p=l;break}}}}else{o=g;p=h}if((c[o+552>>2]|f|0)!=0){break}Bb(c[o+472>>2]|0,p);i=c[1414]|0;Bb(c[i+488>>2]|0,c[i+492>>2]|0);f=1}Hb(c[o+492>>2]|0,c[o+488>>2]|0,d);xb(e,d);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function oc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;if((c[(c[1414]|0)+552>>2]|0)!=0){d=0;return d|0}if((Fb(a)|0)==0){d=0;return d|0}if((Fb(b)|0)<1){d=0;return d|0}e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)+1;e=c[1414]|0;f=c[e+28>>2]|0;do{if((f|0)<24){c[e+32+(f<<2)>>2]=3;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=e}}while(0);Bb(b,c[h+472>>2]|0);h=ac(c[(c[1414]|0)+472>>2]|0,8)|0;if((h&1|0)==0){b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;d=0;return d|0}if((Fb(a)|0)<0){Cb(a,c[(c[1414]|0)+468>>2]|0);i=((h|0)%4|0|0)==3?-1:1}else{Bb(a,c[(c[1414]|0)+468>>2]|0);i=1}if((Fb(c[(c[1414]|0)+468>>2]|0)|0)==0){j=i}else{a=h;h=i;while(1){i=(a|0)==3;a:do{if((a|0)==5){b=h;while(1){e=(ac(c[(c[1414]|0)+468>>2]|0,2)|0)==0;f=c[1414]|0;g=c[f+468>>2]|0;if(!e){k=f;l=g;m=b;break a}$b(g,2,g)|0;b=-b|0}}else{b=h;while(1){if(i){g=(ac(c[(c[1414]|0)+468>>2]|0,2)|0)==0;f=c[1414]|0;e=c[f+468>>2]|0;if(!g){k=f;l=e;m=b;break a}$b(e,2,e)|0}else{do{e=(ac(c[(c[1414]|0)+468>>2]|0,2)|0)==0;f=c[1414]|0;g=c[f+468>>2]|0;if(!e){k=f;l=g;m=b;break a}$b(g,2,g)|0}while(!((a|0)==5|(a|0)==3))}b=-b|0}}}while(0);if((Gb(l,c[k+472>>2]|0)|0)<0){i=c[1414]|0;b=i+468|0;g=c[b>>2]|0;c[b>>2]=c[i+472>>2];c[(c[1414]|0)+472>>2]=g;g=ac(c[(c[1414]|0)+472>>2]|0,8)|0;n=((a|0)%4|0|0)==3&((g|0)%4|0|0)==3?-m|0:m;o=g}else{n=m;o=a}g=c[1414]|0;i=c[g+468>>2]|0;Sb(i,c[g+472>>2]|0,i);i=c[(c[1414]|0)+468>>2]|0;$b(i,2,i)|0;if((o|0)==5|(o|0)==3){p=-n|0}else{p=n}if((Fb(c[(c[1414]|0)+468>>2]|0)|0)==0){j=p;break}else{a=o;h=p}}}p=(c[1414]|0)+28|0;c[p>>2]=(c[p>>2]|0)-1;p=(Fb(c[(c[1414]|0)+472>>2]|0)|0)==1;d=p?j:0;return d|0}function pc(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,ba=0,ca=0,da=0,ea=0,fa=0,ga=0,ha=0,ia=0,ja=0,ka=0,la=0,ma=0,na=0,oa=0,pa=0,qa=0,ra=0,sa=0,ta=0,ua=0,va=0,wa=0,xa=0,ya=0,za=0,Aa=0;g=i;i=i+8|0;h=g|0;j=c[1414]|0;if((c[j+552>>2]|0)!=0){k=0;i=g;return k|0}l=j+28|0;c[l>>2]=(c[l>>2]|0)+1;l=c[1414]|0;j=c[l+28>>2]|0;do{if((j|0)<24){c[l+32+(j<<2)>>2]=30;m=c[1414]|0;if((c[m+576>>2]|0)==0){n=m;break}hb();n=c[1414]|0}else{n=l}}while(0);Bb(a,c[n+468>>2]|0);Bb(b,c[(c[1414]|0)+472>>2]|0);n=wb(c[(c[1414]|0)+468>>2]|0)|0;xb(1,c[(c[1414]|0)+468>>2]|0);xb(1,c[(c[1414]|0)+472>>2]|0);nb(1,c[(c[1414]|0)+476>>2]|0);lb(c[(c[1414]|0)+480>>2]|0);a:do{if((Fb(c[(c[1414]|0)+472>>2]|0)|0)==0){o=0}else{l=0;j=0;m=0;p=0;q=0;r=0;s=1;while(1){t=c[1414]|0;u=c[t+468>>2]|0;if((j|0)==0){ec(u,c[t+472>>2]|0,c[t+484>>2]|0);v=c[1414]|0;w=v+468|0;x=c[w>>2]|0;c[w>>2]=c[v+472>>2];c[(c[1414]|0)+472>>2]=x;x=c[1414]|0;dc(c[x+480>>2]|0,c[x+484>>2]|0,c[x+464>>2]|0);x=c[1414]|0;v=c[x+476>>2]|0;Tb(v,c[x+464>>2]|0,v);v=c[1414]|0;x=v+476|0;w=c[x>>2]|0;c[x>>2]=c[v+480>>2];c[(c[1414]|0)+480>>2]=w;y=q+1|0}else{Yb(u,m,c[t+484>>2]|0);t=c[(c[1414]|0)+468>>2]|0;Yb(t,l,t);t=c[1414]|0;Yb(c[t+472>>2]|0,j,c[t+464>>2]|0);t=c[(c[1414]|0)+472>>2]|0;Yb(t,p,t);t=c[1414]|0;if((s|0)==0){u=c[t+468>>2]|0;Sb(c[t+464>>2]|0,u,u);u=c[1414]|0;w=c[u+472>>2]|0;Sb(c[u+484>>2]|0,w,w)}else{w=c[t+468>>2]|0;Sb(w,c[t+464>>2]|0,w);w=c[1414]|0;t=c[w+472>>2]|0;Sb(t,c[w+484>>2]|0,t)}t=c[1414]|0;Yb(c[t+476>>2]|0,m,c[t+484>>2]|0);t=c[(c[1414]|0)+476>>2]|0;Yb(t,l,t);t=c[1414]|0;Yb(c[t+480>>2]|0,j,c[t+464>>2]|0);t=c[(c[1414]|0)+480>>2]|0;Yb(t,p,t);t=c[1414]|0;if((l|0)==0){Bb(c[t+464>>2]|0,c[t+476>>2]|0)}else{w=c[t+476>>2]|0;Rb(w,c[t+464>>2]|0,w)}w=c[1414]|0;t=c[w+480>>2]|0;Rb(t,c[w+484>>2]|0,t);y=q}t=c[1414]|0;if((c[t+552>>2]|0)!=0){o=y;break a}if((Fb(c[t+472>>2]|0)|0)==0){o=y;break a}t=c[1414]|0;w=c[t+468>>2]|0;u=c[w>>2]|0;b:do{if((u|0)==1){z=0;A=c[c[w+4>>2]>>2]|0;B=0;C=c[c[(c[t+472>>2]|0)+4>>2]>>2]|0;D=1;F=28}else{v=u-1|0;x=c[w+4>>2]|0;G=c[x+(v<<2)>>2]|0;H=G+1|0;I=c[t>>2]|0;J=(u|0)>2;do{if((I|0)==0){K=u-2|0;L=c[x+(K<<2)>>2]|0;if(J&(H|0)!=0){M=Zc(G,L,H,h)|0;N=u-3|0;O=Zc(c[h>>2]|0,c[(c[(c[(c[1414]|0)+468>>2]|0)+4>>2]|0)+(N<<2)>>2]|0,H,h)|0;P=c[(c[(c[1414]|0)+472>>2]|0)+4>>2]|0;Q=Zc(c[P+(v<<2)>>2]|0,c[P+(K<<2)>>2]|0,H,h)|0;R=r;S=Zc(c[h>>2]|0,c[(c[(c[(c[1414]|0)+472>>2]|0)+4>>2]|0)+(N<<2)>>2]|0,H,h)|0;T=Q;U=O;V=M}else{M=c[(c[t+472>>2]|0)+4>>2]|0;R=(u|0)==2?1:r;S=c[M+(K<<2)>>2]|0;T=c[M+(v<<2)>>2]|0;U=L;V=G}W=R;X=T|0;Y=S|0;Z=V|0;_=U|0}else{if(J){L=u-2|0;M=Yc(G,I,c[x+(L<<2)>>2]|0,H,h)|0;K=c[1414]|0;O=c[K>>2]|0;Q=Dd(O,0,M,0)|0;M=E;N=u-3|0;P=td(Yc(c[h>>2]|0,O,c[(c[(c[K+468>>2]|0)+4>>2]|0)+(N<<2)>>2]|0,H,h)|0,0,Q,M)|0;M=E;Q=c[1414]|0;K=c[(c[Q+472>>2]|0)+4>>2]|0;O=Yc(c[K+(v<<2)>>2]|0,c[Q>>2]|0,c[K+(L<<2)>>2]|0,H,h)|0;L=c[1414]|0;K=c[L>>2]|0;Q=Dd(K,0,O,0)|0;O=E;$=td(Yc(c[h>>2]|0,K,c[(c[(c[L+472>>2]|0)+4>>2]|0)+(N<<2)>>2]|0,H,h)|0,0,Q,O)|0;W=r;X=E;Y=$;Z=M;_=P;break}else{P=I;M=0;$=Dd(P,M,G,0)|0;O=u-2|0;Q=td(c[x+(O<<2)>>2]|0,0,$,E)|0;$=E;N=c[(c[t+472>>2]|0)+4>>2]|0;L=Dd(c[N+(v<<2)>>2]|0,0,P,M)|0;M=td(L,E,c[N+(O<<2)>>2]|0,0)|0;z=$;A=Q;B=E;C=M;D=1;F=28;break b}}}while(0);if((W|0)==0){ba=1;ca=X;da=Y;ea=Z;fa=_;ga=y;ha=1;ia=0;ja=0;ka=1}else{z=Z;A=_;B=X;C=Y;D=W;F=28;break}while(1){v=ia;x=0;if((ba|0)==0){G=td(da,ca,v,x)|0;I=E;if((G|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=0;ra=W;break b}H=ud(da,ca,ha,0)|0;J=E;if((H|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=0;ra=W;break b}M=ud(fa,ea,ka,0)|0;Q=rc(M,E,G,I)|0;if((Q|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=0;ra=W;break b}I=td(fa,ea,ja,0)|0;if((Q|0)==(rc(I,E,H,J)|0)){sa=Q}else{la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=0;ra=W;break b}}else{Q=ud(da,ca,v,x)|0;x=E;if((Q|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}v=td(da,ca,ha,0)|0;J=E;if((v|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}H=td(fa,ea,ka,0)|0;I=rc(H,E,Q,x)|0;if((I|0)==0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}x=ud(fa,ea,ja,0)|0;if((I|0)==(rc(x,E,v,J)|0)){sa=I}else{la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}}if((sa|0)==1){I=ha+ja|0;if((I|0)<0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}else{ta=ca;ua=da;va=I;wa=ia}}else{if(sa>>>0>=(((-2147483648-ja|0)>>>0)/(ha>>>0)|0)>>>0){la=ka;ma=ja;na=ia;oa=ha;pa=ga;qa=ba;ra=W;break b}I=aa(sa,ia)|0;J=(aa(sa,ha)|0)+ja|0;v=Dd(sa,0,da,ca)|0;ta=E;ua=v;va=J;wa=I}I=ud(fa,ea,ua,ta)|0;J=wa+ka|0;ba=ba^1;ea=ca;ca=E;fa=da;da=I;ga=ga+1|0;ja=ha;ha=va;ka=ia;ia=J}}}while(0);c:do{if((F|0)==28){F=0;t=1;u=B;w=C;J=z;I=A;v=y;x=1;Q=0;H=0;G=1;while(1){if((w|0)==0&(u|0)==0){la=G;ma=H;na=Q;oa=x;pa=v;qa=t;ra=D;break c}M=rc(I,J,w,u)|0;if((M|0)==1){$=x+H|0;if(($|0)<0){la=G;ma=H;na=Q;oa=x;pa=v;qa=t;ra=D;break c}else{xa=u;ya=w;za=$;Aa=Q}}else if((M|0)==0){la=G;ma=H;na=Q;oa=x;pa=v;qa=t;ra=D;break c}else{if(M>>>0>=(((-2147483648-H|0)>>>0)/(x>>>0)|0)>>>0){la=G;ma=H;na=Q;oa=x;pa=v;qa=t;ra=D;break c}$=aa(M,Q)|0;O=(aa(M,x)|0)+H|0;N=Dd(M,0,w,u)|0;xa=E;ya=N;za=O;Aa=$}$=ud(I,J,ya,xa)|0;O=Aa+G|0;t=t^1;J=u;u=E;I=w;w=$;v=v+1|0;H=x;x=za;G=Q;Q=O}}}while(0);Q=(pa|0)%2|0;if((Fb(c[(c[1414]|0)+472>>2]|0)|0)==0){o=Q;break}else{l=la;j=ma;m=na;p=oa;q=Q;r=ra;s=qa}}}}while(0);if(((o+((n|0)==-1)|0)%2|0|0)==1){n=c[(c[1414]|0)+476>>2]|0;Ub(b,n,n)}if((d|0)!=(e|0)){Cb(a,c[(c[1414]|0)+472>>2]|0);a=c[1414]|0;n=c[a+480>>2]|0;gc(c[a+472>>2]|0,c[a+476>>2]|0,c[a+468>>2]|0,b,n,n);Bb(c[(c[1414]|0)+480>>2]|0,e)}Bb(c[(c[1414]|0)+476>>2]|0,d);if(!((f|0)==(d|0)|(f|0)==(e|0))){Bb(c[(c[1414]|0)+468>>2]|0,f)}f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;k=Fb(c[(c[1414]|0)+468>>2]|0)|0;i=g;return k|0}function qc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)+1;e=c[1414]|0;f=c[e+28>>2]|0;do{if((f|0)<24){c[e+32+(f<<2)>>2]=213;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);f=pc(a,b,d,d,d)|0;d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return f|0}function rc(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;e=ud(a,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&e>>>0<c>>>0){f=1;return f|0}a=ud(e,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&a>>>0<c>>>0){f=2;return f|0}e=ud(a,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&e>>>0<c>>>0){f=3;return f|0}a=ud(e,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&a>>>0<c>>>0){f=4;return f|0}e=ud(a,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&e>>>0<c>>>0){f=5;return f|0}a=ud(e,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&a>>>0<c>>>0){f=6;return f|0}e=ud(a,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&e>>>0<c>>>0){f=7;return f|0}a=ud(e,b,c,d)|0;b=E;if(b>>>0<d>>>0|b>>>0==d>>>0&a>>>0<c>>>0){f=8;return f|0}else{f=Ed(a,b,c,d)|0;d=td(f,E,8,0)|0;f=E;c=0;return(f>>>0>c>>>0|f>>>0==c>>>0&d>>>0>2147483647>>>0?0:d)|0}return 0}function sc(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((c[(c[1414]|0)+552>>2]|0)!=0){b=0;return b|0}if((Fb(a)|0)==0){b=0;return b|0}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)+1;d=c[1414]|0;e=c[d+28>>2]|0;do{if((e|0)<24){c[d+32+(e<<2)>>2]=49;f=c[1414]|0;if((c[f+576>>2]|0)==0){g=f;break}hb();g=c[1414]|0}else{g=d}}while(0);do{if((c[g>>2]|0)==(c[g+16>>2]|0)){d=(c[a>>2]&2147483647)-1|0;e=aa(d,c[g+12>>2]|0)|0;f=c[(c[a+4>>2]|0)+(d<<2)>>2]|0;if((f|0)==0){h=e;i=g;break}else{j=e;k=f}while(1){f=j+1|0;e=k>>>1;if((e|0)==0){h=f;i=g;break}else{j=f;k=e}}}else{Bb(a,c[g+464>>2]|0);xb(1,c[(c[1414]|0)+464>>2]|0);e=c[1414]|0;f=c[e+464>>2]|0;if((c[f>>2]|0)>>>0>1>>>0){d=0;l=e;m=f;while(1){_b(m,c[l+16>>2]|0,m)|0;n=c[1414]|0;o=(c[n+12>>2]|0)+d|0;p=c[n+464>>2]|0;if((c[p>>2]|0)>>>0>1>>>0){d=o;l=n;m=p}else{q=o;r=n;s=p;break}}}else{q=0;r=e;s=f}m=c[s+4>>2]|0;l=c[m>>2]|0;if((l|0)==0){h=q;i=r;break}else{t=q;u=m;v=l}while(1){l=t+1|0;c[u>>2]=v>>>1;m=c[1414]|0;d=c[(c[m+464>>2]|0)+4>>2]|0;p=c[d>>2]|0;if((p|0)==0){h=l;i=m;break}else{t=l;u=d;v=p}}}}while(0);v=i+28|0;c[v>>2]=(c[v>>2]|0)-1;b=h;return b|0}function tc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;if((c[(c[1414]|0)+552>>2]|0)!=0){return}nb(1,b);if((a|0)==0){return}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)+1;d=c[1414]|0;e=c[d+28>>2]|0;do{if((e|0)<24){c[d+32+(e<<2)>>2]=149;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)<0){fb(10);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}e=c[1414]|0;d=c[e+12>>2]|0;f=(a|0)/(d|0)|0;g=(a|0)%(d|0)|0;d=c[e+16>>2]|0;if((c[e>>2]|0)==(d|0)){Eb(b,f,b);e=b|0;a=b+4|0;h=ib(c[(c[a>>2]|0)+((c[e>>2]|0)-1<<2)>>2]|0,g)|0;c[(c[a>>2]|0)+((c[e>>2]|0)-1<<2)>>2]=h}else{a:do{if((f|0)>=1){h=1;e=d;while(1){Yb(b,e,b);if((h|0)>=(f|0)){break a}h=h+1|0;e=c[(c[1414]|0)+16>>2]|0}}}while(0);Yb(b,ib(1,g)|0,b)}b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}function uc(a){a=a|0;var b=0,d=0,e=0,f=0;b=c[1414]|0;if((c[b+552>>2]|0)!=0){d=0;return d|0}do{if((Fb(c[b+312>>2]|0)|0)!=0){if((Gb(a,c[(c[1414]|0)+312>>2]|0)|0)!=0){break}d=c[(c[1414]|0)+308>>2]|0;return d|0}}while(0);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)+1;b=c[1414]|0;e=c[b+28>>2]|0;do{if((e|0)<24){c[b+32+(e<<2)>>2]=80;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Fb(a)|0)<3){fb(19);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=0;return d|0}lb(c[(c[1414]|0)+488>>2]|0);lb(c[(c[1414]|0)+524>>2]|0);e=ac(a,8)|0;c[(c[1414]|0)+716>>2]=e;e=c[1414]|0;switch(c[e+716>>2]|0){case 5:{c[e+708>>2]=-2;break};case 7:{c[e+708>>2]=-1;break};case 3:{c[e+708>>2]=-1;break};case 0:case 1:case 2:case 4:case 6:{c[e+708>>2]=0;break};default:{}}e=ac(a,9)|0;c[(c[1414]|0)+720>>2]=e;c[(c[1414]|0)+724>>2]=0;e=a+4|0;if((c[(c[e>>2]|0)+((c[a>>2]|0)-1<<2)>>2]|0)>>>0<1342177280>>>0){c[(c[1414]|0)+724>>2]=1}c[(c[1414]|0)+324>>2]=1;nb(1,c[(c[1414]|0)+532>>2]|0);b=c[1414]|0;if((c[b+324>>2]|0)==0){Bb(a,c[b+312>>2]|0);c[(c[1414]|0)+308>>2]=0;f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;d=0;return d|0}c[c[b+488>>2]>>2]=2;c[c[(c[(c[1414]|0)+488>>2]|0)+4>>2]>>2]=0;c[(c[(c[(c[1414]|0)+488>>2]|0)+4>>2]|0)+4>>2]=1;c[c[(c[1414]|0)+524>>2]>>2]=1;c[c[(c[(c[1414]|0)+524>>2]|0)+4>>2]>>2]=c[c[e>>2]>>2];e=c[1414]|0;if((qc(c[e+524>>2]|0,c[e+488>>2]|0,c[e+520>>2]|0)|0)==1){e=c[1414]|0;c[e+308>>2]=(c[e>>2]|0)-(c[c[(c[e+520>>2]|0)+4>>2]>>2]|0);Bb(a,c[(c[1414]|0)+312>>2]|0);c[(c[1414]|0)+128>>2]=0;a=c[1414]|0;e=c[a+312>>2]|0;Eb(e,c[e>>2]|0,c[a+316>>2]|0);c[(c[1414]|0)+128>>2]=1;a=c[(c[1414]|0)+532>>2]|0;vc(a,a);a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)-1;d=c[(c[1414]|0)+308>>2]|0;return d|0}else{fb(19);a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)-1;d=0;return d|0}return 0}function vc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){return}e=d+28|0;c[e>>2]=(c[e>>2]|0)+1;e=c[1414]|0;d=c[e+28>>2]|0;do{if((d|0)<24){c[e+32+(d<<2)>>2]=81;f=c[1414]|0;if((c[f+576>>2]|0)==0){g=f;break}hb();g=c[1414]|0}else{g=e}}while(0);if((Fb(c[g+312>>2]|0)|0)==0){fb(20);g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}Bb(a,b);a=c[(c[1414]|0)+312>>2]|0;ec(b,a,a);if((Fb(b)|0)<0){Tb(b,c[(c[1414]|0)+312>>2]|0,b)}a=c[1414]|0;if((c[a+324>>2]|0)==0){g=a+28|0;c[g>>2]=(c[g>>2]|0)-1;return}else{c[a+128>>2]=0;a=c[1414]|0;Eb(b,c[c[a+312>>2]>>2]|0,c[a+464>>2]|0);a=c[1414]|0;g=c[a+312>>2]|0;ec(c[a+464>>2]|0,g,g);c[(c[1414]|0)+128>>2]=1;Bb(c[(c[1414]|0)+464>>2]|0,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}}function wc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;d=i;i=i+16|0;e=d|0;f=d+8|0;g=c[1414]|0;if((c[g+552>>2]|0)!=0){i=d;return}h=g+28|0;c[h>>2]=(c[h>>2]|0)+1;h=c[1414]|0;g=c[h+28>>2]|0;do{if((g|0)<24){c[h+32+(g<<2)>>2]=82;j=c[1414]|0;if((c[j+576>>2]|0)==0){k=j;break}hb();k=c[1414]|0}else{k=h}}while(0);h=c[k+464>>2]|0;g=c[k+312>>2]|0;j=c[k+308>>2]|0;Bb(a,h);a=c[1414]|0;if((c[a+324>>2]|0)==0){ec(h,g,g);Bb(h,b);k=(c[1414]|0)+28|0;c[k>>2]=(c[k>>2]|0)-1;i=d;return}k=c[g>>2]|0;l=k<<1;m=c[a>>2]|0;a:do{if((m|0)==0){a=g+4|0;n=h+4|0;if((k|0)<=0){o=0;p=n;break}q=0;r=0;s=c[n>>2]|0;while(1){t=aa(c[s+(q<<2)>>2]|0,j)|0;c[f>>2]=t;c[e>>2]=0;u=0;v=t;while(1){$c(v,c[(c[a>>2]|0)+(u<<2)>>2]|0,e,(c[n>>2]|0)+(u+q<<2)|0);t=u+1|0;if((t|0)>=(k|0)){break}u=t;v=c[f>>2]|0}v=q+k|0;u=(c[n>>2]|0)+(v<<2)|0;c[u>>2]=(c[u>>2]|0)+r;u=(c[n>>2]|0)+(v<<2)|0;t=c[u>>2]|0;c[u>>2]=(c[e>>2]|0)+t;u=c[n>>2]|0;w=(c[u+(v<<2)>>2]|0)>>>0<(c[e>>2]|0)>>>0?1:t>>>0<r>>>0|0;t=q+1|0;if((t|0)<(k|0)){q=t;r=w;s=u}else{o=w;p=n;break}}}else{n=h+4|0;if((k|0)<=0){o=0;p=n;break}s=g+4|0;r=0;q=0;a=m;while(1){Yc(c[(c[n>>2]|0)+(r<<2)>>2]|0,j,0,a,f)|0;c[e>>2]=0;w=0;u=0;do{t=(c[n>>2]|0)+(w+r<<2)|0;u=Yc(c[(c[s>>2]|0)+(w<<2)>>2]|0,c[f>>2]|0,u+(c[t>>2]|0)|0,c[c[1414]>>2]|0,t)|0;c[e>>2]=u;w=w+1|0;}while((w|0)<(k|0));w=r+k|0;t=(c[n>>2]|0)+(w<<2)|0;c[t>>2]=u+q+(c[t>>2]|0);t=(c[n>>2]|0)+(w<<2)|0;w=c[t>>2]|0;v=c[c[1414]>>2]|0;if(w>>>0<v>>>0){x=0}else{c[t>>2]=w-v;x=1}v=r+1|0;if((v|0)>=(k|0)){o=x;p=n;break a}r=v;q=x;a=c[c[1414]>>2]|0}}}while(0);c[(c[p>>2]|0)+(l<<2)>>2]=o;c[h>>2]=l|1;Eb(h,-k|0,h);rb(h);if((Gb(h,g)|0)>-1){Sb(h,g,h)}Bb(h,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;i=d;return}function xc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=(a|0)==0;do{if(e){if((d|0)!=0){break}return}else{if(!((c[a>>2]|0)==0&(a|0)==(d|0))){break}return}}while(0);do{if((b|0)==0){if((d|0)!=0){break}return}else{f=(c[b>>2]|0)==0;if(f&(b|0)==(d|0)){return}if(e){break}if((c[a>>2]|0)==0|f){break}f=c[1414]|0;if((c[f+552>>2]|0)!=0){return}g=f+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;f=c[g+28>>2]|0;do{if((f|0)<24){c[g+32+(f<<2)>>2]=83;h=c[1414]|0;if((c[h+576>>2]|0)==0){i=h;break}hb();i=c[1414]|0}else{i=g}}while(0);c[i+128>>2]=0;dc(a,b,c[(c[1414]|0)+464>>2]|0);wc(c[(c[1414]|0)+464>>2]|0,d);c[(c[1414]|0)+128>>2]=1;g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}}while(0);lb(d);return}function yc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=90;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);Rb(a,b,d);if((Gb(d,c[(c[1414]|0)+312>>2]|0)|0)>-1){Sb(d,c[(c[1414]|0)+312>>2]|0,d)}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function zc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){return}f=e+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;e=c[f+28>>2]|0;do{if((e|0)<24){c[f+32+(e<<2)>>2]=91;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Gb(a,b)|0)>-1){Sb(a,b,d)}else{Sb(b,a,d);Sb(c[(c[1414]|0)+312>>2]|0,d,d)}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Ac(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if((Fb(a)|0)==0){lb(b);return}d=c[1414]|0;if((c[d+552>>2]|0)!=0){return}e=d+28|0;c[e>>2]=(c[e>>2]|0)+1;e=c[1414]|0;d=c[e+28>>2]|0;do{if((d|0)<24){c[e+32+(d<<2)>>2]=92;f=c[1414]|0;if((c[f+576>>2]|0)==0){g=f;break}hb();g=c[1414]|0}else{g=e}}while(0);Sb(c[g+312>>2]|0,a,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}function Bc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)+1;d=c[1414]|0;e=c[d+28>>2]|0;do{if((e|0)<24){c[d+32+(e<<2)>>2]=198;f=c[1414]|0;if((c[f+576>>2]|0)==0){g=f;break}hb();g=c[1414]|0}else{g=d}}while(0);Bb(a,c[g+468>>2]|0);if((ac(c[(c[1414]|0)+468>>2]|0,2)|0)!=0){g=c[1414]|0;a=c[g+468>>2]|0;Tb(a,c[g+312>>2]|0,a)}a=c[(c[1414]|0)+468>>2]|0;$b(a,2,a)|0;Bb(c[(c[1414]|0)+468>>2]|0,b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;return}function Cc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=c[1414]|0;if((c[e+552>>2]|0)!=0){f=0;return f|0}g=e+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;e=c[g+28>>2]|0;do{if((e|0)<24){c[g+32+(e<<2)>>2]=85;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)==(b|0)){fb(7);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;f=0;return f|0}wc(b,c[(c[1414]|0)+488>>2]|0);b=c[1414]|0;e=c[b+488>>2]|0;g=qc(e,c[b+312>>2]|0,e)|0;if((g|0)==1){e=c[(c[1414]|0)+488>>2]|0;vc(e,e);xc(a,c[(c[1414]|0)+488>>2]|0,d)}else{lb(d)}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;f=g;return f|0}function Dc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;if((b|0)==0){lb(d);return}e=(b|0)<0;f=e?-b|0:b;b=c[1414]|0;if((c[b+552>>2]|0)!=0){return}g=b+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;b=c[g+28>>2]|0;do{if((b|0)<24){c[g+32+(b<<2)>>2]=102;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((f|0)>=7){Yb(a,f,c[(c[1414]|0)+464>>2]|0);b=c[1414]|0;g=c[b+312>>2]|0;ec(c[b+464>>2]|0,g,g);Bb(c[(c[1414]|0)+464>>2]|0,d);if(e){Ac(d,d)}g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;return}switch(f|0){case 2:{yc(a,a,d);break};case 4:{yc(a,a,d);yc(d,d,d);break};case 3:{yc(a,a,c[(c[1414]|0)+464>>2]|0);yc(a,c[(c[1414]|0)+464>>2]|0,d);break};case 1:{Bb(a,d);break};case 5:{yc(a,a,c[(c[1414]|0)+464>>2]|0);f=c[(c[1414]|0)+464>>2]|0;yc(f,f,f);yc(a,c[(c[1414]|0)+464>>2]|0,d);break};case 6:{yc(a,a,d);yc(d,d,c[(c[1414]|0)+464>>2]|0);yc(d,c[(c[1414]|0)+464>>2]|0,d);break};default:{}}if(e){Ac(d,d)}d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;return}function Ec(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;if((a|0)==0){e=1;return e|0}if((a|0)<0){e=0;return e|0}f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;g=c[f+28>>2]|0;do{if((g|0)<24){c[f+32+(g<<2)>>2]=118;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((b|0)==(d|0)){fb(7);g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;e=0;return e|0}if((a|0)==1){Bb(c[(c[1414]|0)+532>>2]|0,c[d>>2]|0);g=c[d>>2]|0;Cc(g,c[b>>2]|0,g)|0;g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;e=1;return e|0}nb(1,c[d>>2]|0);Bb(c[b>>2]|0,c[d+4>>2]|0);if((a|0)>2){g=2;do{f=g-1|0;xc(c[d+(f<<2)>>2]|0,c[b+(f<<2)>>2]|0,c[d+(g<<2)>>2]|0);g=g+1|0;}while((g|0)<(a|0))}g=a-1|0;f=d+(g<<2)|0;h=b+(g<<2)|0;xc(c[f>>2]|0,c[h>>2]|0,c[(c[1414]|0)+488>>2]|0);if((Fb(c[(c[1414]|0)+488>>2]|0)|0)==0){fb(2);g=(c[1414]|0)+28|0;c[g>>2]=(c[g>>2]|0)-1;e=0;return e|0}g=c[(c[1414]|0)+488>>2]|0;wc(g,g);g=c[(c[1414]|0)+488>>2]|0;wc(g,g);g=c[1414]|0;i=c[g+488>>2]|0;qc(i,c[g+312>>2]|0,i)|0;Bb(c[h>>2]|0,c[(c[1414]|0)+484>>2]|0);h=c[f>>2]|0;xc(h,c[(c[1414]|0)+488>>2]|0,h);h=a-2|0;if((h|0)!=0){a=h;do{h=d+(a<<2)|0;f=c[h>>2]|0;xc(f,c[(c[1414]|0)+484>>2]|0,f);f=c[h>>2]|0;xc(f,c[(c[1414]|0)+488>>2]|0,f);f=c[(c[1414]|0)+484>>2]|0;xc(f,c[b+(a<<2)>>2]|0,f);a=a-1|0;}while((a|0)!=0)}a=c[1414]|0;xc(c[a+484>>2]|0,c[a+488>>2]|0,c[d>>2]|0);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;e=1;return e|0}function Fc(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=c[1414]|0;if((c[f+552>>2]|0)!=0){return}g=f+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;f=c[g+28>>2]|0;do{if((f|0)<24){c[g+32+(f<<2)>>2]=93;h=c[1414]|0;if((c[h+576>>2]|0)==0){i=h;break}hb();i=c[1414]|0}else{i=g}}while(0);c[i+328>>2]=0;uc(d)|0;i=Fb(a)|0;c[(c[1414]|0)+340>>2]=i;i=c[1414]|0;g=c[i+340>>2]|0;do{if((((g|0)<0?-g|0:g)|0)==1073741824&(g|0)>-1){Bb(a,c[i+468>>2]|0);ec(c[(c[1414]|0)+468>>2]|0,d,d);f=c[(c[1414]|0)+468>>2]|0;Ub(d,f,f);f=Fb(c[(c[1414]|0)+468>>2]|0)|0;if((f|0)>=1073741824){break}c[(c[1414]|0)+340>>2]=-f}}while(0);vc(a,c[(c[1414]|0)+536>>2]|0);a=Fb(b)|0;c[(c[1414]|0)+344>>2]=a;a=c[1414]|0;i=c[a+344>>2]|0;do{if((((i|0)<0?-i|0:i)|0)==1073741824&(i|0)>-1){Bb(b,c[a+468>>2]|0);ec(c[(c[1414]|0)+468>>2]|0,d,d);g=c[(c[1414]|0)+468>>2]|0;Ub(d,g,g);g=Fb(c[(c[1414]|0)+468>>2]|0)|0;if((g|0)>=1073741824){break}c[(c[1414]|0)+344>>2]=-g}}while(0);vc(b,c[(c[1414]|0)+540>>2]|0);c[(c[1414]|0)+336>>2]=(e|0)==2?0:e;e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}function Gc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((c[(c[1414]|0)+552>>2]|0)!=0){d=0;return d|0}Bb(a,b);if((Fb(b)|0)==0){d=1;return d|0}a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)+1;a=c[1414]|0;e=c[a+28>>2]|0;do{if((e|0)<24){c[a+32+(e<<2)>>2]=100;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);wc(b,b);if((Fb(b)|0)==1){vc(b,b);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=1;return d|0}if((Fb(b)|0)==4){nb(2,b);vc(b,b);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=1;return d|0}if((oc(b,c[(c[1414]|0)+312>>2]|0)|0)!=1){lb(b);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=0;return d|0}e=c[1414]|0;a=((c[e+716>>2]|0)%4|0)-2|0;Vb(c[e+312>>2]|0,a,c[e+504>>2]|0);e=c[(c[1414]|0)+504>>2]|0;$b(e,4,e)|0;e=c[1414]|0;a:do{if((a|0)==1){vc(b,c[e+472>>2]|0);Bb(c[(c[1414]|0)+532>>2]|0,b);while(1){f=c[1414]|0;g=c[f+20>>2]|0;if((g|0)==0){h=f}else{Ka[g&1]()|0;h=c[1414]|0}g=c[h+504>>2]|0;if(($b(g,2,g)|0)!=0){xc(b,c[(c[1414]|0)+472>>2]|0,b)}g=c[1414]|0;if((c[g+552>>2]|0)!=0){break a}if((Fb(c[g+504>>2]|0)|0)==0){break a}g=c[(c[1414]|0)+472>>2]|0;xc(g,g,g)}}else{g=1;f=e;while(1){i=(g|0)==1;j=c[f+480>>2]|0;if(i){Bb(b,j)}else{Zb(b,g,j);j=c[1414]|0;k=c[j+312>>2]|0;ec(c[j+480>>2]|0,k,k);k=c[(c[1414]|0)+480>>2]|0;Zb(k,g,k);k=c[1414]|0;j=c[k+312>>2]|0;ec(c[k+480>>2]|0,j,j)}j=c[1414]|0;Wb(c[j+480>>2]|0,4,c[j+468>>2]|0);j=c[1414]|0;k=(oc(c[j+468>>2]|0,c[j+312>>2]|0)|0)==(a|0);l=c[1414]|0;if(k){break}if((c[l+552>>2]|0)==0){g=g+1|0;f=l}else{break}}Wb(c[l+480>>2]|0,2,c[l+476>>2]|0);f=c[(c[1414]|0)+476>>2]|0;vc(f,f);f=c[1414]|0;Xc(c[f+476>>2]|0,c[f+504>>2]|0,b,b);if(i){break}nb(g,c[(c[1414]|0)+508>>2]|0);f=c[(c[1414]|0)+508>>2]|0;vc(f,f);Cc(b,c[(c[1414]|0)+508>>2]|0,b)|0}}while(0);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;d=1;return d|0}function Hc(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=c[1414]|0;if((c[f+552>>2]|0)!=0){g=0;return g|0}h=f+28|0;c[h>>2]=(c[h>>2]|0)+1;h=c[1414]|0;f=c[h+28>>2]|0;do{if((f|0)<24){c[h+32+(f<<2)>>2]=97;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)==0|(b|0)==0){Bb(c[(c[1414]|0)+532>>2]|0,c[e+4>>2]|0);Bb(c[(c[1414]|0)+532>>2]|0,c[e+8>>2]|0);c[e>>2]=2;f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;g=1;return g|0}f=e+4|0;vc(a,c[f>>2]|0);Rc(c[f>>2]|0,c[(c[1414]|0)+476>>2]|0);if((a|0)==(b|0)){a=e+8|0;f=Gc(c[(c[1414]|0)+476>>2]|0,c[a>>2]|0)|0;wc(c[a>>2]|0,c[(c[1414]|0)+468>>2]|0);if((ac(c[(c[1414]|0)+468>>2]|0,2)|0)!=(d|0)){d=c[a>>2]|0;Sb(c[(c[1414]|0)+312>>2]|0,d,d)}if((f|0)==0){i=13}else{i=12}}else{f=e+8|0;vc(b,c[f>>2]|0);b=c[f>>2]|0;xc(b,b,c[(c[1414]|0)+468>>2]|0);b=c[1414]|0;if((Gb(c[b+468>>2]|0,c[b+476>>2]|0)|0)==0){i=12}else{i=13}}if((i|0)==12){c[e>>2]=1;e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;g=1;return g|0}else if((i|0)==13){i=(c[1414]|0)+28|0;c[i>>2]=(c[i>>2]|0)-1;g=0;return g|0}return 0}function Ic(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;if((c[a>>2]|0)==2){lb(b);lb(d);e=0;return e|0}f=c[1414]|0;if((c[f+552>>2]|0)!=0){e=0;return e|0}g=f+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;f=c[g+28>>2]|0;do{if((f|0)<24){c[g+32+(f<<2)>>2]=98;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Jc(a)|0)==0){f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;e=-1;return e|0}wc(c[a+4>>2]|0,b);wc(c[a+8>>2]|0,c[(c[1414]|0)+468>>2]|0);if((b|0)!=(d|0)){Bb(c[(c[1414]|0)+468>>2]|0,d)}d=ac(c[(c[1414]|0)+468>>2]|0,2)|0;b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;e=d;return e|0}function Jc(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=c[1414]|0;if((c[b+336>>2]|0)==1){d=1;return d|0}e=a|0;if((c[e>>2]|0)!=0){d=1;return d|0}if((c[b+552>>2]|0)!=0){d=0;return d|0}f=b+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;b=c[f+28>>2]|0;do{if((b|0)<24){c[f+32+(b<<2)>>2]=117;g=c[1414]|0;if((c[g+576>>2]|0)==0){h=g;break}hb();h=c[1414]|0}else{h=f}}while(0);Bb(c[h+532>>2]|0,c[h+496>>2]|0);h=c[(c[1414]|0)+496>>2]|0;f=a+12|0;if((Cc(h,c[f>>2]|0,h)|0)>1){Hc(0,0,0,a)|0;fb(28);h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;d=0;return d|0}else{h=c[1414]|0;b=c[h+496>>2]|0;xc(b,b,c[h+468>>2]|0);h=c[a+4>>2]|0;xc(h,c[(c[1414]|0)+468>>2]|0,h);h=c[1414]|0;b=c[h+468>>2]|0;xc(b,c[h+496>>2]|0,b);b=c[a+8>>2]|0;xc(b,c[(c[1414]|0)+468>>2]|0,b);Bb(c[(c[1414]|0)+532>>2]|0,c[f>>2]|0);c[e>>2]=1;e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;d=1;return d|0}return 0}function Kc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;i=i+256|0;f=e|0;g=c[1414]|0;if((c[g+336>>2]|0)==1){h=1;i=e;return h|0}if((c[g+552>>2]|0)!=0|(a|0)>64){h=0;i=e;return h|0}j=g+28|0;c[j>>2]=(c[j>>2]|0)+1;j=c[1414]|0;g=c[j+28>>2]|0;do{if((g|0)<24){c[j+32+(g<<2)>>2]=190;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);g=(a|0)>0;a:do{if(g){j=0;while(1){k=c[d+(j<<2)>>2]|0;l=k|0;if((c[l>>2]|0)==1){c[f+(j<<2)>>2]=c[(c[1414]|0)+532>>2]}else{c[f+(j<<2)>>2]=c[k+12>>2]}j=j+1|0;if((c[l>>2]|0)==2){m=0;break}if((j|0)>=(a|0)){break a}}do{Jc(c[d+(m<<2)>>2]|0)|0;m=m+1|0;}while((m|0)<(a|0));j=(c[1414]|0)+28|0;c[j>>2]=(c[j>>2]|0)-1;h=1;i=e;return h|0}}while(0);m=(Ec(a,f|0,b)|0)==0;f=c[1414]|0;if(m){m=f+28|0;c[m>>2]=(c[m>>2]|0)-1;h=0;i=e;return h|0}if(g){g=0;m=f;while(1){j=d+(g<<2)|0;Bb(c[m+532>>2]|0,c[(c[j>>2]|0)+12>>2]|0);c[c[j>>2]>>2]=1;l=b+(g<<2)|0;k=c[l>>2]|0;xc(k,k,c[(c[1414]|0)+468>>2]|0);k=c[(c[j>>2]|0)+4>>2]|0;xc(k,c[(c[1414]|0)+468>>2]|0,k);k=c[(c[1414]|0)+468>>2]|0;xc(k,c[l>>2]|0,k);k=c[(c[j>>2]|0)+8>>2]|0;xc(k,c[(c[1414]|0)+468>>2]|0,k);k=g+1|0;j=c[1414]|0;if((k|0)<(a|0)){g=k;m=j}else{n=j;break}}}else{n=f}f=n+28|0;c[f>>2]=(c[f>>2]|0)-1;h=1;i=e;return h|0}function Lc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}f=d+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;d=c[f+28>>2]|0;do{if((d|0)<24){c[f+32+(d<<2)>>2]=94;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)==(b|0)){Nc(a);d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;e=(c[a>>2]|0)==2?0:2;return e|0}d=b|0;if((c[d>>2]|0)==2){Mc(a,b);f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;e=1;return e|0}if((c[a>>2]|0)==2){f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;e=1;return e|0}if((Sc(a,b)|0)==0){Nc(b);b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;e=2;return e|0}else{b=(c[1414]|0)+28|0;c[b>>2]=(c[b>>2]|0)-1;e=(c[d>>2]|0)!=2|0;return e|0}return 0}function Mc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;if((a|0)==(b|0)|(b|0)==0){return}Bb(c[a+4>>2]|0,c[b+4>>2]|0);Bb(c[a+8>>2]|0,c[b+8>>2]|0);d=a|0;e=c[d>>2]|0;if((e|0)==0){Bb(c[a+12>>2]|0,c[b+12>>2]|0);f=c[d>>2]|0}else{f=e}c[b>>2]=f;return}function Nc(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;b=c[1414]|0;if((c[b+552>>2]|0)!=0){return}d=a|0;if((c[d>>2]|0)==2){return}e=(c[b+336>>2]|0)==1;b=a+8|0;f=(Fb(c[b>>2]|0)|0)==0;if(e){if(f){Hc(0,0,0,a)|0;return}e=a+4|0;g=c[e>>2]|0;xc(g,g,c[(c[1414]|0)+496>>2]|0);g=c[(c[1414]|0)+496>>2]|0;Dc(g,3,g);g=c[1414]|0;h=c[g+340>>2]|0;if((((h|0)<0?-h|0:h)|0)==1073741824){i=c[g+496>>2]|0;yc(i,c[g+536>>2]|0,i)}else{nb(h,c[g+472>>2]|0);g=c[(c[1414]|0)+472>>2]|0;vc(g,g);g=c[1414]|0;h=c[g+496>>2]|0;yc(h,c[g+472>>2]|0,h)}Dc(c[b>>2]|0,2,c[(c[1414]|0)+488>>2]|0);h=c[1414]|0;g=c[h+496>>2]|0;if((Cc(g,c[h+488>>2]|0,g)|0)>1){Hc(0,0,0,a)|0;fb(28);return}else{g=c[1414]|0;h=c[g+496>>2]|0;xc(h,h,c[g+472>>2]|0);Dc(c[e>>2]|0,2,c[(c[1414]|0)+468>>2]|0);g=c[1414]|0;h=c[g+468>>2]|0;zc(c[g+472>>2]|0,h,h);h=c[1414]|0;zc(c[e>>2]|0,c[h+468>>2]|0,c[h+472>>2]|0);h=c[1414]|0;g=c[h+472>>2]|0;xc(g,c[h+496>>2]|0,g);g=c[b>>2]|0;zc(c[(c[1414]|0)+472>>2]|0,g,g);Bb(c[(c[1414]|0)+468>>2]|0,c[e>>2]|0);return}}if(f){Hc(0,0,0,a)|0;return}nb(1,c[(c[1414]|0)+468>>2]|0);f=c[1414]|0;e=c[f+340>>2]|0;do{if((((e|0)<0?-e|0:e)|0)<1073741824){do{if((e|0)!=0){if((c[d>>2]|0)==1){vc(c[f+468>>2]|0,c[f+488>>2]|0);break}else{g=c[a+12>>2]|0;xc(g,g,c[f+488>>2]|0);break}}}while(0);g=c[1414]|0;h=c[g+340>>2]|0;if((h|0)==(-3|0)){i=a+4|0;zc(c[i>>2]|0,c[g+488>>2]|0,c[g+476>>2]|0);j=c[1414]|0;yc(c[i>>2]|0,c[j+488>>2]|0,c[j+496>>2]|0);j=c[1414]|0;i=c[j+476>>2]|0;xc(i,c[j+496>>2]|0,i);i=c[1414]|0;j=c[i+476>>2]|0;yc(j,j,c[i+496>>2]|0);i=c[1414]|0;j=c[i+496>>2]|0;yc(j,c[i+476>>2]|0,j);break}else if((h|0)==0){k=g}else{h=c[g+488>>2]|0;xc(h,h,c[g+476>>2]|0);g=c[1414]|0;h=c[g+476>>2]|0;Dc(h,c[g+340>>2]|0,h);k=c[1414]|0}h=c[a+4>>2]|0;xc(h,h,c[k+468>>2]|0);h=c[1414]|0;g=c[h+468>>2]|0;yc(g,g,c[h+496>>2]|0);h=c[1414]|0;g=c[h+496>>2]|0;yc(g,c[h+468>>2]|0,g);g=c[1414]|0;if((c[g+340>>2]|0)==0){break}h=c[g+496>>2]|0;yc(h,c[g+476>>2]|0,h)}else{if((c[d>>2]|0)==1){vc(c[f+468>>2]|0,c[f+488>>2]|0)}else{h=c[a+12>>2]|0;xc(h,h,c[f+488>>2]|0)}h=c[1414]|0;g=c[h+488>>2]|0;xc(g,g,c[h+476>>2]|0);h=c[1414]|0;g=c[h+476>>2]|0;xc(g,c[h+536>>2]|0,g);g=c[a+4>>2]|0;xc(g,g,c[(c[1414]|0)+468>>2]|0);g=c[1414]|0;h=c[g+468>>2]|0;yc(h,h,c[g+496>>2]|0);g=c[1414]|0;h=c[g+496>>2]|0;yc(h,c[g+468>>2]|0,h);h=c[1414]|0;g=c[h+496>>2]|0;yc(g,c[h+476>>2]|0,g)}}while(0);f=c[b>>2]|0;xc(f,f,c[(c[1414]|0)+472>>2]|0);f=a+4|0;k=c[1414]|0;xc(c[f>>2]|0,c[k+472>>2]|0,c[k+476>>2]|0);k=c[(c[1414]|0)+476>>2]|0;yc(k,k,k);k=c[(c[1414]|0)+476>>2]|0;yc(k,k,k);k=c[(c[1414]|0)+496>>2]|0;xc(k,k,c[f>>2]|0);k=c[f>>2]|0;zc(k,c[(c[1414]|0)+476>>2]|0,k);k=c[f>>2]|0;zc(k,c[(c[1414]|0)+476>>2]|0,k);if((c[d>>2]|0)==1){Bb(c[b>>2]|0,c[a+12>>2]|0)}else{k=c[a+12>>2]|0;xc(k,c[b>>2]|0,k)}k=c[a+12>>2]|0;yc(k,k,k);k=c[1414]|0;a=c[k+472>>2]|0;yc(a,a,c[k+492>>2]|0);k=c[1414]|0;a=c[k+492>>2]|0;xc(a,a,c[k+472>>2]|0);k=c[(c[1414]|0)+472>>2]|0;yc(k,k,k);k=c[(c[1414]|0)+476>>2]|0;zc(k,c[f>>2]|0,k);k=c[1414]|0;xc(c[k+496>>2]|0,c[k+476>>2]|0,c[b>>2]|0);k=c[b>>2]|0;zc(k,c[(c[1414]|0)+472>>2]|0,k);c[d>>2]=0;return}function Oc(a){a=a|0;var b=0,d=0;b=c[1414]|0;if((c[b+552>>2]|0)!=0){return}if((c[a>>2]|0)==2){return}d=b+28|0;c[d>>2]=(c[d>>2]|0)+1;d=c[1414]|0;b=c[d+28>>2]|0;do{if((b|0)<24){c[d+32+(b<<2)>>2]=121;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);b=a+8|0;if((Fb(c[b>>2]|0)|0)!=0){a=c[b>>2]|0;Sb(c[(c[1414]|0)+312>>2]|0,a,a)}a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)-1;return}function Pc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[1414]|0;if((c[d+552>>2]|0)!=0){e=0;return e|0}f=d+28|0;c[f>>2]=(c[f>>2]|0)+1;f=c[1414]|0;d=c[f+28>>2]|0;do{if((d|0)<24){c[f+32+(d<<2)>>2]=104;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((a|0)==(b|0)){Hc(0,0,0,a)|0;d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;e=0;return e|0}if((c[a>>2]|0)==2){d=(c[1414]|0)+28|0;c[d>>2]=(c[d>>2]|0)-1;e=1;return e|0}else{Oc(a);d=Lc(a,b)|0;Oc(a);a=(c[1414]|0)+28|0;c[a>>2]=(c[a>>2]|0)-1;e=d;return e|0}return 0}function Qc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;e=i;i=i+80|0;f=e|0;g=e+8|0;h=e+16|0;j=e+48|0;k=c[1414]|0;if((c[k+552>>2]|0)!=0){l=0;i=e;return l|0}m=k+28|0;c[m>>2]=(c[m>>2]|0)+1;m=c[1414]|0;k=c[m+28>>2]|0;do{if((k|0)<24){c[m+32+(k<<2)>>2]=95;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Fb(a)|0)==0){Hc(0,0,0,d)|0;k=(c[1414]|0)+28|0;c[k>>2]=(c[k>>2]|0)-1;l=0;i=e;return l|0}Bb(a,c[(c[1414]|0)+500>>2]|0);Mc(b,d);if((Fb(c[(c[1414]|0)+500>>2]|0)|0)<0){b=c[(c[1414]|0)+500>>2]|0;Cb(b,b);Oc(d)}b=(Fb(c[(c[1414]|0)+500>>2]|0)|0)==1;a=c[1414]|0;if(b){b=a+28|0;c[b>>2]=(c[b>>2]|0)-1;l=0;i=e;return l|0}Zb(c[a+500>>2]|0,3,c[a+504>>2]|0);a=c[1414]|0;if((c[a>>2]|0)==(c[a+16>>2]|0)){a=Pb(8)|0;b=ub(8)|0;k=h|0;c[k>>2]=Ob(a,0)|0;m=j|0;c[m>>2]=qb(b,0)|0;n=h+4|0;c[n>>2]=Ob(a,1)|0;c[j+4>>2]=qb(b,1)|0;o=Ob(a,2)|0;c[h+8>>2]=o;c[j+8>>2]=qb(b,2)|0;p=Ob(a,3)|0;c[h+12>>2]=p;c[j+12>>2]=qb(b,3)|0;q=Ob(a,4)|0;c[h+16>>2]=q;c[j+16>>2]=qb(b,4)|0;r=Ob(a,5)|0;c[h+20>>2]=r;c[j+20>>2]=qb(b,5)|0;s=Ob(a,6)|0;c[h+24>>2]=s;c[j+24>>2]=qb(b,6)|0;t=h+28|0;c[t>>2]=Ob(a,7)|0;c[j+28>>2]=qb(b,7)|0;Mc(d,c[k>>2]|0);Mc(c[k>>2]|0,c[t>>2]|0);Nc(c[t>>2]|0);j=c[t>>2]|0;Mc(c[k>>2]|0,c[n>>2]|0);t=c[n>>2]|0;Lc(j,t)|0;Mc(t,o);Lc(j,o)|0;Mc(o,p);Lc(j,p)|0;Mc(p,q);Lc(j,q)|0;Mc(q,r);Lc(j,r)|0;Mc(r,s);Lc(j,s)|0;Lc(s,j)|0;Kc(8,m,k)|0;k=sc(c[(c[1414]|0)+504>>2]|0)|0;Hc(0,0,0,d)|0;m=k-1|0;if((m|0)>0){k=0;j=m;while(1){m=c[1414]|0;s=c[m+20>>2]|0;if((s|0)==0){u=m}else{Ka[s&1]()|0;u=c[1414]|0}s=Lb(c[u+500>>2]|0,c[u+504>>2]|0,j,f,g,8)|0;if((c[f>>2]|0)>0){m=0;do{Nc(d);m=m+1|0;}while((m|0)<(c[f>>2]|0))}if((s|0)>0){Lc(c[h+(((s|0)/2|0)<<2)>>2]|0,d)|0;v=k+1|0}else{v=k}if((s|0)<0){Pc(c[h+(((s|0)/-2|0)<<2)>>2]|0,d)|0;w=v+1|0}else{w=v}m=j-(c[f>>2]|0)|0;r=c[g>>2]|0;if((r|0)==0){x=m}else{if((r|0)>0){q=0;while(1){Nc(d);p=q+1|0;o=c[g>>2]|0;if((p|0)<(o|0)){q=p}else{y=o;break}}}else{y=r}x=m-y|0}if((x|0)>0){k=w;j=x}else{z=w;break}}}else{z=0}Qb(a,8);vb(b,8);A=z}else{z=Pb(1)|0;b=Ob(z,0)|0;Jc(d)|0;Mc(d,b);a=(sc(c[(c[1414]|0)+504>>2]|0)|0)-1|0;tc(a,c[(c[1414]|0)+508>>2]|0);a=c[1414]|0;w=c[a+504>>2]|0;Sb(w,c[a+508>>2]|0,w);w=c[(c[1414]|0)+508>>2]|0;$b(w,2,w)|0;if((Fb(c[(c[1414]|0)+508>>2]|0)|0)>1){w=0;while(1){a=c[(c[1414]|0)+20>>2]|0;if((a|0)!=0){Ka[a&1]()|0}Nc(d);a=c[1414]|0;x=Gb(c[a+500>>2]|0,c[a+508>>2]|0)|0;a=c[1414]|0;j=Gb(c[a+504>>2]|0,c[a+508>>2]|0)|0;if((j|0)>-1){if((x|0)<0){Lc(b,d)|0;B=w+1|0}else{B=w}a=c[1414]|0;k=c[a+504>>2]|0;Sb(k,c[a+508>>2]|0,k);C=B}else{C=w}if((x|0)>-1){if((j|0)<0){Pc(b,d)|0;D=C+1|0}else{D=C}j=c[1414]|0;x=c[j+500>>2]|0;Sb(x,c[j+508>>2]|0,x);E=D}else{E=C}x=c[(c[1414]|0)+508>>2]|0;$b(x,2,x)|0;if((Fb(c[(c[1414]|0)+508>>2]|0)|0)>1){w=E}else{F=E;break}}}else{F=0}Qb(z,1);A=F}F=(c[1414]|0)+28|0;c[F>>2]=(c[F>>2]|0)-1;l=A;i=e;return l|0}function Rc(a,b){a=a|0;b=b|0;var d=0,e=0;xc(a,a,b);xc(b,a,b);d=c[1414]|0;e=c[d+340>>2]|0;if((((e|0)<0?-e|0:e)|0)==1073741824){xc(a,c[d+536>>2]|0,c[d+468>>2]|0)}else{Dc(a,e,c[d+468>>2]|0)}yc(b,c[(c[1414]|0)+468>>2]|0,b);d=c[1414]|0;e=c[d+344>>2]|0;if((((e|0)<0?-e|0:e)|0)==1073741824){yc(b,c[d+540>>2]|0,b);return}else{nb(e,c[d+468>>2]|0);d=c[(c[1414]|0)+468>>2]|0;vc(d,d);yc(b,c[(c[1414]|0)+468>>2]|0,b);return}}function Sc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=c[1414]|0;if((c[d+336>>2]|0)==1){e=b+8|0;zc(c[a+8>>2]|0,c[e>>2]|0,c[d+496>>2]|0);f=a+4|0;g=b+4|0;zc(c[f>>2]|0,c[g>>2]|0,c[(c[1414]|0)+488>>2]|0);h=(Fb(c[(c[1414]|0)+488>>2]|0)|0)==0;i=c[1414]|0;j=c[i+496>>2]|0;if(h){if((Fb(j)|0)==0){k=0;return k|0}Hc(0,0,0,b)|0;k=1;return k|0}if((Cc(j,c[i+488>>2]|0,j)|0)>1){Hc(0,0,0,b)|0;fb(28);k=1;return k|0}else{j=c[1414]|0;i=c[j+496>>2]|0;xc(i,i,c[j+472>>2]|0);j=c[1414]|0;zc(c[j+472>>2]|0,c[f>>2]|0,c[j+468>>2]|0);j=c[(c[1414]|0)+468>>2]|0;zc(j,c[g>>2]|0,j);j=c[1414]|0;zc(c[g>>2]|0,c[j+468>>2]|0,c[j+472>>2]|0);j=c[1414]|0;f=c[j+472>>2]|0;xc(f,c[j+496>>2]|0,f);f=c[e>>2]|0;zc(c[(c[1414]|0)+472>>2]|0,f,f);Bb(c[(c[1414]|0)+468>>2]|0,c[g>>2]|0);c[b>>2]=1;k=1;return k|0}}g=a|0;if((c[g>>2]|0)==1){Bb(c[b+4>>2]|0,c[d+468>>2]|0);Bb(c[b+8>>2]|0,c[(c[1414]|0)+496>>2]|0)}else{f=a+12|0;e=c[f>>2]|0;xc(e,e,c[d+488>>2]|0);d=c[1414]|0;xc(c[b+4>>2]|0,c[d+488>>2]|0,c[d+468>>2]|0);d=c[(c[1414]|0)+488>>2]|0;xc(d,c[f>>2]|0,d);d=c[1414]|0;xc(c[b+8>>2]|0,c[d+488>>2]|0,c[d+496>>2]|0)}d=b|0;if((c[d>>2]|0)==1){f=c[1414]|0;Bb(c[f+532>>2]|0,c[f+488>>2]|0)}else{f=c[b+12>>2]|0;xc(f,f,c[(c[1414]|0)+488>>2]|0)}f=c[1414]|0;xc(c[a+4>>2]|0,c[f+488>>2]|0,c[f+480>>2]|0);if((c[d>>2]|0)!=1){f=c[(c[1414]|0)+488>>2]|0;xc(f,c[b+12>>2]|0,f)}f=c[1414]|0;xc(c[a+8>>2]|0,c[f+488>>2]|0,c[f+484>>2]|0);f=c[1414]|0;e=c[f+468>>2]|0;zc(e,c[f+480>>2]|0,e);e=c[1414]|0;f=c[e+496>>2]|0;zc(f,c[e+484>>2]|0,f);f=(Fb(c[(c[1414]|0)+468>>2]|0)|0)==0;e=c[1414]|0;if(f){if((Fb(c[e+496>>2]|0)|0)==0){k=0;return k|0}Hc(0,0,0,b)|0;k=1;return k|0}f=c[e+480>>2]|0;yc(f,f,c[e+488>>2]|0);e=c[1414]|0;yc(c[e+468>>2]|0,c[e+488>>2]|0,c[e+480>>2]|0);e=c[1414]|0;f=c[e+484>>2]|0;yc(f,f,c[e+488>>2]|0);e=c[1414]|0;yc(c[e+496>>2]|0,c[e+488>>2]|0,c[e+484>>2]|0);e=(c[d>>2]|0)!=1;do{if((c[g>>2]|0)==1){if(e){f=c[b+12>>2]|0;xc(f,c[(c[1414]|0)+468>>2]|0,f);break}else{Bb(c[(c[1414]|0)+468>>2]|0,c[b+12>>2]|0);break}}else{if(e){f=b+12|0;xc(c[f>>2]|0,c[a+12>>2]|0,c[(c[1414]|0)+476>>2]|0);l=f}else{Bb(c[a+12>>2]|0,c[(c[1414]|0)+476>>2]|0);l=b+12|0}f=c[1414]|0;xc(c[f+476>>2]|0,c[f+468>>2]|0,c[l>>2]|0)}}while(0);l=c[1414]|0;a=c[l+468>>2]|0;xc(a,a,c[l+488>>2]|0);l=c[1414]|0;a=c[l+468>>2]|0;xc(a,c[l+488>>2]|0,a);a=c[1414]|0;l=c[a+488>>2]|0;xc(l,c[a+480>>2]|0,l);l=c[1414]|0;a=c[l+496>>2]|0;xc(a,a,c[l+480>>2]|0);l=c[1414]|0;a=b+4|0;zc(c[l+480>>2]|0,c[l+488>>2]|0,c[a>>2]|0);l=c[(c[1414]|0)+488>>2]|0;zc(l,c[a>>2]|0,l);l=c[(c[1414]|0)+488>>2]|0;zc(l,c[a>>2]|0,l);l=c[1414]|0;xc(c[l+496>>2]|0,c[l+488>>2]|0,c[l+472>>2]|0);l=c[1414]|0;a=c[l+468>>2]|0;xc(a,c[l+484>>2]|0,a);a=c[1414]|0;zc(c[a+472>>2]|0,c[a+468>>2]|0,c[a+484>>2]|0);Bc(c[(c[1414]|0)+484>>2]|0,c[b+8>>2]|0);c[d>>2]=0;k=1;return k|0}function Tc(a){a=a|0;od(a+40|0,0,320)|0;c[a+4>>2]=0;c[a>>2]=0;c[a+8>>2]=1732584193;c[a+12>>2]=-271733879;c[a+16>>2]=-1732584194;c[a+20>>2]=271733878;c[a+24>>2]=-1009589776;return}function Uc(a,b){a=a|0;b=b|0;var d=0,e=0;d=a|0;e=a+40+(((c[d>>2]|0)>>>5&15)<<2)|0;c[e>>2]=c[e>>2]<<8|b&255;b=(c[d>>2]|0)+8|0;c[d>>2]=b;do{if((b|0)==0){e=a+4|0;c[e>>2]=(c[e>>2]|0)+1;c[d>>2]=0}else{if((b&511|0)==0){break}return}}while(0);Wc(a);return}function Vc(b,d){b=b|0;d=d|0;var e=0,f=0,g=0;e=b|0;f=c[e>>2]|0;g=c[b+4>>2]|0;Uc(b,128);if((c[e>>2]&511|0)!=448){do{Uc(b,0);}while((c[e>>2]&511|0)!=448)}c[b+96>>2]=g;c[b+100>>2]=f;Wc(b);f=0;do{a[d+f|0]=(c[b+8+(((f|0)/4|0)<<2)>>2]|0)>>>(3-((f|0)%4|0)<<3>>>0);f=f+1|0;}while((f|0)<20);Tc(b);return}function Wc(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;b=16;do{d=c[a+40+(b-8<<2)>>2]^c[a+40+(b-3<<2)>>2]^c[a+40+(b-14<<2)>>2]^c[a+40+(b-16<<2)>>2];c[a+40+(b<<2)>>2]=d<<1|d>>>31;b=b+1|0;}while((b|0)<80);b=a+8|0;d=b;e=int32x4(c[d>>2]|0,c[d+4>>2]|0,c[d+8>>2]|0,c[d+12>>2]|0);d=a+24|0;f=c[d>>2]|0;g=e.x;h=e.y;i=e.z;j=e.w;k=f;l=0;while(1){m=(g<<5|g>>>27)+1518500249+k+((j^i)&h^j)+(c[a+40+(l<<2)>>2]|0)|0;n=h<<30|h>>>2;o=l+1|0;if((o|0)<20){h=g;g=m;k=j;j=i;i=n;l=o}else{p=m;q=g;r=n;s=i;t=j;u=20;break}}while(1){j=(p<<5|p>>>27)+1859775393+(r^q^s)+t+(c[a+40+(u<<2)>>2]|0)|0;i=q<<30|q>>>2;g=u+1|0;if((g|0)<40){q=p;p=j;t=s;s=r;r=i;u=g}else{v=j;w=p;x=i;y=r;z=s;A=40;break}}while(1){s=(v<<5|v>>>27)-1894007588+z+(y&(x|w)|x&w)+(c[a+40+(A<<2)>>2]|0)|0;r=w<<30|w>>>2;p=A+1|0;if((p|0)<60){w=v;v=s;z=y;y=x;x=r;A=p}else{B=s;C=v;D=r;E=x;F=y;G=60;break}}while(1){H=(B<<5|B>>>27)-899497514+(D^C^E)+F+(c[a+40+(G<<2)>>2]|0)|0;I=C<<30|C>>>2;y=G+1|0;if((y|0)<80){C=B;B=H;F=E;E=D;D=I;G=y}else{break}}G=SIMD.int32x4.add(e,SIMD.int32x4.withW(SIMD.int32x4.withZ(SIMD.int32x4.withY(SIMD.int32x4.withX(int32x4.splat(0),H),B),I),D));D=b;c[D>>2]=G.x,c[D+4>>2]=G.y,c[D+8>>2]=G.z,c[D+12>>2]=G.w;c[d>>2]=f+E;return}function Xc(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;f=c[1414]|0;if((c[f+552>>2]|0)!=0){return}g=f+28|0;c[g>>2]=(c[g>>2]|0)+1;g=c[1414]|0;f=c[g+28>>2]|0;do{if((f|0)<24){c[g+32+(f<<2)>>2]=107;if((c[(c[1414]|0)+576>>2]|0)==0){break}hb()}}while(0);if((Fb(b)|0)==0){lb(d);nb(2,e);vc(e,e);f=(c[1414]|0)+28|0;c[f>>2]=(c[f>>2]|0)-1;return}do{if((Fb(b)|0)!=1){if((Fb(b)|0)==-1){break}Bb(a,c[(c[1414]|0)+476>>2]|0);nb(2,c[(c[1414]|0)+480>>2]|0);f=c[(c[1414]|0)+480>>2]|0;vc(f,f);f=c[1414]|0;Bb(c[f+480>>2]|0,c[f+496>>2]|0);f=c[1414]|0;Bb(c[f+476>>2]|0,c[f+500>>2]|0);Bb(b,c[(c[1414]|0)+468>>2]|0);xb(1,c[(c[1414]|0)+468>>2]|0);f=c[(c[1414]|0)+468>>2]|0;Wb(f,1,f);f=c[1414]|0;g=(c[f>>2]|0)==(c[f+16>>2]|0);h=sc(c[f+468>>2]|0)|0;a:do{if(g){if((h|0)>0){i=h}else{break}do{i=i-1|0;f=c[1414]|0;j=c[f+20>>2]|0;if((j|0)==0){k=f}else{Ka[j&1]()|0;k=c[1414]|0}j=(Kb(c[k+468>>2]|0,i)|0)==0;f=c[1414]|0;if(j){j=c[f+500>>2]|0;xc(j,c[f+496>>2]|0,j);j=c[1414]|0;l=c[j+500>>2]|0;zc(l,c[j+476>>2]|0,l);l=c[(c[1414]|0)+496>>2]|0;xc(l,l,l);l=c[1414]|0;j=c[l+496>>2]|0;zc(j,c[l+480>>2]|0,j)}else{j=c[f+496>>2]|0;xc(j,c[f+500>>2]|0,j);j=c[1414]|0;f=c[j+496>>2]|0;zc(f,c[j+476>>2]|0,f);f=c[(c[1414]|0)+500>>2]|0;xc(f,f,f);f=c[1414]|0;j=c[f+500>>2]|0;zc(j,c[f+480>>2]|0,j)}}while((i|0)>0)}else{tc(h-1|0,c[(c[1414]|0)+472>>2]|0);j=c[1414]|0;if((c[j+552>>2]|0)==0){m=j}else{break}do{if((Fb(c[m+472>>2]|0)|0)==0){break a}j=c[1414]|0;f=(Gb(c[j+468>>2]|0,c[j+472>>2]|0)|0)>-1;j=c[1414]|0;if(f){f=c[j+496>>2]|0;xc(f,c[j+500>>2]|0,f);f=c[1414]|0;l=c[f+496>>2]|0;zc(l,c[f+476>>2]|0,l);l=c[(c[1414]|0)+500>>2]|0;xc(l,l,l);l=c[1414]|0;f=c[l+500>>2]|0;zc(f,c[l+480>>2]|0,f);f=c[1414]|0;l=c[f+468>>2]|0;Ub(l,c[f+472>>2]|0,l)}else{l=c[j+500>>2]|0;xc(l,c[j+496>>2]|0,l);l=c[1414]|0;j=c[l+500>>2]|0;zc(j,c[l+476>>2]|0,j);j=c[(c[1414]|0)+496>>2]|0;xc(j,j,j);j=c[1414]|0;l=c[j+496>>2]|0;zc(l,c[j+480>>2]|0,l)}l=c[(c[1414]|0)+472>>2]|0;$b(l,2,l)|0;m=c[1414]|0;}while((c[m+552>>2]|0)==0)}}while(0);Bb(c[(c[1414]|0)+500>>2]|0,e);if((e|0)!=(d|0)){Bb(c[(c[1414]|0)+496>>2]|0,d)}h=(c[1414]|0)+28|0;c[h>>2]=(c[h>>2]|0)-1;return}}while(0);nb(2,d);vc(d,d);Bb(a,e);e=(c[1414]|0)+28|0;c[e>>2]=(c[e>>2]|0)-1;return}function Yc(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0;g=Dd(b,0,a,0)|0;a=td(g,E,d,0)|0;d=E;g=e;e=0;b=Ed(a,d,g,e)|0;h=Dd(b|0,E&0,g,e)|0;e=ud(a,d,h,E)|0;c[f>>2]=e;return b|0}function Zc(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0;f=b;b=0;g=d;d=0;h=Ed(f|0,a|b,g,d)|0;a=Dd(h|0,E&0,g,d)|0;d=ud(f,b,a,E)|0;c[e>>2]=d;return h|0}function _c(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;f=Dd(b,0,a,0)|0;a=td(f,E,d,0)|0;c[e>>2]=a;return E|0}function $c(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;f=Dd(b,0,a,0)|0;a=td(c[d>>2]|0,0,f,E)|0;f=td(a,E,c[e>>2]|0,0)|0;c[e>>2]=f;c[d>>2]=E;return}function ad(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;h=i;i=i+464|0;j=h+80|0;k=h+440|0;l=h|0;pd(l|0,1064,15)|0;m=h+16|0;pd(m|0,1048,15)|0;n=h+32|0;a[n]=a[1120]|0;a[n+1|0]=a[1121]|0;a[n+2|0]=a[1122]|0;a[n+3|0]=a[1123]|0;a[n+4|0]=a[1124]|0;o=h+40|0;a[o]=a[1112]|0;a[o+1|0]=a[1113]|0;a[o+2|0]=a[1114]|0;a[o+3|0]=a[1115]|0;a[o+4|0]=a[1116]|0;p=h+48|0;pd(p|0,1096,15)|0;q=h+64|0;pd(q|0,1080,15)|0;c[(sb(800,16)|0)+564>>2]=16;Tc(j);r=a[b]|0;if(r<<24>>24!=0){s=b;b=r;do{s=s+1|0;Uc(j,b<<24>>24);b=a[s]|0;}while(b<<24>>24!=0)}b=a[d]|0;if(b<<24>>24!=0){s=d;d=b;do{s=s+1|0;Uc(j,d<<24>>24);d=a[s]|0;}while(d<<24>>24!=0)}d=k|0;Vc(j,d);j=ob(0)|0;bc(20,d,j);d=ob(0)|0;kc(d,n)|0;n=ob(0)|0;kc(n,o)|0;o=ob(0)|0;kc(o,l)|0;l=ob(0)|0;kc(l,m)|0;m=ob(0)|0;kc(m,p)|0;p=ob(0)|0;kc(p,q)|0;q=ob(0)|0;kc(q,e)|0;e=ob(0)|0;k=ob(0)|0;s=ob(0)|0;b=ob(0)|0;r=ob(0)|0;t=ob(17)|0;u=ob(53)|0;v=ob(905461)|0;w=ob(60291817)|0;Vb(e,0,e);x=Mb()|0;y=Mb()|0;Fc(d,n,o,0);Hc(m,p,0,x)|0;while(1){Vb(e,1,e);if((fc(e,t)|0)!=0){continue}if((fc(e,u)|0)!=0){continue}if((fc(e,v)|0)!=0){continue}if((fc(e,w)|0)!=0){continue}Qc(e,x,y)|0;Ic(y,k,k)|0;ec(k,l,l);if((Gb(k,r)|0)==0){continue}pc(e,l,b,b,b)|0;gc(q,k,j,l,l,s);gc(s,b,b,l,l,s);if((fc(s,t)|0)!=0){continue}if((fc(s,u)|0)!=0){continue}if((fc(s,v)|0)!=0){continue}if((fc(s,w)|0)==0){break}}mc(k,f)|0;mc(s,g)|0;i=h;return}function bd(b){b=b|0;var c=0,d=0,e=0;c=a[b]|0;if(c<<24>>24==0){return b|0}else{d=b;e=c}do{a[d]=qa(e<<24>>24|0)|0;d=d+1|0;e=a[d]|0;}while(e<<24>>24!=0);return b|0}function cd(b,c){b=b|0;c=c|0;var d=0,e=0;d=qd(c|0)|0;if((d|0)>-1){e=d;while(1){a[c+(e+1)|0]=a[c+e|0]|0;if((e|0)>0){e=e-1|0}else{break}}}a[c]=a[b]|0;return b|0}function dd(b){b=b|0;var c=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;c=a[b]|0;if(c<<24>>24==0){d=0;e=0;f=0;g=0;h=1016+g|0;i=a[h]|0;j=b+3|0;a[j]=i;k=1016+f|0;l=a[k]|0;m=b+2|0;a[m]=l;n=1016+e|0;o=a[n]|0;p=b+1|0;a[p]=o;q=1016+d|0;r=a[q]|0;a[b]=r;s=b+4|0;a[s]=0;return}else{t=0;u=0;v=0;w=c}while(1){do{if((w-48&255)>>>0<10>>>0){c=td(u<<4|0>>>28,t<<4|u>>>28,-48,-1)|0;x=td(c,E,w<<24>>24,w<<24>>24<0|0?-1:0)|0;y=E;z=x}else{if((w-65&255)>>>0<6>>>0){x=td(u<<4|0>>>28,t<<4|u>>>28,-55,-1)|0;c=td(x,E,w<<24>>24,w<<24>>24<0|0?-1:0)|0;y=E;z=c;break}if((w-97&255)>>>0>=6>>>0){y=t;z=u;break}c=td(u<<4|0>>>28,t<<4|u>>>28,-87,-1)|0;x=td(c,E,w<<24>>24,w<<24>>24<0|0?-1:0)|0;y=E;z=x}}while(0);x=v+1|0;c=a[b+x|0]|0;if(c<<24>>24==0){break}else{t=y;u=z;v=x;w=c}}d=(z>>>15|y<<17)&31;e=(z>>>10|y<<22)&31;f=(z>>>5|y<<27)&31;g=z&31;h=1016+g|0;i=a[h]|0;j=b+3|0;a[j]=i;k=1016+f|0;l=a[k]|0;m=b+2|0;a[m]=l;n=1016+e|0;o=a[n]|0;p=b+1|0;a[p]=o;q=1016+d|0;r=a[q]|0;a[b]=r;s=b+4|0;a[s]=0;return}function ed(){return 8}function fd(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0;e=i;i=i+8|0;f=e|0;bd(b)|0;bd(c)|0;bd(d)|0;do{if((la(b|0,1792,3)|0)==0){g=a[c]|0;if((g<<24>>24|0)==77|(g<<24>>24|0)==53){h=c;break}h=hd(c,f|0)|0}else{h=c}}while(0);f=nd(64,1)|0;g=nd(64,1)|0;ad(b,h,d,f,g);d=nd(128,1)|0;do{if((la(b|0,1792,3)|0)==0){h=a[c]|0;if((h<<24>>24|0)==77|(h<<24>>24|0)==53){break}id(f,g,c,d);md(f);md(g);i=e;return d|0}}while(0);jd(f,g,c,d);md(f);md(g);i=e;return d|0}function gd(a){a=a|0;var b=0,c=0;bd(a)|0;do{if((la(a|0,1776,3)|0)==0){b=5616}else{if((la(a|0,1760,3)|0)==0){b=5600;break}if((la(a|0,1744,3)|0)==0){b=5600;break}if((la(a|0,1696,3)|0)==0){b=5584;break}c=(la(a|0,1792,3)|0)==0;b=c?5632:5648}}while(0);return b|0}function hd(b,d){b=b|0;d=d|0;var e=0,f=0,g=0;e=0;f=0;do{g=kd(a[b+(c[24+(e<<2)>>2]|0)|0]|0)|0;f=g|f<<5;e=e+1|0;}while((e|0)<4);a[d]=a[1128+(g&15)|0]|0;a[d+1|0]=a[1128+(f>>>4&15)|0]|0;a[d+2|0]=a[1128+(f>>>8&15)|0]|0;a[d+3|0]=a[1128+(f>>>12&15)|0]|0;a[d+4|0]=a[1128+(f>>>16&15)|0]|0;a[d+5|0]=0;return d|0}function id(b,c,d,e){b=b|0;c=c|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;f=sa(b|0,0,16)|0;b=0;g=E;h=f;while(1){f=524288;i=h<<4|0>>>28;j=g<<4|h>>>28;if(g>>>0<f>>>0|g>>>0==f>>>0&h>>>0<0>>>0){b=b+1|0;g=j;h=i}else{break}}h=b|i;i=((b|0)<0|0?-1:0)|j;a[e+4|0]=a[1128+(h&31)|0]|0;a[e+11|0]=a[1128+((h>>>5|i<<27)&31)|0]|0;a[e+16|0]=a[1128+((h>>>10|i<<22)&31)|0]|0;a[e+23|0]=a[1128+((h>>>15|i<<17)&31)|0]|0;a[e]=a[1128+((h>>>20|i<<12)&31)|0]|0;a[e+24|0]=a[1128+((h>>>25|i<<7)&31)|0]|0;a[e+6|0]=a[1128+((h>>>30|i<<2)&31)|0]|0;a[e+22|0]=a[1128+((i>>>3|0<<29)&31)|0]|0;a[e+8|0]=a[1128+((i>>>8|0<<24)&31)|0]|0;a[e+20|0]=a[1128+((i>>>13|0<<19)&31)|0]|0;a[e+18|0]=a[1128+((i>>>18|0<<14)&31)|0]|0;a[e+25|0]=a[1128+((i>>>23|0<<9)&31)|0]|0;i=sa(c|0,0,16)|0;c=E;h=524288;j=i<<4|0>>>28;b=c<<4|i>>>28;if(c>>>0<h>>>0|c>>>0==h>>>0&i>>>0<0>>>0){i=0;h=b;c=j;do{g=c|5;f=h|0;i=i+1|0;k=524288;c=g<<4|0>>>28;h=f<<4|g>>>28}while(f>>>0<k>>>0|f>>>0==k>>>0&g>>>0<0>>>0);l=(i|0)<0|0?-1:0;m=i;n=h;o=c}else{l=0;m=0;n=b;o=j}j=m|o;o=l|n;a[e+3|0]=a[1128+(j&31)|0]|0;a[e+14|0]=a[1128+((j>>>5|o<<27)&31)|0]|0;a[e+19|0]=a[1128+((j>>>10|o<<22)&31)|0]|0;a[e+9|0]=a[1128+((j>>>15|o<<17)&31)|0]|0;a[e+26|0]=a[1128+((j>>>20|o<<12)&31)|0]|0;a[e+5|0]=a[1128+((j>>>25|o<<7)&31)|0]|0;a[e+1|0]=a[1128+((j>>>30|o<<2)&31)|0]|0;a[e+10|0]=a[1128+((o>>>3|0<<29)&31)|0]|0;a[e+12|0]=a[1128+((o>>>8|0<<24)&31)|0]|0;a[e+13|0]=a[1128+((o>>>13|0<<19)&31)|0]|0;a[e+15|0]=a[1128+((o>>>18|0<<14)&31)|0]|0;a[e+21|0]=a[1128+((o>>>23|0<<9)&31)|0]|0;a[e+2|0]=a[d]|0;a[e+7|0]=a[d+1|0]|0;a[e+17|0]=a[d+2|0]|0;a[e+27|0]=a[d+3|0]|0;a[e+28|0]=0;return}function jd(b,c,d,e){b=b|0;c=c|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;if((qd(b|0)|0)>>>0<14>>>0){do{cd(1648,b)|0;}while((qd(b|0)|0)>>>0<14>>>0)}if((qd(c|0)|0)>>>0<14>>>0){do{cd(1648,c)|0;}while((qd(c|0)|0)>>>0<14>>>0)}f=nd(128,1)|0;g=nd(128,1)|0;h=nd(6,1)|0;rd(f|0,b|0)|0;b=f+(qd(f|0)|0)|0;w=48;a[b]=w;w=w>>8;a[b+1|0]=w;sd(f|0,c|0)|0;c=f+(qd(f|0)|0)|0;w=48;a[c]=w;w=w>>8;a[c+1|0]=w;if((a[f]|0)!=0){c=0;do{b=f+c|0;a[h]=a[b]|0;a[h+1|0]=a[b+1|0]|0;a[h+2|0]=a[b+2|0]|0;a[h+3|0]=a[b+3|0]|0;a[h+4|0]=a[b+4|0]|0;dd(h);sd(g|0,h|0)|0;c=c+5|0;}while(c>>>0<(qd(f|0)|0)>>>0)}c=d+1|0;b=d+2|0;i=d+3|0;j=0;k=0;while(1){switch(k|0){case 1:{a[e+1|0]=a[d]|0;l=j;break};case 7:{a[e+7|0]=45;l=j;break};case 10:{a[e+10|0]=a[c]|0;l=j;break};case 15:{a[e+15|0]=45;l=j;break};case 23:{a[e+23|0]=45;l=j;break};case 28:{a[e+28|0]=a[i]|0;l=j;break};case 19:{a[e+19|0]=a[b]|0;l=j;break};default:{a[e+k|0]=a[g+j|0]|0;l=j+1|0}}m=k+1|0;if(l>>>0>(qd(g|0)|0)>>>0){break}else{j=l;k=m}}a[e+m|0]=0;md(f);md(h);md(g);return}function kd(b){b=b|0;var c=0,d=0,e=0,f=0;c=0;while(1){d=c+1|0;if((a[1128+c|0]|0)==b<<24>>24){e=c;f=4;break}if(d>>>0<32>>>0){c=d}else{e=d;f=4;break}}if((f|0)==4){return e|0}return 0}function ld(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ba=0,ca=0,da=0,ea=0,fa=0,ga=0,ha=0,ia=0,ja=0,ka=0,la=0,na=0,oa=0,qa=0,ra=0,sa=0,ta=0,ua=0,va=0,wa=0,xa=0,ya=0,za=0,Aa=0,Ba=0,Ca=0,Da=0,Ea=0,Ia=0,Ja=0,Ka=0,La=0,Ma=0,Na=0,Oa=0;do{if(a>>>0<245>>>0){if(a>>>0<11>>>0){b=16}else{b=a+11&-8}d=b>>>3;e=c[1424]|0;f=e>>>(d>>>0);if((f&3|0)!=0){g=(f&1^1)+d|0;h=g<<1;i=5736+(h<<2)|0;j=5736+(h+2<<2)|0;h=c[j>>2]|0;k=h+8|0;l=c[k>>2]|0;do{if((i|0)==(l|0)){c[1424]=e&~(1<<g)}else{if(l>>>0<(c[1428]|0)>>>0){pa();return 0}m=l+12|0;if((c[m>>2]|0)==(h|0)){c[m>>2]=i;c[j>>2]=l;break}else{pa();return 0}}}while(0);l=g<<3;c[h+4>>2]=l|3;j=h+(l|4)|0;c[j>>2]=c[j>>2]|1;n=k;return n|0}if(b>>>0<=(c[1426]|0)>>>0){o=b;break}if((f|0)!=0){j=2<<d;l=f<<d&(j|-j);j=(l&-l)-1|0;l=j>>>12&16;i=j>>>(l>>>0);j=i>>>5&8;m=i>>>(j>>>0);i=m>>>2&4;p=m>>>(i>>>0);m=p>>>1&2;q=p>>>(m>>>0);p=q>>>1&1;r=(j|l|i|m|p)+(q>>>(p>>>0))|0;p=r<<1;q=5736+(p<<2)|0;m=5736+(p+2<<2)|0;p=c[m>>2]|0;i=p+8|0;l=c[i>>2]|0;do{if((q|0)==(l|0)){c[1424]=e&~(1<<r)}else{if(l>>>0<(c[1428]|0)>>>0){pa();return 0}j=l+12|0;if((c[j>>2]|0)==(p|0)){c[j>>2]=q;c[m>>2]=l;break}else{pa();return 0}}}while(0);l=r<<3;m=l-b|0;c[p+4>>2]=b|3;q=p;e=q+b|0;c[q+(b|4)>>2]=m|1;c[q+l>>2]=m;l=c[1426]|0;if((l|0)!=0){q=c[1429]|0;d=l>>>3;l=d<<1;f=5736+(l<<2)|0;k=c[1424]|0;h=1<<d;do{if((k&h|0)==0){c[1424]=k|h;s=f;t=5736+(l+2<<2)|0}else{d=5736+(l+2<<2)|0;g=c[d>>2]|0;if(g>>>0>=(c[1428]|0)>>>0){s=g;t=d;break}pa();return 0}}while(0);c[t>>2]=q;c[s+12>>2]=q;c[q+8>>2]=s;c[q+12>>2]=f}c[1426]=m;c[1429]=e;n=i;return n|0}l=c[1425]|0;if((l|0)==0){o=b;break}h=(l&-l)-1|0;l=h>>>12&16;k=h>>>(l>>>0);h=k>>>5&8;p=k>>>(h>>>0);k=p>>>2&4;r=p>>>(k>>>0);p=r>>>1&2;d=r>>>(p>>>0);r=d>>>1&1;g=c[6e3+((h|l|k|p|r)+(d>>>(r>>>0))<<2)>>2]|0;r=g;d=g;p=(c[g+4>>2]&-8)-b|0;while(1){g=c[r+16>>2]|0;if((g|0)==0){k=c[r+20>>2]|0;if((k|0)==0){break}else{u=k}}else{u=g}g=(c[u+4>>2]&-8)-b|0;k=g>>>0<p>>>0;r=u;d=k?u:d;p=k?g:p}r=d;i=c[1428]|0;if(r>>>0<i>>>0){pa();return 0}e=r+b|0;m=e;if(r>>>0>=e>>>0){pa();return 0}e=c[d+24>>2]|0;f=c[d+12>>2]|0;do{if((f|0)==(d|0)){q=d+20|0;g=c[q>>2]|0;if((g|0)==0){k=d+16|0;l=c[k>>2]|0;if((l|0)==0){v=0;break}else{w=l;x=k}}else{w=g;x=q}while(1){q=w+20|0;g=c[q>>2]|0;if((g|0)!=0){w=g;x=q;continue}q=w+16|0;g=c[q>>2]|0;if((g|0)==0){break}else{w=g;x=q}}if(x>>>0<i>>>0){pa();return 0}else{c[x>>2]=0;v=w;break}}else{q=c[d+8>>2]|0;if(q>>>0<i>>>0){pa();return 0}g=q+12|0;if((c[g>>2]|0)!=(d|0)){pa();return 0}k=f+8|0;if((c[k>>2]|0)==(d|0)){c[g>>2]=f;c[k>>2]=q;v=f;break}else{pa();return 0}}}while(0);a:do{if((e|0)!=0){f=d+28|0;i=6e3+(c[f>>2]<<2)|0;do{if((d|0)==(c[i>>2]|0)){c[i>>2]=v;if((v|0)!=0){break}c[1425]=c[1425]&~(1<<c[f>>2]);break a}else{if(e>>>0<(c[1428]|0)>>>0){pa();return 0}q=e+16|0;if((c[q>>2]|0)==(d|0)){c[q>>2]=v}else{c[e+20>>2]=v}if((v|0)==0){break a}}}while(0);if(v>>>0<(c[1428]|0)>>>0){pa();return 0}c[v+24>>2]=e;f=c[d+16>>2]|0;do{if((f|0)!=0){if(f>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[v+16>>2]=f;c[f+24>>2]=v;break}}}while(0);f=c[d+20>>2]|0;if((f|0)==0){break}if(f>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[v+20>>2]=f;c[f+24>>2]=v;break}}}while(0);if(p>>>0<16>>>0){e=p+b|0;c[d+4>>2]=e|3;f=r+(e+4)|0;c[f>>2]=c[f>>2]|1}else{c[d+4>>2]=b|3;c[r+(b|4)>>2]=p|1;c[r+(p+b)>>2]=p;f=c[1426]|0;if((f|0)!=0){e=c[1429]|0;i=f>>>3;f=i<<1;q=5736+(f<<2)|0;k=c[1424]|0;g=1<<i;do{if((k&g|0)==0){c[1424]=k|g;y=q;z=5736+(f+2<<2)|0}else{i=5736+(f+2<<2)|0;l=c[i>>2]|0;if(l>>>0>=(c[1428]|0)>>>0){y=l;z=i;break}pa();return 0}}while(0);c[z>>2]=e;c[y+12>>2]=e;c[e+8>>2]=y;c[e+12>>2]=q}c[1426]=p;c[1429]=m}n=d+8|0;return n|0}else{if(a>>>0>4294967231>>>0){o=-1;break}f=a+11|0;g=f&-8;k=c[1425]|0;if((k|0)==0){o=g;break}r=-g|0;i=f>>>8;do{if((i|0)==0){A=0}else{if(g>>>0>16777215>>>0){A=31;break}f=(i+1048320|0)>>>16&8;l=i<<f;h=(l+520192|0)>>>16&4;j=l<<h;l=(j+245760|0)>>>16&2;B=14-(h|f|l)+(j<<l>>>15)|0;A=g>>>((B+7|0)>>>0)&1|B<<1}}while(0);i=c[6e3+(A<<2)>>2]|0;b:do{if((i|0)==0){C=0;D=r;E=0}else{if((A|0)==31){F=0}else{F=25-(A>>>1)|0}d=0;m=r;p=i;q=g<<F;e=0;while(1){B=c[p+4>>2]&-8;l=B-g|0;if(l>>>0<m>>>0){if((B|0)==(g|0)){C=p;D=l;E=p;break b}else{G=p;H=l}}else{G=d;H=m}l=c[p+20>>2]|0;B=c[p+16+(q>>>31<<2)>>2]|0;j=(l|0)==0|(l|0)==(B|0)?e:l;if((B|0)==0){C=G;D=H;E=j;break}else{d=G;m=H;p=B;q=q<<1;e=j}}}}while(0);if((E|0)==0&(C|0)==0){i=2<<A;r=k&(i|-i);if((r|0)==0){o=g;break}i=(r&-r)-1|0;r=i>>>12&16;e=i>>>(r>>>0);i=e>>>5&8;q=e>>>(i>>>0);e=q>>>2&4;p=q>>>(e>>>0);q=p>>>1&2;m=p>>>(q>>>0);p=m>>>1&1;I=c[6e3+((i|r|e|q|p)+(m>>>(p>>>0))<<2)>>2]|0}else{I=E}if((I|0)==0){J=D;K=C}else{p=I;m=D;q=C;while(1){e=(c[p+4>>2]&-8)-g|0;r=e>>>0<m>>>0;i=r?e:m;e=r?p:q;r=c[p+16>>2]|0;if((r|0)!=0){p=r;m=i;q=e;continue}r=c[p+20>>2]|0;if((r|0)==0){J=i;K=e;break}else{p=r;m=i;q=e}}}if((K|0)==0){o=g;break}if(J>>>0>=((c[1426]|0)-g|0)>>>0){o=g;break}q=K;m=c[1428]|0;if(q>>>0<m>>>0){pa();return 0}p=q+g|0;k=p;if(q>>>0>=p>>>0){pa();return 0}e=c[K+24>>2]|0;i=c[K+12>>2]|0;do{if((i|0)==(K|0)){r=K+20|0;d=c[r>>2]|0;if((d|0)==0){j=K+16|0;B=c[j>>2]|0;if((B|0)==0){L=0;break}else{M=B;N=j}}else{M=d;N=r}while(1){r=M+20|0;d=c[r>>2]|0;if((d|0)!=0){M=d;N=r;continue}r=M+16|0;d=c[r>>2]|0;if((d|0)==0){break}else{M=d;N=r}}if(N>>>0<m>>>0){pa();return 0}else{c[N>>2]=0;L=M;break}}else{r=c[K+8>>2]|0;if(r>>>0<m>>>0){pa();return 0}d=r+12|0;if((c[d>>2]|0)!=(K|0)){pa();return 0}j=i+8|0;if((c[j>>2]|0)==(K|0)){c[d>>2]=i;c[j>>2]=r;L=i;break}else{pa();return 0}}}while(0);c:do{if((e|0)!=0){i=K+28|0;m=6e3+(c[i>>2]<<2)|0;do{if((K|0)==(c[m>>2]|0)){c[m>>2]=L;if((L|0)!=0){break}c[1425]=c[1425]&~(1<<c[i>>2]);break c}else{if(e>>>0<(c[1428]|0)>>>0){pa();return 0}r=e+16|0;if((c[r>>2]|0)==(K|0)){c[r>>2]=L}else{c[e+20>>2]=L}if((L|0)==0){break c}}}while(0);if(L>>>0<(c[1428]|0)>>>0){pa();return 0}c[L+24>>2]=e;i=c[K+16>>2]|0;do{if((i|0)!=0){if(i>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[L+16>>2]=i;c[i+24>>2]=L;break}}}while(0);i=c[K+20>>2]|0;if((i|0)==0){break}if(i>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[L+20>>2]=i;c[i+24>>2]=L;break}}}while(0);d:do{if(J>>>0<16>>>0){e=J+g|0;c[K+4>>2]=e|3;i=q+(e+4)|0;c[i>>2]=c[i>>2]|1}else{c[K+4>>2]=g|3;c[q+(g|4)>>2]=J|1;c[q+(J+g)>>2]=J;i=J>>>3;if(J>>>0<256>>>0){e=i<<1;m=5736+(e<<2)|0;r=c[1424]|0;j=1<<i;do{if((r&j|0)==0){c[1424]=r|j;O=m;P=5736+(e+2<<2)|0}else{i=5736+(e+2<<2)|0;d=c[i>>2]|0;if(d>>>0>=(c[1428]|0)>>>0){O=d;P=i;break}pa();return 0}}while(0);c[P>>2]=k;c[O+12>>2]=k;c[q+(g+8)>>2]=O;c[q+(g+12)>>2]=m;break}e=p;j=J>>>8;do{if((j|0)==0){Q=0}else{if(J>>>0>16777215>>>0){Q=31;break}r=(j+1048320|0)>>>16&8;i=j<<r;d=(i+520192|0)>>>16&4;B=i<<d;i=(B+245760|0)>>>16&2;l=14-(d|r|i)+(B<<i>>>15)|0;Q=J>>>((l+7|0)>>>0)&1|l<<1}}while(0);j=6e3+(Q<<2)|0;c[q+(g+28)>>2]=Q;c[q+(g+20)>>2]=0;c[q+(g+16)>>2]=0;m=c[1425]|0;l=1<<Q;if((m&l|0)==0){c[1425]=m|l;c[j>>2]=e;c[q+(g+24)>>2]=j;c[q+(g+12)>>2]=e;c[q+(g+8)>>2]=e;break}l=c[j>>2]|0;if((Q|0)==31){R=0}else{R=25-(Q>>>1)|0}e:do{if((c[l+4>>2]&-8|0)==(J|0)){S=l}else{j=l;m=J<<R;while(1){T=j+16+(m>>>31<<2)|0;i=c[T>>2]|0;if((i|0)==0){break}if((c[i+4>>2]&-8|0)==(J|0)){S=i;break e}else{j=i;m=m<<1}}if(T>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[T>>2]=e;c[q+(g+24)>>2]=j;c[q+(g+12)>>2]=e;c[q+(g+8)>>2]=e;break d}}}while(0);l=S+8|0;m=c[l>>2]|0;i=c[1428]|0;if(S>>>0>=i>>>0&m>>>0>=i>>>0){c[m+12>>2]=e;c[l>>2]=e;c[q+(g+8)>>2]=m;c[q+(g+12)>>2]=S;c[q+(g+24)>>2]=0;break}else{pa();return 0}}}while(0);n=K+8|0;return n|0}}while(0);K=c[1426]|0;if(K>>>0>=o>>>0){S=K-o|0;T=c[1429]|0;if(S>>>0>15>>>0){J=T;c[1429]=J+o;c[1426]=S;c[J+(o+4)>>2]=S|1;c[J+K>>2]=S;c[T+4>>2]=o|3}else{c[1426]=0;c[1429]=0;c[T+4>>2]=K|3;S=T+(K+4)|0;c[S>>2]=c[S>>2]|1}n=T+8|0;return n|0}T=c[1427]|0;if(T>>>0>o>>>0){S=T-o|0;c[1427]=S;T=c[1430]|0;K=T;c[1430]=K+o;c[K+(o+4)>>2]=S|1;c[T+4>>2]=o|3;n=T+8|0;return n|0}do{if((c[1416]|0)==0){T=ma(30)|0;if((T-1&T|0)==0){c[1418]=T;c[1417]=T;c[1419]=-1;c[1420]=-1;c[1421]=0;c[1535]=0;c[1416]=(Ha(0)|0)&-16^1431655768;break}else{pa();return 0}}}while(0);T=o+48|0;S=c[1418]|0;K=o+47|0;J=S+K|0;R=-S|0;S=J&R;if(S>>>0<=o>>>0){n=0;return n|0}Q=c[1534]|0;do{if((Q|0)!=0){O=c[1532]|0;P=O+S|0;if(P>>>0<=O>>>0|P>>>0>Q>>>0){n=0}else{break}return n|0}}while(0);f:do{if((c[1535]&4|0)==0){Q=c[1430]|0;g:do{if((Q|0)==0){U=181}else{P=Q;O=6144;while(1){V=O|0;L=c[V>>2]|0;if(L>>>0<=P>>>0){W=O+4|0;if((L+(c[W>>2]|0)|0)>>>0>P>>>0){break}}L=c[O+8>>2]|0;if((L|0)==0){U=181;break g}else{O=L}}if((O|0)==0){U=181;break}P=J-(c[1427]|0)&R;if(P>>>0>=2147483647>>>0){X=0;break}e=Fa(P|0)|0;if((e|0)==((c[V>>2]|0)+(c[W>>2]|0)|0)){Y=e;Z=P;U=190}else{_=P;$=e;U=191}}}while(0);do{if((U|0)==181){Q=Fa(0)|0;if((Q|0)==-1){X=0;break}e=Q;P=c[1417]|0;L=P-1|0;if((L&e|0)==0){aa=S}else{aa=S-e+(L+e&-P)|0}P=c[1532]|0;e=P+aa|0;if(!(aa>>>0>o>>>0&aa>>>0<2147483647>>>0)){X=0;break}L=c[1534]|0;if((L|0)!=0){if(e>>>0<=P>>>0|e>>>0>L>>>0){X=0;break}}L=Fa(aa|0)|0;if((L|0)==(Q|0)){Y=Q;Z=aa;U=190}else{_=aa;$=L;U=191}}}while(0);h:do{if((U|0)==190){if((Y|0)==-1){X=Z}else{ba=Z;ca=Y;U=201;break f}}else if((U|0)==191){L=-_|0;do{if(($|0)!=-1&_>>>0<2147483647>>>0&T>>>0>_>>>0){Q=c[1418]|0;e=K-_+Q&-Q;if(e>>>0>=2147483647>>>0){da=_;break}if((Fa(e|0)|0)==-1){Fa(L|0)|0;X=0;break h}else{da=e+_|0;break}}else{da=_}}while(0);if(($|0)==-1){X=0}else{ba=da;ca=$;U=201;break f}}}while(0);c[1535]=c[1535]|4;ea=X;U=198}else{ea=0;U=198}}while(0);do{if((U|0)==198){if(S>>>0>=2147483647>>>0){break}X=Fa(S|0)|0;$=Fa(0)|0;if(!((X|0)!=-1&($|0)!=-1&X>>>0<$>>>0)){break}da=$-X|0;$=da>>>0>(o+40|0)>>>0;if($){ba=$?da:ea;ca=X;U=201}}}while(0);do{if((U|0)==201){ea=(c[1532]|0)+ba|0;c[1532]=ea;if(ea>>>0>(c[1533]|0)>>>0){c[1533]=ea}ea=c[1430]|0;i:do{if((ea|0)==0){S=c[1428]|0;if((S|0)==0|ca>>>0<S>>>0){c[1428]=ca}c[1536]=ca;c[1537]=ba;c[1539]=0;c[1433]=c[1416];c[1432]=-1;S=0;do{X=S<<1;da=5736+(X<<2)|0;c[5736+(X+3<<2)>>2]=da;c[5736+(X+2<<2)>>2]=da;S=S+1|0;}while(S>>>0<32>>>0);S=ca+8|0;if((S&7|0)==0){fa=0}else{fa=-S&7}S=ba-40-fa|0;c[1430]=ca+fa;c[1427]=S;c[ca+(fa+4)>>2]=S|1;c[ca+(ba-36)>>2]=40;c[1431]=c[1420]}else{S=6144;while(1){ga=c[S>>2]|0;ha=S+4|0;ia=c[ha>>2]|0;if((ca|0)==(ga+ia|0)){U=213;break}da=c[S+8>>2]|0;if((da|0)==0){break}else{S=da}}do{if((U|0)==213){if((c[S+12>>2]&8|0)!=0){break}da=ea;if(!(da>>>0>=ga>>>0&da>>>0<ca>>>0)){break}c[ha>>2]=ia+ba;da=c[1430]|0;X=(c[1427]|0)+ba|0;$=da;_=da+8|0;if((_&7|0)==0){ja=0}else{ja=-_&7}_=X-ja|0;c[1430]=$+ja;c[1427]=_;c[$+(ja+4)>>2]=_|1;c[$+(X+4)>>2]=40;c[1431]=c[1420];break i}}while(0);if(ca>>>0<(c[1428]|0)>>>0){c[1428]=ca}S=ca+ba|0;X=6144;while(1){ka=X|0;if((c[ka>>2]|0)==(S|0)){U=223;break}$=c[X+8>>2]|0;if(($|0)==0){break}else{X=$}}do{if((U|0)==223){if((c[X+12>>2]&8|0)!=0){break}c[ka>>2]=ca;S=X+4|0;c[S>>2]=(c[S>>2]|0)+ba;S=ca+8|0;if((S&7|0)==0){la=0}else{la=-S&7}S=ca+(ba+8)|0;if((S&7|0)==0){na=0}else{na=-S&7}S=ca+(na+ba)|0;$=S;_=la+o|0;da=ca+_|0;K=da;T=S-(ca+la)-o|0;c[ca+(la+4)>>2]=o|3;j:do{if(($|0)==(c[1430]|0)){Y=(c[1427]|0)+T|0;c[1427]=Y;c[1430]=K;c[ca+(_+4)>>2]=Y|1}else{if(($|0)==(c[1429]|0)){Y=(c[1426]|0)+T|0;c[1426]=Y;c[1429]=K;c[ca+(_+4)>>2]=Y|1;c[ca+(Y+_)>>2]=Y;break}Y=ba+4|0;Z=c[ca+(Y+na)>>2]|0;if((Z&3|0)==1){aa=Z&-8;W=Z>>>3;k:do{if(Z>>>0<256>>>0){V=c[ca+((na|8)+ba)>>2]|0;R=c[ca+(ba+12+na)>>2]|0;J=5736+(W<<1<<2)|0;do{if((V|0)!=(J|0)){if(V>>>0<(c[1428]|0)>>>0){pa();return 0}if((c[V+12>>2]|0)==($|0)){break}pa();return 0}}while(0);if((R|0)==(V|0)){c[1424]=c[1424]&~(1<<W);break}do{if((R|0)==(J|0)){oa=R+8|0}else{if(R>>>0<(c[1428]|0)>>>0){pa();return 0}L=R+8|0;if((c[L>>2]|0)==($|0)){oa=L;break}pa();return 0}}while(0);c[V+12>>2]=R;c[oa>>2]=V}else{J=S;L=c[ca+((na|24)+ba)>>2]|0;O=c[ca+(ba+12+na)>>2]|0;do{if((O|0)==(J|0)){e=na|16;Q=ca+(Y+e)|0;P=c[Q>>2]|0;if((P|0)==0){M=ca+(e+ba)|0;e=c[M>>2]|0;if((e|0)==0){qa=0;break}else{ra=e;sa=M}}else{ra=P;sa=Q}while(1){Q=ra+20|0;P=c[Q>>2]|0;if((P|0)!=0){ra=P;sa=Q;continue}Q=ra+16|0;P=c[Q>>2]|0;if((P|0)==0){break}else{ra=P;sa=Q}}if(sa>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[sa>>2]=0;qa=ra;break}}else{Q=c[ca+((na|8)+ba)>>2]|0;if(Q>>>0<(c[1428]|0)>>>0){pa();return 0}P=Q+12|0;if((c[P>>2]|0)!=(J|0)){pa();return 0}M=O+8|0;if((c[M>>2]|0)==(J|0)){c[P>>2]=O;c[M>>2]=Q;qa=O;break}else{pa();return 0}}}while(0);if((L|0)==0){break}O=ca+(ba+28+na)|0;V=6e3+(c[O>>2]<<2)|0;do{if((J|0)==(c[V>>2]|0)){c[V>>2]=qa;if((qa|0)!=0){break}c[1425]=c[1425]&~(1<<c[O>>2]);break k}else{if(L>>>0<(c[1428]|0)>>>0){pa();return 0}R=L+16|0;if((c[R>>2]|0)==(J|0)){c[R>>2]=qa}else{c[L+20>>2]=qa}if((qa|0)==0){break k}}}while(0);if(qa>>>0<(c[1428]|0)>>>0){pa();return 0}c[qa+24>>2]=L;J=na|16;O=c[ca+(J+ba)>>2]|0;do{if((O|0)!=0){if(O>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[qa+16>>2]=O;c[O+24>>2]=qa;break}}}while(0);O=c[ca+(Y+J)>>2]|0;if((O|0)==0){break}if(O>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[qa+20>>2]=O;c[O+24>>2]=qa;break}}}while(0);ta=ca+((aa|na)+ba)|0;ua=aa+T|0}else{ta=$;ua=T}Y=ta+4|0;c[Y>>2]=c[Y>>2]&-2;c[ca+(_+4)>>2]=ua|1;c[ca+(ua+_)>>2]=ua;Y=ua>>>3;if(ua>>>0<256>>>0){W=Y<<1;Z=5736+(W<<2)|0;O=c[1424]|0;L=1<<Y;do{if((O&L|0)==0){c[1424]=O|L;va=Z;wa=5736+(W+2<<2)|0}else{Y=5736+(W+2<<2)|0;V=c[Y>>2]|0;if(V>>>0>=(c[1428]|0)>>>0){va=V;wa=Y;break}pa();return 0}}while(0);c[wa>>2]=K;c[va+12>>2]=K;c[ca+(_+8)>>2]=va;c[ca+(_+12)>>2]=Z;break}W=da;L=ua>>>8;do{if((L|0)==0){xa=0}else{if(ua>>>0>16777215>>>0){xa=31;break}O=(L+1048320|0)>>>16&8;aa=L<<O;Y=(aa+520192|0)>>>16&4;V=aa<<Y;aa=(V+245760|0)>>>16&2;R=14-(Y|O|aa)+(V<<aa>>>15)|0;xa=ua>>>((R+7|0)>>>0)&1|R<<1}}while(0);L=6e3+(xa<<2)|0;c[ca+(_+28)>>2]=xa;c[ca+(_+20)>>2]=0;c[ca+(_+16)>>2]=0;Z=c[1425]|0;R=1<<xa;if((Z&R|0)==0){c[1425]=Z|R;c[L>>2]=W;c[ca+(_+24)>>2]=L;c[ca+(_+12)>>2]=W;c[ca+(_+8)>>2]=W;break}R=c[L>>2]|0;if((xa|0)==31){ya=0}else{ya=25-(xa>>>1)|0}l:do{if((c[R+4>>2]&-8|0)==(ua|0)){za=R}else{L=R;Z=ua<<ya;while(1){Aa=L+16+(Z>>>31<<2)|0;aa=c[Aa>>2]|0;if((aa|0)==0){break}if((c[aa+4>>2]&-8|0)==(ua|0)){za=aa;break l}else{L=aa;Z=Z<<1}}if(Aa>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[Aa>>2]=W;c[ca+(_+24)>>2]=L;c[ca+(_+12)>>2]=W;c[ca+(_+8)>>2]=W;break j}}}while(0);R=za+8|0;Z=c[R>>2]|0;J=c[1428]|0;if(za>>>0>=J>>>0&Z>>>0>=J>>>0){c[Z+12>>2]=W;c[R>>2]=W;c[ca+(_+8)>>2]=Z;c[ca+(_+12)>>2]=za;c[ca+(_+24)>>2]=0;break}else{pa();return 0}}}while(0);n=ca+(la|8)|0;return n|0}}while(0);X=ea;_=6144;while(1){Ba=c[_>>2]|0;if(Ba>>>0<=X>>>0){Ca=c[_+4>>2]|0;Da=Ba+Ca|0;if(Da>>>0>X>>>0){break}}_=c[_+8>>2]|0}_=Ba+(Ca-39)|0;if((_&7|0)==0){Ea=0}else{Ea=-_&7}_=Ba+(Ca-47+Ea)|0;da=_>>>0<(ea+16|0)>>>0?X:_;_=da+8|0;K=ca+8|0;if((K&7|0)==0){Ia=0}else{Ia=-K&7}K=ba-40-Ia|0;c[1430]=ca+Ia;c[1427]=K;c[ca+(Ia+4)>>2]=K|1;c[ca+(ba-36)>>2]=40;c[1431]=c[1420];c[da+4>>2]=27;c[_>>2]=c[1536];c[_+4>>2]=c[1537];c[_+8>>2]=c[1538];c[_+12>>2]=c[1539];c[1536]=ca;c[1537]=ba;c[1539]=0;c[1538]=_;_=da+28|0;c[_>>2]=7;if((da+32|0)>>>0<Da>>>0){K=_;while(1){_=K+4|0;c[_>>2]=7;if((K+8|0)>>>0<Da>>>0){K=_}else{break}}}if((da|0)==(X|0)){break}K=da-ea|0;_=X+(K+4)|0;c[_>>2]=c[_>>2]&-2;c[ea+4>>2]=K|1;c[X+K>>2]=K;_=K>>>3;if(K>>>0<256>>>0){T=_<<1;$=5736+(T<<2)|0;S=c[1424]|0;j=1<<_;do{if((S&j|0)==0){c[1424]=S|j;Ja=$;Ka=5736+(T+2<<2)|0}else{_=5736+(T+2<<2)|0;Z=c[_>>2]|0;if(Z>>>0>=(c[1428]|0)>>>0){Ja=Z;Ka=_;break}pa();return 0}}while(0);c[Ka>>2]=ea;c[Ja+12>>2]=ea;c[ea+8>>2]=Ja;c[ea+12>>2]=$;break}T=ea;j=K>>>8;do{if((j|0)==0){La=0}else{if(K>>>0>16777215>>>0){La=31;break}S=(j+1048320|0)>>>16&8;X=j<<S;da=(X+520192|0)>>>16&4;_=X<<da;X=(_+245760|0)>>>16&2;Z=14-(da|S|X)+(_<<X>>>15)|0;La=K>>>((Z+7|0)>>>0)&1|Z<<1}}while(0);j=6e3+(La<<2)|0;c[ea+28>>2]=La;c[ea+20>>2]=0;c[ea+16>>2]=0;$=c[1425]|0;Z=1<<La;if(($&Z|0)==0){c[1425]=$|Z;c[j>>2]=T;c[ea+24>>2]=j;c[ea+12>>2]=ea;c[ea+8>>2]=ea;break}Z=c[j>>2]|0;if((La|0)==31){Ma=0}else{Ma=25-(La>>>1)|0}m:do{if((c[Z+4>>2]&-8|0)==(K|0)){Na=Z}else{j=Z;$=K<<Ma;while(1){Oa=j+16+($>>>31<<2)|0;X=c[Oa>>2]|0;if((X|0)==0){break}if((c[X+4>>2]&-8|0)==(K|0)){Na=X;break m}else{j=X;$=$<<1}}if(Oa>>>0<(c[1428]|0)>>>0){pa();return 0}else{c[Oa>>2]=T;c[ea+24>>2]=j;c[ea+12>>2]=ea;c[ea+8>>2]=ea;break i}}}while(0);K=Na+8|0;Z=c[K>>2]|0;$=c[1428]|0;if(Na>>>0>=$>>>0&Z>>>0>=$>>>0){c[Z+12>>2]=T;c[K>>2]=T;c[ea+8>>2]=Z;c[ea+12>>2]=Na;c[ea+24>>2]=0;break}else{pa();return 0}}}while(0);ea=c[1427]|0;if(ea>>>0<=o>>>0){break}Z=ea-o|0;c[1427]=Z;ea=c[1430]|0;K=ea;c[1430]=K+o;c[K+(o+4)>>2]=Z|1;c[ea+4>>2]=o|3;n=ea+8|0;return n|0}}while(0);c[(Ga()|0)>>2]=12;n=0;return n|0}function md(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;if((a|0)==0){return}b=a-8|0;d=b;e=c[1428]|0;if(b>>>0<e>>>0){pa()}f=c[a-4>>2]|0;g=f&3;if((g|0)==1){pa()}h=f&-8;i=a+(h-8)|0;j=i;a:do{if((f&1|0)==0){k=c[b>>2]|0;if((g|0)==0){return}l=-8-k|0;m=a+l|0;n=m;o=k+h|0;if(m>>>0<e>>>0){pa()}if((n|0)==(c[1429]|0)){p=a+(h-4)|0;if((c[p>>2]&3|0)!=3){q=n;r=o;break}c[1426]=o;c[p>>2]=c[p>>2]&-2;c[a+(l+4)>>2]=o|1;c[i>>2]=o;return}p=k>>>3;if(k>>>0<256>>>0){k=c[a+(l+8)>>2]|0;s=c[a+(l+12)>>2]|0;t=5736+(p<<1<<2)|0;do{if((k|0)!=(t|0)){if(k>>>0<e>>>0){pa()}if((c[k+12>>2]|0)==(n|0)){break}pa()}}while(0);if((s|0)==(k|0)){c[1424]=c[1424]&~(1<<p);q=n;r=o;break}do{if((s|0)==(t|0)){u=s+8|0}else{if(s>>>0<e>>>0){pa()}v=s+8|0;if((c[v>>2]|0)==(n|0)){u=v;break}pa()}}while(0);c[k+12>>2]=s;c[u>>2]=k;q=n;r=o;break}t=m;p=c[a+(l+24)>>2]|0;v=c[a+(l+12)>>2]|0;do{if((v|0)==(t|0)){w=a+(l+20)|0;x=c[w>>2]|0;if((x|0)==0){y=a+(l+16)|0;z=c[y>>2]|0;if((z|0)==0){A=0;break}else{B=z;C=y}}else{B=x;C=w}while(1){w=B+20|0;x=c[w>>2]|0;if((x|0)!=0){B=x;C=w;continue}w=B+16|0;x=c[w>>2]|0;if((x|0)==0){break}else{B=x;C=w}}if(C>>>0<e>>>0){pa()}else{c[C>>2]=0;A=B;break}}else{w=c[a+(l+8)>>2]|0;if(w>>>0<e>>>0){pa()}x=w+12|0;if((c[x>>2]|0)!=(t|0)){pa()}y=v+8|0;if((c[y>>2]|0)==(t|0)){c[x>>2]=v;c[y>>2]=w;A=v;break}else{pa()}}}while(0);if((p|0)==0){q=n;r=o;break}v=a+(l+28)|0;m=6e3+(c[v>>2]<<2)|0;do{if((t|0)==(c[m>>2]|0)){c[m>>2]=A;if((A|0)!=0){break}c[1425]=c[1425]&~(1<<c[v>>2]);q=n;r=o;break a}else{if(p>>>0<(c[1428]|0)>>>0){pa()}k=p+16|0;if((c[k>>2]|0)==(t|0)){c[k>>2]=A}else{c[p+20>>2]=A}if((A|0)==0){q=n;r=o;break a}}}while(0);if(A>>>0<(c[1428]|0)>>>0){pa()}c[A+24>>2]=p;t=c[a+(l+16)>>2]|0;do{if((t|0)!=0){if(t>>>0<(c[1428]|0)>>>0){pa()}else{c[A+16>>2]=t;c[t+24>>2]=A;break}}}while(0);t=c[a+(l+20)>>2]|0;if((t|0)==0){q=n;r=o;break}if(t>>>0<(c[1428]|0)>>>0){pa()}else{c[A+20>>2]=t;c[t+24>>2]=A;q=n;r=o;break}}else{q=d;r=h}}while(0);d=q;if(d>>>0>=i>>>0){pa()}A=a+(h-4)|0;e=c[A>>2]|0;if((e&1|0)==0){pa()}do{if((e&2|0)==0){if((j|0)==(c[1430]|0)){B=(c[1427]|0)+r|0;c[1427]=B;c[1430]=q;c[q+4>>2]=B|1;if((q|0)!=(c[1429]|0)){return}c[1429]=0;c[1426]=0;return}if((j|0)==(c[1429]|0)){B=(c[1426]|0)+r|0;c[1426]=B;c[1429]=q;c[q+4>>2]=B|1;c[d+B>>2]=B;return}B=(e&-8)+r|0;C=e>>>3;b:do{if(e>>>0<256>>>0){u=c[a+h>>2]|0;g=c[a+(h|4)>>2]|0;b=5736+(C<<1<<2)|0;do{if((u|0)!=(b|0)){if(u>>>0<(c[1428]|0)>>>0){pa()}if((c[u+12>>2]|0)==(j|0)){break}pa()}}while(0);if((g|0)==(u|0)){c[1424]=c[1424]&~(1<<C);break}do{if((g|0)==(b|0)){D=g+8|0}else{if(g>>>0<(c[1428]|0)>>>0){pa()}f=g+8|0;if((c[f>>2]|0)==(j|0)){D=f;break}pa()}}while(0);c[u+12>>2]=g;c[D>>2]=u}else{b=i;f=c[a+(h+16)>>2]|0;t=c[a+(h|4)>>2]|0;do{if((t|0)==(b|0)){p=a+(h+12)|0;v=c[p>>2]|0;if((v|0)==0){m=a+(h+8)|0;k=c[m>>2]|0;if((k|0)==0){E=0;break}else{F=k;G=m}}else{F=v;G=p}while(1){p=F+20|0;v=c[p>>2]|0;if((v|0)!=0){F=v;G=p;continue}p=F+16|0;v=c[p>>2]|0;if((v|0)==0){break}else{F=v;G=p}}if(G>>>0<(c[1428]|0)>>>0){pa()}else{c[G>>2]=0;E=F;break}}else{p=c[a+h>>2]|0;if(p>>>0<(c[1428]|0)>>>0){pa()}v=p+12|0;if((c[v>>2]|0)!=(b|0)){pa()}m=t+8|0;if((c[m>>2]|0)==(b|0)){c[v>>2]=t;c[m>>2]=p;E=t;break}else{pa()}}}while(0);if((f|0)==0){break}t=a+(h+20)|0;u=6e3+(c[t>>2]<<2)|0;do{if((b|0)==(c[u>>2]|0)){c[u>>2]=E;if((E|0)!=0){break}c[1425]=c[1425]&~(1<<c[t>>2]);break b}else{if(f>>>0<(c[1428]|0)>>>0){pa()}g=f+16|0;if((c[g>>2]|0)==(b|0)){c[g>>2]=E}else{c[f+20>>2]=E}if((E|0)==0){break b}}}while(0);if(E>>>0<(c[1428]|0)>>>0){pa()}c[E+24>>2]=f;b=c[a+(h+8)>>2]|0;do{if((b|0)!=0){if(b>>>0<(c[1428]|0)>>>0){pa()}else{c[E+16>>2]=b;c[b+24>>2]=E;break}}}while(0);b=c[a+(h+12)>>2]|0;if((b|0)==0){break}if(b>>>0<(c[1428]|0)>>>0){pa()}else{c[E+20>>2]=b;c[b+24>>2]=E;break}}}while(0);c[q+4>>2]=B|1;c[d+B>>2]=B;if((q|0)!=(c[1429]|0)){H=B;break}c[1426]=B;return}else{c[A>>2]=e&-2;c[q+4>>2]=r|1;c[d+r>>2]=r;H=r}}while(0);r=H>>>3;if(H>>>0<256>>>0){d=r<<1;e=5736+(d<<2)|0;A=c[1424]|0;E=1<<r;do{if((A&E|0)==0){c[1424]=A|E;I=e;J=5736+(d+2<<2)|0}else{r=5736+(d+2<<2)|0;h=c[r>>2]|0;if(h>>>0>=(c[1428]|0)>>>0){I=h;J=r;break}pa()}}while(0);c[J>>2]=q;c[I+12>>2]=q;c[q+8>>2]=I;c[q+12>>2]=e;return}e=q;I=H>>>8;do{if((I|0)==0){K=0}else{if(H>>>0>16777215>>>0){K=31;break}J=(I+1048320|0)>>>16&8;d=I<<J;E=(d+520192|0)>>>16&4;A=d<<E;d=(A+245760|0)>>>16&2;r=14-(E|J|d)+(A<<d>>>15)|0;K=H>>>((r+7|0)>>>0)&1|r<<1}}while(0);I=6e3+(K<<2)|0;c[q+28>>2]=K;c[q+20>>2]=0;c[q+16>>2]=0;r=c[1425]|0;d=1<<K;c:do{if((r&d|0)==0){c[1425]=r|d;c[I>>2]=e;c[q+24>>2]=I;c[q+12>>2]=q;c[q+8>>2]=q}else{A=c[I>>2]|0;if((K|0)==31){L=0}else{L=25-(K>>>1)|0}d:do{if((c[A+4>>2]&-8|0)==(H|0)){M=A}else{J=A;E=H<<L;while(1){N=J+16+(E>>>31<<2)|0;h=c[N>>2]|0;if((h|0)==0){break}if((c[h+4>>2]&-8|0)==(H|0)){M=h;break d}else{J=h;E=E<<1}}if(N>>>0<(c[1428]|0)>>>0){pa()}else{c[N>>2]=e;c[q+24>>2]=J;c[q+12>>2]=q;c[q+8>>2]=q;break c}}}while(0);A=M+8|0;B=c[A>>2]|0;E=c[1428]|0;if(M>>>0>=E>>>0&B>>>0>=E>>>0){c[B+12>>2]=e;c[A>>2]=e;c[q+8>>2]=B;c[q+12>>2]=M;c[q+24>>2]=0;break}else{pa()}}}while(0);q=(c[1432]|0)-1|0;c[1432]=q;if((q|0)==0){O=6152}else{return}while(1){q=c[O>>2]|0;if((q|0)==0){break}else{O=q+8|0}}c[1432]=-1;return}function nd(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((a|0)==0){d=0}else{e=aa(b,a)|0;if((b|a)>>>0<=65535>>>0){d=e;break}d=((e>>>0)/(a>>>0)|0|0)==(b|0)?e:-1}}while(0);b=ld(d)|0;if((b|0)==0){return b|0}if((c[b-4>>2]&3|0)==0){return b|0}od(b|0,0,d|0)|0;return b|0}function od(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=b+e|0;if((e|0)>=20){d=d&255;g=b&3;h=d|d<<8|d<<16|d<<24;i=f&~3;if(g){g=b+4-g|0;while((b|0)<(g|0)){a[b]=d;b=b+1|0}}while((b|0)<(i|0)){c[b>>2]=h;b=b+4|0}}while((b|0)<(f|0)){a[b]=d;b=b+1|0}return b-e|0}function pd(b,d,e){b=b|0;d=d|0;e=e|0;var f=0;f=b|0;if((b&3)==(d&3)){while(b&3){if((e|0)==0)return f|0;a[b]=a[d]|0;b=b+1|0;d=d+1|0;e=e-1|0}while((e|0)>=4){c[b>>2]=c[d>>2];b=b+4|0;d=d+4|0;e=e-4|0}}while((e|0)>0){a[b]=a[d]|0;b=b+1|0;d=d+1|0;e=e-1|0}return f|0}function qd(b){b=b|0;var c=0;c=b;while(a[c]|0){c=c+1|0}return c-b|0}function rd(b,c){b=b|0;c=c|0;var d=0;do{a[b+d|0]=a[c+d|0];d=d+1|0}while(a[c+(d-1)|0]|0);return b|0}function sd(b,c){b=b|0;c=c|0;var d=0,e=0;d=b+(qd(b)|0)|0;do{a[d+e|0]=a[c+e|0];e=e+1|0}while(a[c+(e-1)|0]|0);return b|0}function td(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=a+c>>>0;return(E=b+d+(e>>>0<a>>>0|0)>>>0,e|0)|0}function ud(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=b-d>>>0;e=b-d-(c>>>0>a>>>0|0)>>>0;return(E=e,a-c>>>0|0)|0}function vd(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){E=b<<c|(a&(1<<c)-1<<32-c)>>>32-c;return a<<c}E=a<<c-32;return 0}function wd(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){E=b>>>c;return a>>>c|(b&(1<<c)-1)<<32-c}E=0;return b>>>c-32|0}function xd(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){E=b>>c;return a>>>c|(b&(1<<c)-1)<<32-c}E=(b|0)<0?-1:0;return b>>c-32|0}function yd(b){b=b|0;var c=0;c=a[n+(b>>>24)|0]|0;if((c|0)<8)return c|0;c=a[n+(b>>16&255)|0]|0;if((c|0)<8)return c+8|0;c=a[n+(b>>8&255)|0]|0;if((c|0)<8)return c+16|0;return(a[n+(b&255)|0]|0)+24|0}function zd(b){b=b|0;var c=0;c=a[m+(b&255)|0]|0;if((c|0)<8)return c|0;c=a[m+(b>>8&255)|0]|0;if((c|0)<8)return c+8|0;c=a[m+(b>>16&255)|0]|0;if((c|0)<8)return c+16|0;return(a[m+(b>>>24)|0]|0)+24|0}function Ad(a,b){a=a|0;b=b|0;var c=0,d=0,e=0,f=0;c=a&65535;d=b&65535;e=aa(d,c)|0;f=a>>>16;a=(e>>>16)+(aa(d,f)|0)|0;d=b>>>16;b=aa(d,c)|0;return(E=(a>>>16)+(aa(d,f)|0)+(((a&65535)+b|0)>>>16)|0,a+b<<16|e&65535|0)|0}function Bd(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=b>>31|((b|0)<0?-1:0)<<1;f=((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1;g=d>>31|((d|0)<0?-1:0)<<1;h=((d|0)<0?-1:0)>>31|((d|0)<0?-1:0)<<1;i=ud(e^a,f^b,e,f)|0;b=E;a=g^e;e=h^f;f=ud((Gd(i,b,ud(g^c,h^d,g,h)|0,E,0)|0)^a,E^e,a,e)|0;return(E=E,f)|0}function Cd(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0;f=i;i=i+8|0;g=f|0;h=b>>31|((b|0)<0?-1:0)<<1;j=((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1;k=e>>31|((e|0)<0?-1:0)<<1;l=((e|0)<0?-1:0)>>31|((e|0)<0?-1:0)<<1;m=ud(h^a,j^b,h,j)|0;b=E;Gd(m,b,ud(k^d,l^e,k,l)|0,E,g)|0;l=ud(c[g>>2]^h,c[g+4>>2]^j,h,j)|0;j=E;i=f;return(E=j,l)|0}function Dd(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;e=a;a=c;c=Ad(e,a)|0;f=E;return(E=(aa(b,a)|0)+(aa(d,e)|0)+f|f&0,c|0|0)|0}function Ed(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=Gd(a,b,c,d,0)|0;return(E=E,e)|0}function Fd(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;f=i;i=i+8|0;g=f|0;Gd(a,b,d,e,g)|0;i=f;return(E=c[g+4>>2]|0,c[g>>2]|0)|0}function Gd(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;g=a;h=b;i=h;j=d;k=e;l=k;if((i|0)==0){m=(f|0)!=0;if((l|0)==0){if(m){c[f>>2]=(g>>>0)%(j>>>0);c[f+4>>2]=0}n=0;o=(g>>>0)/(j>>>0)>>>0;return(E=n,o)|0}else{if(!m){n=0;o=0;return(E=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=b&0;n=0;o=0;return(E=n,o)|0}}m=(l|0)==0;do{if((j|0)==0){if(m){if((f|0)!=0){c[f>>2]=(i>>>0)%(j>>>0);c[f+4>>2]=0}n=0;o=(i>>>0)/(j>>>0)>>>0;return(E=n,o)|0}if((g|0)==0){if((f|0)!=0){c[f>>2]=0;c[f+4>>2]=(i>>>0)%(l>>>0)}n=0;o=(i>>>0)/(l>>>0)>>>0;return(E=n,o)|0}p=l-1|0;if((p&l|0)==0){if((f|0)!=0){c[f>>2]=a|0;c[f+4>>2]=p&i|b&0}n=0;o=i>>>((zd(l|0)|0)>>>0);return(E=n,o)|0}p=(yd(l|0)|0)-(yd(i|0)|0)|0;if(p>>>0<=30){q=p+1|0;r=31-p|0;s=q;t=i<<r|g>>>(q>>>0);u=i>>>(q>>>0);v=0;w=g<<r;break}if((f|0)==0){n=0;o=0;return(E=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=h|b&0;n=0;o=0;return(E=n,o)|0}else{if(!m){r=(yd(l|0)|0)-(yd(i|0)|0)|0;if(r>>>0<=31){q=r+1|0;p=31-r|0;x=r-31>>31;s=q;t=g>>>(q>>>0)&x|i<<p;u=i>>>(q>>>0)&x;v=0;w=g<<p;break}if((f|0)==0){n=0;o=0;return(E=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=h|b&0;n=0;o=0;return(E=n,o)|0}p=j-1|0;if((p&j|0)!=0){x=(yd(j|0)|0)+33-(yd(i|0)|0)|0;q=64-x|0;r=32-x|0;y=r>>31;z=x-32|0;A=z>>31;s=x;t=r-1>>31&i>>>(z>>>0)|(i<<r|g>>>(x>>>0))&A;u=A&i>>>(x>>>0);v=g<<q&y;w=(i<<q|g>>>(z>>>0))&y|g<<r&x-33>>31;break}if((f|0)!=0){c[f>>2]=p&g;c[f+4>>2]=0}if((j|0)==1){n=h|b&0;o=a|0|0;return(E=n,o)|0}else{p=zd(j|0)|0;n=i>>>(p>>>0)|0;o=i<<32-p|g>>>(p>>>0)|0;return(E=n,o)|0}}}while(0);if((s|0)==0){B=w;C=v;D=u;F=t;G=0;H=0}else{g=d|0|0;d=k|e&0;e=td(g,d,-1,-1)|0;k=E;i=w;w=v;v=u;u=t;t=s;s=0;while(1){I=w>>>31|i<<1;J=s|w<<1;j=u<<1|i>>>31|0;a=u>>>31|v<<1|0;ud(e,k,j,a)|0;b=E;h=b>>31|((b|0)<0?-1:0)<<1;K=h&1;L=ud(j,a,h&g,(((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1)&d)|0;M=E;b=t-1|0;if((b|0)==0){break}else{i=I;w=J;v=M;u=L;t=b;s=K}}B=I;C=J;D=M;F=L;G=0;H=K}K=C;C=0;if((f|0)!=0){c[f>>2]=F;c[f+4>>2]=D}n=(K|0)>>>31|(B|C)<<1|(C<<1|K>>>31)&0|G;o=(K<<1|0>>>31)&-2|H;return(E=n,o)|0}function Hd(a){a=a|0;return Ka[a&1]()|0}function Id(a,b){a=a|0;b=b|0;La[a&1](b|0)}function Jd(a,b){a=a|0;b=b|0;return Ma[a&1](b|0)|0}function Kd(a){a=a|0;Na[a&1]()}function Ld(a,b,c){a=a|0;b=b|0;c=c|0;return Oa[a&1](b|0,c|0)|0}function Md(){ba(0);return 0}function Nd(a){a=a|0;ba(1)}function Od(a){a=a|0;ba(2);return 0}function Pd(){ba(3)}function Qd(a,b){a=a|0;b=b|0;ba(4);return 0}




// EMSCRIPTEN_END_FUNCS
var Ka=[Md,Md];var La=[Nd,Nd];var Ma=[Od,Od];var Na=[Pd,Pd];var Oa=[Qd,Qd];return{_strlen:qd,_strcat:sd,_free:md,_select_priv_key:gd,_memset:od,_malloc:ld,_memcpy:pd,_get_version:ed,_make_licence:fd,_strcpy:rd,_calloc:nd,runPostSets:db,stackAlloc:Pa,stackSave:Qa,stackRestore:Ra,setThrew:Sa,setTempRet0:Va,setTempRet1:Wa,setTempRet2:Xa,setTempRet3:Ya,setTempRet4:Za,setTempRet5:_a,setTempRet6:$a,setTempRet7:ab,setTempRet8:bb,setTempRet9:cb,dynCall_i:Hd,dynCall_vi:Id,dynCall_ii:Jd,dynCall_v:Kd,dynCall_iii:Ld}})


// EMSCRIPTEN_END_ASM
({ "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array }, { "abort": abort, "assert": assert, "asmPrintInt": asmPrintInt, "asmPrintFloat": asmPrintFloat, "min": Math_min, "invoke_i": invoke_i, "invoke_vi": invoke_vi, "invoke_ii": invoke_ii, "invoke_v": invoke_v, "invoke_iii": invoke_iii, "_strncmp": _strncmp, "_sysconf": _sysconf, "_fgetc": _fgetc, "_fread": _fread, "_abort": _abort, "_toupper": _toupper, "_pread": _pread, "_strtoll": _strtoll, "_fflush": _fflush, "_fputc": _fputc, "___setErrNo": ___setErrNo, "_send": _send, "_write": _write, "_exit": _exit, "_isspace": _isspace, "_read": _read, "_llvm_uadd_with_overflow_i32": _llvm_uadd_with_overflow_i32, "_recv": _recv, "__parseInt64": __parseInt64, "_pwrite": _pwrite, "_sbrk": _sbrk, "___errno_location": ___errno_location, "_time": _time, "__exit": __exit, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity, "_stdout": _stdout }, buffer);
var _strlen = Module["_strlen"] = asm["_strlen"];
var _strcat = Module["_strcat"] = asm["_strcat"];
var _free = Module["_free"] = asm["_free"];
var _select_priv_key = Module["_select_priv_key"] = asm["_select_priv_key"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _get_version = Module["_get_version"] = asm["_get_version"];
var _make_licence = Module["_make_licence"] = asm["_make_licence"];
var _strcpy = Module["_strcpy"] = asm["_strcpy"];
var _calloc = Module["_calloc"] = asm["_calloc"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];

Runtime.stackAlloc = function(size) { return asm['stackAlloc'](size) };
Runtime.stackSave = function() { return asm['stackSave']() };
Runtime.stackRestore = function(top) { asm['stackRestore'](top) };

// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======


/*
  Copyright (C) 2013

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  https://github.com/johnmccutchan/ecmascript_simd/blob/master/src/ecmascript_simd.js
*/

"use strict";

/**
  * Construct a new instance of float32x4 number.
  * @param {double} value used for x lane.
  * @param {double} value used for y lane.
  * @param {double} value used for z lane.
  * @param {double} value used for w lane.
  * @constructor
  */
function float32x4(x, y, z, w) {
  if (!(this instanceof float32x4)) {
    return new float32x4(x, y, z, w);
  }
  this.storage_ = new Float32Array(4);
  this.storage_[0] = x;
  this.storage_[1] = y;
  this.storage_[2] = z;
  this.storage_[3] = w;
}

/**
  * Construct a new instance of float32x4 number with 0.0 in all lanes.
  * @constructor
  */
float32x4.zero = function() {
  return float32x4(0.0, 0.0, 0.0, 0.0);
}

/**
  * Construct a new instance of float32x4 number with the same value
  * in all lanes.
  * @param {double} value used for all lanes.
  * @constructor
  */
float32x4.splat = function(s) {
  return float32x4(s, s, s, s);
}

Object.defineProperty(float32x4.prototype, 'x', {
  get: function() { return this.storage_[0]; }
});

Object.defineProperty(float32x4.prototype, 'y', {
  get: function() { return this.storage_[1]; }
});

Object.defineProperty(float32x4.prototype, 'z', {
  get: function() { return this.storage_[2]; }
});

Object.defineProperty(float32x4.prototype, 'w',
  { get: function() { return this.storage_[3]; }
});

/**
  * Extract the sign bit from each lane return them in the first 4 bits.
  */
Object.defineProperty(float32x4.prototype, 'signMask', {
  get: function() {
    var mx = this.x < 0.0 ? 1 : 0;
    var my = this.y < 0.0 ? 1 : 0;
    var mz = this.z < 0.0 ? 1 : 0;
    var mw = this.w < 0.0 ? 1 : 0;
    return mx | my << 1 | mz << 2 | mw << 3;
  }
});

/**
  * Construct a new instance of int32x4 number.
  * @param {integer} 32-bit unsigned value used for x lane.
  * @param {integer} 32-bit unsigned value used for y lane.
  * @param {integer} 32-bit unsigned value used for z lane.
  * @param {integer} 32-bit unsigned value used for w lane.
  * @constructor
  */
function int32x4(x, y, z, w) {
  if (!(this instanceof int32x4)) {
    return new int32x4(x, y, z, w);
  }
  this.storage_ = new Int32Array(4);
  this.storage_[0] = x;
  this.storage_[1] = y;
  this.storage_[2] = z;
  this.storage_[3] = w;
}

/**
  * Construct a new instance of int32x4 number with 0xFFFFFFFF or 0x0 in each
  * lane, depending on the truth value in x, y, z, and w.
  * @param {boolean} flag used for x lane.
  * @param {boolean} flag used for y lane.
  * @param {boolean} flag used for z lane.
  * @param {boolean} flag used for w lane.
  * @constructor
  */
int32x4.bool = function(x, y, z, w) {
  return int32x4(x ? -1 : 0x0,
                  y ? -1 : 0x0,
                  z ? -1 : 0x0,
                  w ? -1 : 0x0);
}

/**
  * Construct a new instance of int32x4 number with the same value
  * in all lanes.
  * @param {integer} value used for all lanes.
  * @constructor
  */
int32x4.splat = function(s) {
  return int32x4(s, s, s, s);
}

Object.defineProperty(int32x4.prototype, 'x', {
  get: function() { return this.storage_[0]; }
});

Object.defineProperty(int32x4.prototype, 'y', {
  get: function() { return this.storage_[1]; }
});

Object.defineProperty(int32x4.prototype, 'z', {
  get: function() { return this.storage_[2]; }
});

Object.defineProperty(int32x4.prototype, 'w',
  { get: function() { return this.storage_[3]; }
});

Object.defineProperty(int32x4.prototype, 'flagX', {
  get: function() { return this.storage_[0] != 0x0; }
});

Object.defineProperty(int32x4.prototype, 'flagY', {
  get: function() { return this.storage_[1] != 0x0; }
});

Object.defineProperty(int32x4.prototype, 'flagZ', {
  get: function() { return this.storage_[2] != 0x0; }
});

Object.defineProperty(int32x4.prototype, 'flagW',
  { get: function() { return this.storage_[3] != 0x0; }
});

/**
  * Extract the sign bit from each lane return them in the first 4 bits.
  */
Object.defineProperty(int32x4.prototype, 'signMask', {
  get: function() {
    var mx = (this.storage_[0] & 0x80000000) >>> 31;
    var my = (this.storage_[1] & 0x80000000) >>> 31;
    var mz = (this.storage_[2] & 0x80000000) >>> 31;
    var mw = (this.storage_[3] & 0x80000000) >>> 31;
    return mx | my << 1 | mz << 2 | mw << 3;
  }
});

function isNumber(o) {
    return typeof o == "number" || (typeof o == "object" && o.constructor === Number);
}

function isTypedArray(o) {
  return (o instanceof Int8Array) ||
         (o instanceof Uint8Array) ||
         (o instanceof Uint8ClampedArray) ||
         (o instanceof Int16Array) ||
         (o instanceof Uint16Array) ||
         (o instanceof Int32Array) ||
         (o instanceof Uint32Array) ||
         (o instanceof Float32Array) ||
         (o instanceof Float64Array) ||
         (o instanceof Float32x4Array);
}

function isArrayBuffer(o) {
  return (o instanceof ArrayBuffer);
}

function Float32x4Array(a, b, c) {
  if (isNumber(a)) {
    this.storage_ = new Float32Array(a*4);
    this.length_ = a;
    this.byteOffset_ = 0;
    return;
  } else if (isTypedArray(a)) {
    if (!(a instanceof Float32x4Array)) {
      throw "Copying typed array of non-Float32x4Array is unimplemented.";
    }
    this.storage_ = new Float32Array(a.length * 4);
    this.length_ = a.length;
    this.byteOffset_ = 0;
    // Copy floats.
    for (var i = 0; i < a.length*4; i++) {
      this.storage_[i] = a.storage_[i];
    }
  } else if (isArrayBuffer(a)) {
    if ((b != undefined) && (b % Float32x4Array.BYTES_PER_ELEMENT) != 0) {
      throw "byteOffset must be a multiple of 16.";
    }
    if (c != undefined) {
      c *= 4;
      this.storage_ = new Float32Array(a, b, c);
    }
    else {
      // Note: new Float32Array(a, b) is NOT equivalent to new Float32Array(a, b, undefined)
      this.storage_ = new Float32Array(a, b);
    }
    this.length_ = this.storage_.length / 4;
    this.byteOffset_ = b != undefined ? b : 0;
  } else {
    throw "Unknown type of first argument.";
  }
}

Object.defineProperty(Float32x4Array.prototype, 'length',
  { get: function() { return this.length_; }
});

Object.defineProperty(Float32x4Array.prototype, 'byteLength',
  { get: function() { return this.length_ * Float32x4Array.BYTES_PER_ELEMENT; }
});

Object.defineProperty(Float32x4Array, 'BYTES_PER_ELEMENT',
  { get: function() { return 16; }
});

Object.defineProperty(Float32x4Array.prototype, 'BYTES_PER_ELEMENT',
  { get: function() { return 16; }
});

Object.defineProperty(Float32x4Array.prototype, 'byteOffset',
  { get: function() { return this.byteOffset_; }
});

Object.defineProperty(Float32x4Array.prototype, 'buffer',
  { get: function() { return this.storage_.buffer; }
});

Float32x4Array.prototype.getAt = function(i) {
  if (i < 0) {
    throw "Index must be >= 0.";
  }
  if (i >= this.length) {
    throw "Index out of bounds.";
  }
  var x = this.storage_[i*4+0];
  var y = this.storage_[i*4+1];
  var z = this.storage_[i*4+2];
  var w = this.storage_[i*4+3];
  return float32x4(x, y, z, w);
}

Float32x4Array.prototype.setAt = function(i, v) {
  if (i < 0) {
    throw "Index must be >= 0.";
  }
  if (i >= this.length) {
    throw "Index out of bounds.";
  }
  if (!(v instanceof float32x4)) {
    throw "Value is not a float32x4.";
  }
  this.storage_[i*4+0] = v.x;
  this.storage_[i*4+1] = v.y;
  this.storage_[i*4+2] = v.z;
  this.storage_[i*4+3] = v.w;
}


function Int32x4Array(a, b, c) {

  function isNumber(o) {
      return typeof o == "number" || (typeof o == "object" && o.constructor === Number);
  }

  function isTypedArray(o) {
    return (o instanceof Int8Array) ||
           (o instanceof Uint8Array) ||
           (o instanceof Uint8ClampedArray) ||
           (o instanceof Int16Array) ||
           (o instanceof Uint16Array) ||
           (o instanceof Int32Array) ||
           (o instanceof Uint32Array) ||
           (o instanceof Float32Array) ||
           (o instanceof Float64Array) ||
           (o instanceof Int32x4Array) ||
           (o instanceof Float32x4Array);
  }

  function isArrayBuffer(o) {
    return (o instanceof ArrayBuffer);
  }

  if (isNumber(a)) {
    this.storage_ = new Int32Array(a*4);
    this.length_ = a;
    this.byteOffset_ = 0;
    return;
  } else if (isTypedArray(a)) {
    if (!(a instanceof Int32x4Array)) {
      throw "Copying typed array of non-Int32x4Array is unimplemented.";
    }
    this.storage_ = new Int32Array(a.length * 4);
    this.length_ = a.length;
    this.byteOffset_ = 0;
    // Copy floats.
    for (var i = 0; i < a.length*4; i++) {
      this.storage_[i] = a.storage_[i];
    }
  } else if (isArrayBuffer(a)) {
    if ((b != undefined) && (b % Int32x4Array.BYTES_PER_ELEMENT) != 0) {
      throw "byteOffset must be a multiple of 16.";
    }
    if (c != undefined) {
      c *= 4;
      this.storage_ = new Int32Array(a, b, c);
    }
    else {
      // Note: new Int32Array(a, b) is NOT equivalent to new Float32Array(a, b, undefined)
      this.storage_ = new Int32Array(a, b);
    }
    this.length_ = this.storage_.length / 4;
    this.byteOffset_ = b != undefined ? b : 0;
  } else {
    throw "Unknown type of first argument.";
  }
}

Object.defineProperty(Int32x4Array.prototype, 'length',
  { get: function() { return this.length_; }
});

Object.defineProperty(Int32x4Array.prototype, 'byteLength',
  { get: function() { return this.length_ * Int32x4Array.BYTES_PER_ELEMENT; }
});

Object.defineProperty(Int32x4Array, 'BYTES_PER_ELEMENT',
  { get: function() { return 16; }
});

Object.defineProperty(Int32x4Array.prototype, 'BYTES_PER_ELEMENT',
  { get: function() { return 16; }
});

Object.defineProperty(Int32x4Array.prototype, 'byteOffset',
  { get: function() { return this.byteOffset_; }
});

Object.defineProperty(Int32x4Array.prototype, 'buffer',
  { get: function() { return this.storage_.buffer; }
});

Int32x4Array.prototype.getAt = function(i) {
  if (i < 0) {
    throw "Index must be >= 0.";
  }
  if (i >= this.length) {
    throw "Index out of bounds.";
  }
  var x = this.storage_[i*4+0];
  var y = this.storage_[i*4+1];
  var z = this.storage_[i*4+2];
  var w = this.storage_[i*4+3];
  return float32x4(x, y, z, w);
}

Int32x4Array.prototype.setAt = function(i, v) {
  if (i < 0) {
    throw "Index must be >= 0.";
  }
  if (i >= this.length) {
    throw "Index out of bounds.";
  }
  if (!(v instanceof int32x4)) {
    throw "Value is not a int32x4.";
  }
  this.storage_[i*4+0] = v.x;
  this.storage_[i*4+1] = v.y;
  this.storage_[i*4+2] = v.z;
  this.storage_[i*4+3] = v.w;
}

var SIMD = (function () {
  return {
    float32x4: {
        /**
        * @return {float32x4} New instance of float32x4 with absolute values of
        * t.
        */
      abs: function(t) {
        return new float32x4(Math.abs(t.x), Math.abs(t.y), Math.abs(t.z),
                             Math.abs(t.w));
      },
      /**
        * @return {float32x4} New instance of float32x4 with negated values of
        * t.
        */
      neg: function(t) {
        return new float32x4(-t.x, -t.y, -t.z, -t.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with a + b.
        */
      add: function(a, b) {
        return new float32x4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with a - b.
        */
      sub: function(a, b) {
        return new float32x4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with a * b.
        */
      mul: function(a, b) {
        return new float32x4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with a / b.
        */
      div: function(a, b) {
        return new float32x4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with t's values clamped
        * between lowerLimit and upperLimit.
        */
      clamp: function(t, lowerLimit, upperLimit) {
        var cx = t.x < lowerLimit.x ? lowerLimit.x : t.x;
        var cy = t.y < lowerLimit.y ? lowerLimit.y : t.y;
        var cz = t.z < lowerLimit.z ? lowerLimit.z : t.z;
        var cw = t.w < lowerLimit.w ? lowerLimit.w : t.w;
        cx = cx > upperLimit.x ? upperLimit.x : cx;
        cy = cy > upperLimit.y ? upperLimit.y : cy;
        cz = cz > upperLimit.z ? upperLimit.z : cz;
        cw = cw > upperLimit.w ? upperLimit.w : cw;
        return new float32x4(cx, cy, cz, cw);
      },
      /**
        * @return {float32x4} New instance of float32x4 with the minimum value of
        * t and other.
        */
      min: function(t, other) {
        var cx = t.x > other.x ? other.x : t.x;
        var cy = t.y > other.y ? other.y : t.y;
        var cz = t.z > other.z ? other.z : t.z;
        var cw = t.w > other.w ? other.w : t.w;
        return new float32x4(cx, cy, cz, cw);
      },
      /**
        * @return {float32x4} New instance of float32x4 with the maximum value of
        * t and other.
        */
      max: function(t, other) {
        var cx = t.x < other.x ? other.x : t.x;
        var cy = t.y < other.y ? other.y : t.y;
        var cz = t.z < other.z ? other.z : t.z;
        var cw = t.w < other.w ? other.w : t.w;
        return new float32x4(cx, cy, cz, cw);
      },
      /**
        * @return {float32x4} New instance of float32x4 with reciprocal value of
        * t.
        */
      reciprocal: function(t) {
        return new float32x4(1.0 / t.x, 1.0 / t.y, 1.0 / t.z, 1.0 / t.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with square root of the
        * reciprocal value of t.
        */
      reciprocalSqrt: function(t) {
        return new float32x4(Math.sqrt(1.0 / t.x), Math.sqrt(1.0 / t.y),
                             Math.sqrt(1.0 / t.z), Math.sqrt(1.0 / t.w));
      },
      /**
        * @return {float32x4} New instance of float32x4 with values of t
        * scaled by s.
        */
      scale: function(t, s) {
        return new float32x4(s * t.x, s * t.y, s * t.z, s * t.w);
      },
      /**
        * @return {float32x4} New instance of float32x4 with square root of
        * values of t.
        */
      sqrt: function(t) {
        return new float32x4(Math.sqrt(t.x), Math.sqrt(t.y),
                             Math.sqrt(t.z), Math.sqrt(t.w));
      },
      /**
        * @param {float32x4} t An instance of float32x4 to be shuffled.
        * @param {integer} mask One of the 256 shuffle masks, for example, SIMD.XXXX.
        * @return {float32x4} New instance of float32x4 with lanes shuffled.
        */
      shuffle: function(t, mask) {
        var _x = (mask) & 0x3;
        var _y = (mask >> 2) & 0x3;
        var _z = (mask >> 4) & 0x3;
        var _w = (mask >> 6) & 0x3;
        return new float32x4(t.storage_[_x], t.storage_[_y], t.storage_[_z],
                             t.storage_[_w]);
      },
      /**
        * @param {float32x4} t1 An instance of float32x4 to be shuffled. XY lanes in result
        * @param {float32x4} t2 An instance of float32x4 to be shuffled. ZW lanes in result
        * @param {integer} mask One of the 256 shuffle masks, for example, SIMD.XXXX.
        * @return {float32x4} New instance of float32x4 with lanes shuffled.
        */
      shuffleMix: function(t1, t2, mask) {
        var _x = (mask) & 0x3;
        var _y = (mask >> 2) & 0x3;
        var _z = (mask >> 4) & 0x3;
        var _w = (mask >> 6) & 0x3;
        return new float32x4(t1.storage_[_x], t1.storage_[_y], t2.storage_[_z],
                             t2.storage_[_w]);
      },
      /**
        * @param {double} value used for x lane.
        * @return {float32x4} New instance of float32x4 with the values in t and
        * x replaced with {x}.
        */
      withX: function(t, x) {
        return new float32x4(x, t.y, t.z, t.w);
      },
      /**
        * @param {double} value used for y lane.
        * @return {float32x4} New instance of float32x4 with the values in t and
        * y replaced with {y}.
        */
      withY: function(t, y) {
        return new float32x4(t.x, y, t.z, t.w);
      },
      /**
        * @param {double} value used for z lane.
        * @return {float32x4} New instance of float32x4 with the values in t and
        * z replaced with {z}.
        */
      withZ: function(t, z) {
        return new float32x4(t.x, t.y, z, t.w);
      },
      /**
        * @param {double} value used for w lane.
        * @return {float32x4} New instance of float32x4 with the values in t and
        * w replaced with {w}.
        */
      withW: function(t, w) {
        return new float32x4(t.x, t.y, t.z, w);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t < other.
        */
      lessThan: function(t, other) {
        var cx = t.x < other.x;
        var cy = t.y < other.y;
        var cz = t.z < other.z;
        var cw = t.w < other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t <= other.
        */
      lessThanOrEqual: function(t, other) {
        var cx = t.x <= other.x;
        var cy = t.y <= other.y;
        var cz = t.z <= other.z;
        var cw = t.w <= other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t == other.
        */
      equal: function(t, other) {
        var cx = t.x == other.x;
        var cy = t.y == other.y;
        var cz = t.z == other.z;
        var cw = t.w == other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t != other.
        */
      notEqual: function(t, other) {
        var cx = t.x != other.x;
        var cy = t.y != other.y;
        var cz = t.z != other.z;
        var cw = t.w != other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t >= other.
        */
      greaterThanOrEqual: function(t, other) {
        var cx = t.x >= other.x;
        var cy = t.y >= other.y;
        var cz = t.z >= other.z;
        var cw = t.w >= other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @param {float32x4} other An instance of float32x4.
        * @return {int32x4} 0xFFFFFFFF or 0x0 in each lane depending on
        * the result of t > other.
        */
      greaterThan: function(t, other) {
        var cx = t.x > other.x;
        var cy = t.y > other.y;
        var cz = t.z > other.z;
        var cw = t.w > other.w;
        return int32x4.bool(cx, cy, cz, cw);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @return {int32x4} a bit-wise copy of t as a int32x4.
        */
      bitsToInt32x4: function(t) {
        var alias = new Int32Array(t.storage_.buffer);
        return new int32x4(alias[0], alias[1], alias[2], alias[3]);
      },
      /**
        * @param {float32x4} t An instance of float32x4.
        * @return {int32x4} with a integer to float conversion of t.
        */
      toInt32x4: function(t) {
        var a = new int32x4(t.storage_[0], t.storage_[1], t.storage_[2],
                             t.storage_[3]);
        return a;
      }
    },
    int32x4: {
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a & b.
        */
      and: function(a, b) {
        return new int32x4(a.x & b.x, a.y & b.y, a.z & b.z, a.w & b.w);
      },
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a | b.
        */
      or: function(a, b) {
        return new int32x4(a.x | b.x, a.y | b.y, a.z | b.z, a.w | b.w);
      },
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a ^ b.
        */
      xor: function(a, b) {
        return new int32x4(a.x ^ b.x, a.y ^ b.y, a.z ^ b.z, a.w ^ b.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of ~t
        */
      not: function(t) {
        return new int32x4(~t.x, ~t.y, ~t.z, ~t.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of -t
        */
      neg: function(t) {
        return new int32x4(-t.x, -t.y, -t.z, -t.w);
      },
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a + b.
        */
      add: function(a, b) {
        return new int32x4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
      },
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a - b.
        */
      sub: function(a, b) {
        return new int32x4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
      },
      /**
        * @param {int32x4} a An instance of int32x4.
        * @param {int32x4} b An instance of int32x4.
        * @return {int32x4} New instance of int32x4 with values of a * b.
        */
      mul: function(a, b) {
        return new int32x4(Math.imul(a.x, b.x), Math.imul(a.y, b.y),
                           Math.imul(a.z, b.z), Math.imul(a.w, b.w));
      },
      /**
        * @param {int32x4} t An instance of float32x4 to be shuffled.
        * @param {integer} mask One of the 256 shuffle masks, for example, SIMD.XXXX.
        * @return {int32x4} New instance of float32x4 with lanes shuffled.
        */
      shuffle: function(t, mask) {
        var _x = (mask) & 0x3;
        var _y = (mask >> 2) & 0x3;
        var _z = (mask >> 4) & 0x3;
        var _w = (mask >> 6) & 0x3;
        return new int32x4(t.storage_[_x], t.storage_[_y], t.storage_[_z],
                             t.storage_[_w]);
      },
      /**
        * @param {int32x4} t1 An instance of float32x4 to be shuffled. XY lanes in result
        * @param {int32x4} t2 An instance of float32x4 to be shuffled. ZW lanes in result
        * @param {integer} mask One of the 256 shuffle masks, for example, SIMD.XXXX.
        * @return {int32x4} New instance of float32x4 with lanes shuffled.
        */
      shuffleMix: function(t1, t2, mask) {
        var _x = (mask) & 0x3;
        var _y = (mask >> 2) & 0x3;
        var _z = (mask >> 4) & 0x3;
        var _w = (mask >> 6) & 0x3;
        return new int32x4(t1.storage_[_x], t1.storage_[_y], t2.storage_[_z],
                             t2.storage_[_w]);
      },
      /**
        * @param {float32x4}
        */
      select: function(t, trueValue, falseValue) {
        var tv = SIMD.float32x4.bitsToInt32x4(trueValue);
        var fv = SIMD.float32x4.bitsToInt32x4(falseValue);
        var tr = SIMD.int32x4.and(t, tv);
        var fr = SIMD.int32x4.and(SIMD.int32x4.not(t), fv);
        return SIMD.int32x4.bitsToFloat32x4(SIMD.int32x4.or(tr, fr));
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {integer} 32-bit value used for x lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * x lane replaced with {x}.
        */
      withX: function(t, x) {
        return new int32x4(x, t.y, t.z, t.w);
      },
      /**
        * param {int32x4} t An instance of int32x4.
        * @param {integer} 32-bit value used for y lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * y lane replaced with {y}.
        */
      withY: function(t, y) {
        return new int32x4(t.x, y, t.z, t.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {integer} 32-bit value used for z lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * z lane replaced with {z}.
        */
      withZ: function(t, z) {
        return new int32x4(t.x, t.y, z, t.w);
      },
      /**
        * @param {integer} 32-bit value used for w lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * w lane replaced with {w}.
        */
      withW: function(t, w) {
        return new int32x4(t.x, t.y, t.z, w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {boolean} x flag used for x lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * x lane replaced with {x}.
        */
      withFlagX: function(t, flagX) {
        var x = flagX ? 0xFFFFFFFF : 0x0;
        return new int32x4(x, t.y, t.z, t.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {boolean} y flag used for y lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * y lane replaced with {y}.
        */
      withFlagY: function(t, flagY) {
        var y = flagY ? 0xFFFFFFFF : 0x0;
        return new int32x4(t.x, y, t.z, t.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {boolean} z flag used for z lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * z lane replaced with {z}.
        */
      withFlagZ: function(t, flagZ) {
        var z = flagZ ? 0xFFFFFFFF : 0x0;
        return new int32x4(t.x, t.y, z, t.w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @param {boolean} w flag used for w lane.
        * @return {int32x4} New instance of int32x4 with the values in t and
        * w lane replaced with {w}.
        */
      withFlagW: function(t, flagW) {
        var w = flagW ? 0xFFFFFFFF : 0x0;
        return new int32x4(t.x, t.y, t.z, w);
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @return {float32x4} a bit-wise copy of t as a float32x4.
        */
      bitsToFloat32x4: function(t) {
        var temp_storage = new Int32Array([t.storage_[0], t.storage_[1], t.storage_[2], t.storage_[3]]);
        var alias = new Float32Array(temp_storage.buffer);
        var fx4 = float32x4.zero();
        fx4.storage_ = alias;
        return fx4;      
      },
      /**
        * @param {int32x4} t An instance of int32x4.
        * @return {float32x4} with a float to integer conversion copy of t.
        */
      toFloat32x4: function(t) {
        var a = float32x4.zero();
        a.storage_[0] = t.storage_[0];
        a.storage_[1] = t.storage_[1];
        a.storage_[2] = t.storage_[2];
        a.storage_[3] = t.storage_[3];
        return a;
      }
    }
  }
})();

Object.defineProperty(SIMD, 'XXXX', { get: function() { return 0x0; } });
Object.defineProperty(SIMD, 'XXXY', { get: function() { return 0x40; } });
Object.defineProperty(SIMD, 'XXXZ', { get: function() { return 0x80; } });
Object.defineProperty(SIMD, 'XXXW', { get: function() { return 0xC0; } });
Object.defineProperty(SIMD, 'XXYX', { get: function() { return 0x10; } });
Object.defineProperty(SIMD, 'XXYY', { get: function() { return 0x50; } });
Object.defineProperty(SIMD, 'XXYZ', { get: function() { return 0x90; } });
Object.defineProperty(SIMD, 'XXYW', { get: function() { return 0xD0; } });
Object.defineProperty(SIMD, 'XXZX', { get: function() { return 0x20; } });
Object.defineProperty(SIMD, 'XXZY', { get: function() { return 0x60; } });
Object.defineProperty(SIMD, 'XXZZ', { get: function() { return 0xA0; } });
Object.defineProperty(SIMD, 'XXZW', { get: function() { return 0xE0; } });
Object.defineProperty(SIMD, 'XXWX', { get: function() { return 0x30; } });
Object.defineProperty(SIMD, 'XXWY', { get: function() { return 0x70; } });
Object.defineProperty(SIMD, 'XXWZ', { get: function() { return 0xB0; } });
Object.defineProperty(SIMD, 'XXWW', { get: function() { return 0xF0; } });
Object.defineProperty(SIMD, 'XYXX', { get: function() { return 0x4; } });
Object.defineProperty(SIMD, 'XYXY', { get: function() { return 0x44; } });
Object.defineProperty(SIMD, 'XYXZ', { get: function() { return 0x84; } });
Object.defineProperty(SIMD, 'XYXW', { get: function() { return 0xC4; } });
Object.defineProperty(SIMD, 'XYYX', { get: function() { return 0x14; } });
Object.defineProperty(SIMD, 'XYYY', { get: function() { return 0x54; } });
Object.defineProperty(SIMD, 'XYYZ', { get: function() { return 0x94; } });
Object.defineProperty(SIMD, 'XYYW', { get: function() { return 0xD4; } });
Object.defineProperty(SIMD, 'XYZX', { get: function() { return 0x24; } });
Object.defineProperty(SIMD, 'XYZY', { get: function() { return 0x64; } });
Object.defineProperty(SIMD, 'XYZZ', { get: function() { return 0xA4; } });
Object.defineProperty(SIMD, 'XYZW', { get: function() { return 0xE4; } });
Object.defineProperty(SIMD, 'XYWX', { get: function() { return 0x34; } });
Object.defineProperty(SIMD, 'XYWY', { get: function() { return 0x74; } });
Object.defineProperty(SIMD, 'XYWZ', { get: function() { return 0xB4; } });
Object.defineProperty(SIMD, 'XYWW', { get: function() { return 0xF4; } });
Object.defineProperty(SIMD, 'XZXX', { get: function() { return 0x8; } });
Object.defineProperty(SIMD, 'XZXY', { get: function() { return 0x48; } });
Object.defineProperty(SIMD, 'XZXZ', { get: function() { return 0x88; } });
Object.defineProperty(SIMD, 'XZXW', { get: function() { return 0xC8; } });
Object.defineProperty(SIMD, 'XZYX', { get: function() { return 0x18; } });
Object.defineProperty(SIMD, 'XZYY', { get: function() { return 0x58; } });
Object.defineProperty(SIMD, 'XZYZ', { get: function() { return 0x98; } });
Object.defineProperty(SIMD, 'XZYW', { get: function() { return 0xD8; } });
Object.defineProperty(SIMD, 'XZZX', { get: function() { return 0x28; } });
Object.defineProperty(SIMD, 'XZZY', { get: function() { return 0x68; } });
Object.defineProperty(SIMD, 'XZZZ', { get: function() { return 0xA8; } });
Object.defineProperty(SIMD, 'XZZW', { get: function() { return 0xE8; } });
Object.defineProperty(SIMD, 'XZWX', { get: function() { return 0x38; } });
Object.defineProperty(SIMD, 'XZWY', { get: function() { return 0x78; } });
Object.defineProperty(SIMD, 'XZWZ', { get: function() { return 0xB8; } });
Object.defineProperty(SIMD, 'XZWW', { get: function() { return 0xF8; } });
Object.defineProperty(SIMD, 'XWXX', { get: function() { return 0xC; } });
Object.defineProperty(SIMD, 'XWXY', { get: function() { return 0x4C; } });
Object.defineProperty(SIMD, 'XWXZ', { get: function() { return 0x8C; } });
Object.defineProperty(SIMD, 'XWXW', { get: function() { return 0xCC; } });
Object.defineProperty(SIMD, 'XWYX', { get: function() { return 0x1C; } });
Object.defineProperty(SIMD, 'XWYY', { get: function() { return 0x5C; } });
Object.defineProperty(SIMD, 'XWYZ', { get: function() { return 0x9C; } });
Object.defineProperty(SIMD, 'XWYW', { get: function() { return 0xDC; } });
Object.defineProperty(SIMD, 'XWZX', { get: function() { return 0x2C; } });
Object.defineProperty(SIMD, 'XWZY', { get: function() { return 0x6C; } });
Object.defineProperty(SIMD, 'XWZZ', { get: function() { return 0xAC; } });
Object.defineProperty(SIMD, 'XWZW', { get: function() { return 0xEC; } });
Object.defineProperty(SIMD, 'XWWX', { get: function() { return 0x3C; } });
Object.defineProperty(SIMD, 'XWWY', { get: function() { return 0x7C; } });
Object.defineProperty(SIMD, 'XWWZ', { get: function() { return 0xBC; } });
Object.defineProperty(SIMD, 'XWWW', { get: function() { return 0xFC; } });
Object.defineProperty(SIMD, 'YXXX', { get: function() { return 0x1; } });
Object.defineProperty(SIMD, 'YXXY', { get: function() { return 0x41; } });
Object.defineProperty(SIMD, 'YXXZ', { get: function() { return 0x81; } });
Object.defineProperty(SIMD, 'YXXW', { get: function() { return 0xC1; } });
Object.defineProperty(SIMD, 'YXYX', { get: function() { return 0x11; } });
Object.defineProperty(SIMD, 'YXYY', { get: function() { return 0x51; } });
Object.defineProperty(SIMD, 'YXYZ', { get: function() { return 0x91; } });
Object.defineProperty(SIMD, 'YXYW', { get: function() { return 0xD1; } });
Object.defineProperty(SIMD, 'YXZX', { get: function() { return 0x21; } });
Object.defineProperty(SIMD, 'YXZY', { get: function() { return 0x61; } });
Object.defineProperty(SIMD, 'YXZZ', { get: function() { return 0xA1; } });
Object.defineProperty(SIMD, 'YXZW', { get: function() { return 0xE1; } });
Object.defineProperty(SIMD, 'YXWX', { get: function() { return 0x31; } });
Object.defineProperty(SIMD, 'YXWY', { get: function() { return 0x71; } });
Object.defineProperty(SIMD, 'YXWZ', { get: function() { return 0xB1; } });
Object.defineProperty(SIMD, 'YXWW', { get: function() { return 0xF1; } });
Object.defineProperty(SIMD, 'YYXX', { get: function() { return 0x5; } });
Object.defineProperty(SIMD, 'YYXY', { get: function() { return 0x45; } });
Object.defineProperty(SIMD, 'YYXZ', { get: function() { return 0x85; } });
Object.defineProperty(SIMD, 'YYXW', { get: function() { return 0xC5; } });
Object.defineProperty(SIMD, 'YYYX', { get: function() { return 0x15; } });
Object.defineProperty(SIMD, 'YYYY', { get: function() { return 0x55; } });
Object.defineProperty(SIMD, 'YYYZ', { get: function() { return 0x95; } });
Object.defineProperty(SIMD, 'YYYW', { get: function() { return 0xD5; } });
Object.defineProperty(SIMD, 'YYZX', { get: function() { return 0x25; } });
Object.defineProperty(SIMD, 'YYZY', { get: function() { return 0x65; } });
Object.defineProperty(SIMD, 'YYZZ', { get: function() { return 0xA5; } });
Object.defineProperty(SIMD, 'YYZW', { get: function() { return 0xE5; } });
Object.defineProperty(SIMD, 'YYWX', { get: function() { return 0x35; } });
Object.defineProperty(SIMD, 'YYWY', { get: function() { return 0x75; } });
Object.defineProperty(SIMD, 'YYWZ', { get: function() { return 0xB5; } });
Object.defineProperty(SIMD, 'YYWW', { get: function() { return 0xF5; } });
Object.defineProperty(SIMD, 'YZXX', { get: function() { return 0x9; } });
Object.defineProperty(SIMD, 'YZXY', { get: function() { return 0x49; } });
Object.defineProperty(SIMD, 'YZXZ', { get: function() { return 0x89; } });
Object.defineProperty(SIMD, 'YZXW', { get: function() { return 0xC9; } });
Object.defineProperty(SIMD, 'YZYX', { get: function() { return 0x19; } });
Object.defineProperty(SIMD, 'YZYY', { get: function() { return 0x59; } });
Object.defineProperty(SIMD, 'YZYZ', { get: function() { return 0x99; } });
Object.defineProperty(SIMD, 'YZYW', { get: function() { return 0xD9; } });
Object.defineProperty(SIMD, 'YZZX', { get: function() { return 0x29; } });
Object.defineProperty(SIMD, 'YZZY', { get: function() { return 0x69; } });
Object.defineProperty(SIMD, 'YZZZ', { get: function() { return 0xA9; } });
Object.defineProperty(SIMD, 'YZZW', { get: function() { return 0xE9; } });
Object.defineProperty(SIMD, 'YZWX', { get: function() { return 0x39; } });
Object.defineProperty(SIMD, 'YZWY', { get: function() { return 0x79; } });
Object.defineProperty(SIMD, 'YZWZ', { get: function() { return 0xB9; } });
Object.defineProperty(SIMD, 'YZWW', { get: function() { return 0xF9; } });
Object.defineProperty(SIMD, 'YWXX', { get: function() { return 0xD; } });
Object.defineProperty(SIMD, 'YWXY', { get: function() { return 0x4D; } });
Object.defineProperty(SIMD, 'YWXZ', { get: function() { return 0x8D; } });
Object.defineProperty(SIMD, 'YWXW', { get: function() { return 0xCD; } });
Object.defineProperty(SIMD, 'YWYX', { get: function() { return 0x1D; } });
Object.defineProperty(SIMD, 'YWYY', { get: function() { return 0x5D; } });
Object.defineProperty(SIMD, 'YWYZ', { get: function() { return 0x9D; } });
Object.defineProperty(SIMD, 'YWYW', { get: function() { return 0xDD; } });
Object.defineProperty(SIMD, 'YWZX', { get: function() { return 0x2D; } });
Object.defineProperty(SIMD, 'YWZY', { get: function() { return 0x6D; } });
Object.defineProperty(SIMD, 'YWZZ', { get: function() { return 0xAD; } });
Object.defineProperty(SIMD, 'YWZW', { get: function() { return 0xED; } });
Object.defineProperty(SIMD, 'YWWX', { get: function() { return 0x3D; } });
Object.defineProperty(SIMD, 'YWWY', { get: function() { return 0x7D; } });
Object.defineProperty(SIMD, 'YWWZ', { get: function() { return 0xBD; } });
Object.defineProperty(SIMD, 'YWWW', { get: function() { return 0xFD; } });
Object.defineProperty(SIMD, 'ZXXX', { get: function() { return 0x2; } });
Object.defineProperty(SIMD, 'ZXXY', { get: function() { return 0x42; } });
Object.defineProperty(SIMD, 'ZXXZ', { get: function() { return 0x82; } });
Object.defineProperty(SIMD, 'ZXXW', { get: function() { return 0xC2; } });
Object.defineProperty(SIMD, 'ZXYX', { get: function() { return 0x12; } });
Object.defineProperty(SIMD, 'ZXYY', { get: function() { return 0x52; } });
Object.defineProperty(SIMD, 'ZXYZ', { get: function() { return 0x92; } });
Object.defineProperty(SIMD, 'ZXYW', { get: function() { return 0xD2; } });
Object.defineProperty(SIMD, 'ZXZX', { get: function() { return 0x22; } });
Object.defineProperty(SIMD, 'ZXZY', { get: function() { return 0x62; } });
Object.defineProperty(SIMD, 'ZXZZ', { get: function() { return 0xA2; } });
Object.defineProperty(SIMD, 'ZXZW', { get: function() { return 0xE2; } });
Object.defineProperty(SIMD, 'ZXWX', { get: function() { return 0x32; } });
Object.defineProperty(SIMD, 'ZXWY', { get: function() { return 0x72; } });
Object.defineProperty(SIMD, 'ZXWZ', { get: function() { return 0xB2; } });
Object.defineProperty(SIMD, 'ZXWW', { get: function() { return 0xF2; } });
Object.defineProperty(SIMD, 'ZYXX', { get: function() { return 0x6; } });
Object.defineProperty(SIMD, 'ZYXY', { get: function() { return 0x46; } });
Object.defineProperty(SIMD, 'ZYXZ', { get: function() { return 0x86; } });
Object.defineProperty(SIMD, 'ZYXW', { get: function() { return 0xC6; } });
Object.defineProperty(SIMD, 'ZYYX', { get: function() { return 0x16; } });
Object.defineProperty(SIMD, 'ZYYY', { get: function() { return 0x56; } });
Object.defineProperty(SIMD, 'ZYYZ', { get: function() { return 0x96; } });
Object.defineProperty(SIMD, 'ZYYW', { get: function() { return 0xD6; } });
Object.defineProperty(SIMD, 'ZYZX', { get: function() { return 0x26; } });
Object.defineProperty(SIMD, 'ZYZY', { get: function() { return 0x66; } });
Object.defineProperty(SIMD, 'ZYZZ', { get: function() { return 0xA6; } });
Object.defineProperty(SIMD, 'ZYZW', { get: function() { return 0xE6; } });
Object.defineProperty(SIMD, 'ZYWX', { get: function() { return 0x36; } });
Object.defineProperty(SIMD, 'ZYWY', { get: function() { return 0x76; } });
Object.defineProperty(SIMD, 'ZYWZ', { get: function() { return 0xB6; } });
Object.defineProperty(SIMD, 'ZYWW', { get: function() { return 0xF6; } });
Object.defineProperty(SIMD, 'ZZXX', { get: function() { return 0xA; } });
Object.defineProperty(SIMD, 'ZZXY', { get: function() { return 0x4A; } });
Object.defineProperty(SIMD, 'ZZXZ', { get: function() { return 0x8A; } });
Object.defineProperty(SIMD, 'ZZXW', { get: function() { return 0xCA; } });
Object.defineProperty(SIMD, 'ZZYX', { get: function() { return 0x1A; } });
Object.defineProperty(SIMD, 'ZZYY', { get: function() { return 0x5A; } });
Object.defineProperty(SIMD, 'ZZYZ', { get: function() { return 0x9A; } });
Object.defineProperty(SIMD, 'ZZYW', { get: function() { return 0xDA; } });
Object.defineProperty(SIMD, 'ZZZX', { get: function() { return 0x2A; } });
Object.defineProperty(SIMD, 'ZZZY', { get: function() { return 0x6A; } });
Object.defineProperty(SIMD, 'ZZZZ', { get: function() { return 0xAA; } });
Object.defineProperty(SIMD, 'ZZZW', { get: function() { return 0xEA; } });
Object.defineProperty(SIMD, 'ZZWX', { get: function() { return 0x3A; } });
Object.defineProperty(SIMD, 'ZZWY', { get: function() { return 0x7A; } });
Object.defineProperty(SIMD, 'ZZWZ', { get: function() { return 0xBA; } });
Object.defineProperty(SIMD, 'ZZWW', { get: function() { return 0xFA; } });
Object.defineProperty(SIMD, 'ZWXX', { get: function() { return 0xE; } });
Object.defineProperty(SIMD, 'ZWXY', { get: function() { return 0x4E; } });
Object.defineProperty(SIMD, 'ZWXZ', { get: function() { return 0x8E; } });
Object.defineProperty(SIMD, 'ZWXW', { get: function() { return 0xCE; } });
Object.defineProperty(SIMD, 'ZWYX', { get: function() { return 0x1E; } });
Object.defineProperty(SIMD, 'ZWYY', { get: function() { return 0x5E; } });
Object.defineProperty(SIMD, 'ZWYZ', { get: function() { return 0x9E; } });
Object.defineProperty(SIMD, 'ZWYW', { get: function() { return 0xDE; } });
Object.defineProperty(SIMD, 'ZWZX', { get: function() { return 0x2E; } });
Object.defineProperty(SIMD, 'ZWZY', { get: function() { return 0x6E; } });
Object.defineProperty(SIMD, 'ZWZZ', { get: function() { return 0xAE; } });
Object.defineProperty(SIMD, 'ZWZW', { get: function() { return 0xEE; } });
Object.defineProperty(SIMD, 'ZWWX', { get: function() { return 0x3E; } });
Object.defineProperty(SIMD, 'ZWWY', { get: function() { return 0x7E; } });
Object.defineProperty(SIMD, 'ZWWZ', { get: function() { return 0xBE; } });
Object.defineProperty(SIMD, 'ZWWW', { get: function() { return 0xFE; } });
Object.defineProperty(SIMD, 'WXXX', { get: function() { return 0x3; } });
Object.defineProperty(SIMD, 'WXXY', { get: function() { return 0x43; } });
Object.defineProperty(SIMD, 'WXXZ', { get: function() { return 0x83; } });
Object.defineProperty(SIMD, 'WXXW', { get: function() { return 0xC3; } });
Object.defineProperty(SIMD, 'WXYX', { get: function() { return 0x13; } });
Object.defineProperty(SIMD, 'WXYY', { get: function() { return 0x53; } });
Object.defineProperty(SIMD, 'WXYZ', { get: function() { return 0x93; } });
Object.defineProperty(SIMD, 'WXYW', { get: function() { return 0xD3; } });
Object.defineProperty(SIMD, 'WXZX', { get: function() { return 0x23; } });
Object.defineProperty(SIMD, 'WXZY', { get: function() { return 0x63; } });
Object.defineProperty(SIMD, 'WXZZ', { get: function() { return 0xA3; } });
Object.defineProperty(SIMD, 'WXZW', { get: function() { return 0xE3; } });
Object.defineProperty(SIMD, 'WXWX', { get: function() { return 0x33; } });
Object.defineProperty(SIMD, 'WXWY', { get: function() { return 0x73; } });
Object.defineProperty(SIMD, 'WXWZ', { get: function() { return 0xB3; } });
Object.defineProperty(SIMD, 'WXWW', { get: function() { return 0xF3; } });
Object.defineProperty(SIMD, 'WYXX', { get: function() { return 0x7; } });
Object.defineProperty(SIMD, 'WYXY', { get: function() { return 0x47; } });
Object.defineProperty(SIMD, 'WYXZ', { get: function() { return 0x87; } });
Object.defineProperty(SIMD, 'WYXW', { get: function() { return 0xC7; } });
Object.defineProperty(SIMD, 'WYYX', { get: function() { return 0x17; } });
Object.defineProperty(SIMD, 'WYYY', { get: function() { return 0x57; } });
Object.defineProperty(SIMD, 'WYYZ', { get: function() { return 0x97; } });
Object.defineProperty(SIMD, 'WYYW', { get: function() { return 0xD7; } });
Object.defineProperty(SIMD, 'WYZX', { get: function() { return 0x27; } });
Object.defineProperty(SIMD, 'WYZY', { get: function() { return 0x67; } });
Object.defineProperty(SIMD, 'WYZZ', { get: function() { return 0xA7; } });
Object.defineProperty(SIMD, 'WYZW', { get: function() { return 0xE7; } });
Object.defineProperty(SIMD, 'WYWX', { get: function() { return 0x37; } });
Object.defineProperty(SIMD, 'WYWY', { get: function() { return 0x77; } });
Object.defineProperty(SIMD, 'WYWZ', { get: function() { return 0xB7; } });
Object.defineProperty(SIMD, 'WYWW', { get: function() { return 0xF7; } });
Object.defineProperty(SIMD, 'WZXX', { get: function() { return 0xB; } });
Object.defineProperty(SIMD, 'WZXY', { get: function() { return 0x4B; } });
Object.defineProperty(SIMD, 'WZXZ', { get: function() { return 0x8B; } });
Object.defineProperty(SIMD, 'WZXW', { get: function() { return 0xCB; } });
Object.defineProperty(SIMD, 'WZYX', { get: function() { return 0x1B; } });
Object.defineProperty(SIMD, 'WZYY', { get: function() { return 0x5B; } });
Object.defineProperty(SIMD, 'WZYZ', { get: function() { return 0x9B; } });
Object.defineProperty(SIMD, 'WZYW', { get: function() { return 0xDB; } });
Object.defineProperty(SIMD, 'WZZX', { get: function() { return 0x2B; } });
Object.defineProperty(SIMD, 'WZZY', { get: function() { return 0x6B; } });
Object.defineProperty(SIMD, 'WZZZ', { get: function() { return 0xAB; } });
Object.defineProperty(SIMD, 'WZZW', { get: function() { return 0xEB; } });
Object.defineProperty(SIMD, 'WZWX', { get: function() { return 0x3B; } });
Object.defineProperty(SIMD, 'WZWY', { get: function() { return 0x7B; } });
Object.defineProperty(SIMD, 'WZWZ', { get: function() { return 0xBB; } });
Object.defineProperty(SIMD, 'WZWW', { get: function() { return 0xFB; } });
Object.defineProperty(SIMD, 'WWXX', { get: function() { return 0xF; } });
Object.defineProperty(SIMD, 'WWXY', { get: function() { return 0x4F; } });
Object.defineProperty(SIMD, 'WWXZ', { get: function() { return 0x8F; } });
Object.defineProperty(SIMD, 'WWXW', { get: function() { return 0xCF; } });
Object.defineProperty(SIMD, 'WWYX', { get: function() { return 0x1F; } });
Object.defineProperty(SIMD, 'WWYY', { get: function() { return 0x5F; } });
Object.defineProperty(SIMD, 'WWYZ', { get: function() { return 0x9F; } });
Object.defineProperty(SIMD, 'WWYW', { get: function() { return 0xDF; } });
Object.defineProperty(SIMD, 'WWZX', { get: function() { return 0x2F; } });
Object.defineProperty(SIMD, 'WWZY', { get: function() { return 0x6F; } });
Object.defineProperty(SIMD, 'WWZZ', { get: function() { return 0xAF; } });
Object.defineProperty(SIMD, 'WWZW', { get: function() { return 0xEF; } });
Object.defineProperty(SIMD, 'WWWX', { get: function() { return 0x3F; } });
Object.defineProperty(SIMD, 'WWWY', { get: function() { return 0x7F; } });
Object.defineProperty(SIMD, 'WWWZ', { get: function() { return 0xBF; } });
Object.defineProperty(SIMD, 'WWWW', { get: function() { return 0xFF; } });


// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer) {
  function applyData(data) {
    HEAPU8.set(data, STATIC_BASE);
  }
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    applyData(Module['readBinary'](memoryInitializer));
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      applyData(data);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
    Module.printErr('preload time: ' + (Date.now() - preloadStartTime) + ' ms');
  }

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString("/bin/this.program"), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    if (!Module['noExitRuntime']) {
      exit(ret);
    }
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    ensureInitRuntime();

    preMain();

    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      if (!ABORT) doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  // TODO We should handle this differently based on environment.
  // In the browser, the best we can do is throw an exception
  // to halt execution, but in node we could process.exit and
  // I'd imagine SM shell would have something equivalent.
  // This would let us set a proper exit status (which
  // would be great for checking test exit statuses).
  // https://github.com/kripken/emscripten/issues/1371

  // throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }

  ABORT = true;
  EXITSTATUS = 1;

  throw 'abort() at ' + stackTrace();
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}

run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}






